package nexus.billing

import android.util.Log
import com.jakewharton.rxrelay3.PublishRelay
import nexus.billing.migrations.BillingMigration
import nexus.billing.models.PlanObject
import nexus.core.NCPCore
import nexus.core.controllers.NetworkController
import nexus.core.kit.Account
import nexus.core.kit.Kit
import nexus.core.kit.KitBuilder
import nexus.core.util.NotifyObject

val NCPCore.billingKit: BillingKit get() { return storage.get<BillingKit>("BillingKit") ?: throw Exception("BillingKit not inited") }

class BillingKit(core: NCPCore): Kit(core) {

    val planChangeEvent: PublishRelay<NotifyObject<PlanObject>> = PublishRelay.create()

    companion object {
        val builder = object : KitBuilder {
            override fun build(core: NCPCore): Kit {
                return BillingKit(core)
            }
        }
        @JvmStatic val billingKit: BillingKit get() { return NCPCore.instance.billingKit }
    }

    @JvmField val controller: BillingController

    init {
        Log.d("BillingKit", "init: ")
        core.storage.set("BillingKit", this)
        this.controller = BillingController(core)
    }

    override fun switchAccount(account: Account?) {
        NCPCore.instance.database.applyMigration(BillingMigration())
        this.controller.switchAccount(account)
    }

    override fun socketChange(state: NetworkController.ConnectionState) {
        this.controller.socketChange(state)
    }

    override fun reSync() {
        this.controller.reSync()
    }
}