package nexus.billing.methods.profile

import nexus.billing.methods.common.Verification
import nexus.billing.methods.service.Service
import nexus.core.methods.NetworkObject
import nexus.core.util.Currency

open class ProfilePlan : NetworkObject() {
    lateinit var id: String
    lateinit var name: String
    lateinit var planID: String
    lateinit var state: State
    var services: Array<ProfileService> = emptyArray()
    var actions: Array<Action> = emptyArray()
    var recurringInterval: Double? = null
    var selectedAt: Double? = null
    var activatedAt: Double? = null
    var suspendedAt: Double? = null
    var expirationDate: Double? = null
    var nextActivationAt: Double? = null

    enum class State {
        pending,
        suspended,
        active,
        inactive,
        removed
    }

    data class Action(
        val asset: Asset?,
        val identity: Identity?,
        val support: Support?,
        val balance: Balance?
    ) {
        enum class State {
            pending,
            completed
        }

        data class Asset(var info: Info) {
            data class Info(
                var id: String,
                var state: State,
                var selectedService: Service?,
                var selectedAsset: nexus.billing.methods.assets.Asset?
            )
        }

        data class Identity(var info: Info) {
            data class Info(
                var id: String,
                var state: State,
                var required: Array<Verification>,
                var applicationID: String?,
                var applicationLevel: Int?
            )
        }
        data class Support(var info: Info) {
            data class Info(
                var id: String,
                var state: State,
            )
        }
        data class Balance(var info: Info) {
            data class Info(
                var id: String,
                var state: State,
                var profilePaymentSource: String?,
                var required: Int,
                var currency: Currency,
            )
        }
    }
}
