package nexus.billing.methods.requests.profileplan

import nexus.billing.methods.plan.Plan
import nexus.billing.methods.profile.ProfilePlan
import nexus.core.methods.EndpointError
import nexus.core.methods.EndpointRequestType
import nexus.core.methods.NetworkObject
import nexus.core.methods.Serializable
import nexus.core.util.Currency
import java.util.UUID

@Serializable
data class ProfilePlanRoute(override var params: Params) : EndpointRequestType<ProfilePlanRoute.Params, ProfilePlanRoute.Response> {
    override var method = "billing.profile.plan.route"
    override var id = UUID.randomUUID().toString().uppercase()
    override var version: Double = 1.0
    override lateinit var result: Response
    override lateinit var error: EndpointError

    enum class RouteType {
        voice,
        sms
    }

    @Serializable
    class Params(
        var type: RouteType,
        var destination: String,
        var profilePlanID: String?
    ) : NetworkObject()

    @Serializable
    class Response(
        var routes: Array<Routes>,
        var plans: Array<Plan>,
        var profilePlans: Array<ProfilePlan>? = null
    ) : NetworkObject() {
        class Routes(var route: Route? = null, var `package`: Package? = null): NetworkObject() {
            class Route(
                var rate: Int?,
                var currency: Currency?,
                var callerID: String,
                var profilePlanID: String,
                var profilePlanServiceID: String,
            ): NetworkObject()

            class Package(
                var total: Int,
                var available: Int,
                var callerID: String,
                var profilePlanID: String,
                var profilePlanServiceID: String,
            ): NetworkObject()

            fun profilePlanId(): String? {
                return this.route?.profilePlanID ?: this.`package`?.profilePlanID
            }

            fun profileServiceId(): String? {
                return this.route?.profilePlanServiceID ?: this.`package`?.profilePlanServiceID
            }
        }
    }
}