package nexus.billing.methods.requests.profileplan.external

import nexus.core.methods.EndpointError
import nexus.core.methods.EndpointRequestType
import nexus.core.methods.NetworkObject
import nexus.core.methods.Serializable
import java.util.UUID

@Serializable
data class ProfilePlanValidate(override var params: Params) : EndpointRequestType<ProfilePlanValidate.Params, ProfilePlanValidate.Response> {
    override var method = "billing.profile.plan.external.validate"
    override var id = UUID.randomUUID().toString().uppercase()
    override var version: Double = 1.0
    override lateinit var result: Response
    override lateinit var error: EndpointError

    @Serializable
    class Params(var providerID: String, var credentials: Credentials) : NetworkObject()
    class Credentials(var login: Login?, var phone: Phone?) : NetworkObject()
    class Login(var username: String, var password: String) : NetworkObject()
    class Phone(var number: String) : NetworkObject()
    @Serializable
    class Response(var status: ValidateStatus, var note: String?) : NetworkObject()

    enum class ValidateStatus {
        valid,
        incorrectPassword,
        passwordNotYetSet,
        invalidCode,
        invalidIdentifier
    }
}



