package nexus.billing.models

import android.annotation.SuppressLint
import android.content.ContentValues
import android.database.Cursor
import androidx.core.database.getDoubleOrNull
import androidx.core.database.getStringOrNull
import com.bloomberg.selekt.android.ConflictAlgorithm
import com.bloomberg.selekt.android.SQLiteDatabase
import nexus.core.db.DAO
import nexus.core.db.DBObject

class DescriptionObject(var id: String, var language: String) : DBObject {
    var planId: String? = null
    var serviceId: String? = null
    var providerId: String? = null
    var planCategoryId: String? = null
    var name: String? = null
    var title: String? = null
    var shortText: String? = null
    var longText: String? = null
    var backgroundImage: String? = null
    var logo: String? = null
    var icon: String? = null
    var updatedAt: Double = 0.0

    companion object : DAO<DescriptionObject, String> {
        override var table: String = "billing_description_table"

        const val ID = "id"
        const val LANGUAGE = "language"
        const val PLAN_ID = "planId"
        const val SERVICE_ID = "serviceId"
        const val PROVIDER_ID = "providerId"
        const val PLAN_CATEGORY_ID = "planCategoryId"
        const val NAME = "name"
        const val TITLE = "title"
        const val SHORT_TEXT = "shortText"
        const val LONG_TEXT = "longText"
        const val BACKGROUND_IMAGE = "backgroundImage"
        const val LOGO = "logo"
        const val ICON = "icon"
        const val UPDATED_AT = "updatedAt"

        override fun createTable(): String {
            return "CREATE TABLE IF NOT EXISTS $table (\"$ID\" TEXT PRIMARY KEY NOT NULL, " +
                    "\"$LANGUAGE\" TEXT, " +
                    "\"$PLAN_ID\" TEXT, " +
                    "\"$SERVICE_ID\" TEXT, " +
                    "\"$PROVIDER_ID\" TEXT, " +
                    "\"$PLAN_CATEGORY_ID\" TEXT, " +
                    "\"$NAME\" TEXT, " +
                    "\"$TITLE\" TEXT, " +
                    "\"$SHORT_TEXT\" TEXT, " +
                    "\"$LONG_TEXT\" TEXT, " +
                    "\"$BACKGROUND_IMAGE\" TEXT, " +
                    "\"$LOGO\" TEXT, " +
                    "\"$ICON\" TEXT, " +
                    "\"$UPDATED_AT\" REAL)"
        }

        override fun findAll(db: SQLiteDatabase): ArrayList<DescriptionObject> {
            val cursor = db.query("SELECT * FROM $table", null)
            val items = ArrayList<DescriptionObject>()
            while (cursor.moveToNext()) {
                items.add(this.unBox(db, cursor))
            }
            return items
        }

        fun findAll(db: SQLiteDatabase, ids: Array<String>): ArrayList<DescriptionObject> {
            val cursor = db.query("SELECT * FROM $table WHERE $ID in (${ids.joinToString(separator = ",") { id -> "'$id'" }})", null)
            val items = ArrayList<DescriptionObject>()
            while (cursor.moveToNext()) {
                items.add(this.unBox(db, cursor))
            }
            return items
        }

        override fun findBy(db: SQLiteDatabase, id: String): DescriptionObject? {
            val cursor: Cursor = db.query("SELECT * FROM $table WHERE $ID = '$id'", null)
            if (cursor.moveToFirst()) {
                return this.unBox(db, cursor)
            }
            return null
        }

        fun findByPlan(db: SQLiteDatabase, planId: String, language: String): DescriptionObject? {
            val cursor = db.query("SELECT * FROM $table WHERE $PLAN_ID = '$planId' AND $LANGUAGE = '$language'", null)
            if (cursor.moveToFirst()) {
                return this.unBox(db, cursor)
            }
            return null
        }

        fun findByCategory(db: SQLiteDatabase, planCategoryId: String, language: String): DescriptionObject? {
            val cursor = db.query("SELECT * FROM $table WHERE $PLAN_CATEGORY_ID = '$planCategoryId' AND $LANGUAGE = '$language'", null)
            if (cursor.moveToFirst()) {
                return this.unBox(db, cursor)
            }
            return null
        }

        fun findByServices(db: SQLiteDatabase, serviceIds: Array<String>, language: String): List<DescriptionObject> {
            val cursor = db.query("SELECT * FROM $table WHERE $SERVICE_ID in (${serviceIds.joinToString(separator = ",") { id -> "'$id'" }}) AND $LANGUAGE = '$language'", null)
            val items = ArrayList<DescriptionObject>()
            while (cursor.moveToNext()) {
                items.add(this.unBox(db, cursor))
            }
            return items
        }

        fun findByProviders(db: SQLiteDatabase, providerIds: Array<String>, language: String): List<DescriptionObject> {
            val cursor = db.query("SELECT * FROM $table WHERE $PROVIDER_ID in (${providerIds.joinToString(separator = ",") { id -> "'$id'" }}) AND $LANGUAGE = '$language'", null)
            val items = ArrayList<DescriptionObject>()
            while (cursor.moveToNext()) {
                items.add(this.unBox(db, cursor))
            }
            return items
        }

        @SuppressLint("Range")
        override fun unBox(db: SQLiteDatabase, cursor: Cursor): DescriptionObject {
            val lang = cursor.getStringOrNull(cursor.getColumnIndex(LANGUAGE)) ?: "en"
            val desc = DescriptionObject(cursor.getString(cursor.getColumnIndex(ID)), lang)
            desc.planId = cursor.getStringOrNull(cursor.getColumnIndex(PLAN_ID))
            desc.serviceId = cursor.getStringOrNull(cursor.getColumnIndex(SERVICE_ID))
            desc.providerId = cursor.getStringOrNull(cursor.getColumnIndex(PROVIDER_ID))
            desc.planCategoryId = cursor.getStringOrNull(cursor.getColumnIndex(PLAN_CATEGORY_ID))
            desc.name = cursor.getStringOrNull(cursor.getColumnIndex(NAME))
            desc.title = cursor.getStringOrNull(cursor.getColumnIndex(TITLE))
            desc.shortText = cursor.getStringOrNull(cursor.getColumnIndex(SHORT_TEXT))
            desc.longText = cursor.getStringOrNull(cursor.getColumnIndex(LONG_TEXT))
            desc.backgroundImage = cursor.getStringOrNull(cursor.getColumnIndex(BACKGROUND_IMAGE))
            desc.logo = cursor.getStringOrNull(cursor.getColumnIndex(LOGO))
            desc.icon = cursor.getStringOrNull(cursor.getColumnIndex(ICON))
            desc.updatedAt = cursor.getDoubleOrNull(cursor.getColumnIndex(UPDATED_AT)) ?: 0.0
            return desc
        }

        fun deleteAll(db: SQLiteDatabase): Int {
            return db.delete(table, null, null)
        }
    }

    override fun insert(db: SQLiteDatabase): Long {
        val values = ContentValues()
        values.apply {
            put(ID, this@DescriptionObject.id)
            put(LANGUAGE, this@DescriptionObject.language)
            put(PLAN_ID, this@DescriptionObject.planId)
            put(SERVICE_ID, this@DescriptionObject.serviceId)
            put(PROVIDER_ID, this@DescriptionObject.providerId)
            put(PLAN_CATEGORY_ID, this@DescriptionObject.planCategoryId)
            put(NAME, this@DescriptionObject.name)
            put(TITLE, this@DescriptionObject.title)
            put(SHORT_TEXT, this@DescriptionObject.shortText)
            put(LONG_TEXT, this@DescriptionObject.longText)
            put(BACKGROUND_IMAGE, this@DescriptionObject.backgroundImage)
            put(LOGO, this@DescriptionObject.logo)
            put(ICON, this@DescriptionObject.icon)
            put(UPDATED_AT, this@DescriptionObject.updatedAt)
        }
        return db.insert(table, values, ConflictAlgorithm.REPLACE)
    }

    override fun update(db: SQLiteDatabase): Int {
        val values = ContentValues()
        values.apply {
            put(LANGUAGE, this@DescriptionObject.language)
            put(PLAN_ID, this@DescriptionObject.planId)
            put(SERVICE_ID, this@DescriptionObject.serviceId)
            put(PROVIDER_ID, this@DescriptionObject.providerId)
            put(PLAN_CATEGORY_ID, this@DescriptionObject.planCategoryId)
            put(NAME, this@DescriptionObject.name)
            put(TITLE, this@DescriptionObject.title)
            put(SHORT_TEXT, this@DescriptionObject.shortText)
            put(LONG_TEXT, this@DescriptionObject.longText)
            put(BACKGROUND_IMAGE, this@DescriptionObject.backgroundImage)
            put(LOGO, this@DescriptionObject.logo)
            put(ICON, this@DescriptionObject.icon)
            put(UPDATED_AT, this@DescriptionObject.updatedAt)
        }
        return db.update(table, values, "$ID = '${this.id}'", null, ConflictAlgorithm.REPLACE)
    }

    override fun delete(db: SQLiteDatabase): Int {
        return db.delete(table, "$ID = '${this.id}'", null)
    }
}