package nexus.billing.models

import nexus.billing.methods.common.Provider
import nexus.billing.methods.common.Verification
import nexus.billing.methods.plan.Category
import nexus.billing.methods.plan.Plan
import nexus.billing.methods.service.Service

open class PlanInfo(
    var planID: String,
    var name: String,
    var provider: Provider,
    var category: Category,
    var verificationRequired: Array<Verification> = emptyArray(),
    var type: Plan.Types,
    var services: Array<Service>
) {
    @Transient
    var descriptions = ArrayList<DescriptionObject>()
    val description: DescriptionObject?
        get() = this.descriptions.find { it.planId == this.planID }
    val providerDesc: DescriptionObject?
        get() = this.descriptions.find { it.providerId == this.provider.id }
    val categoryDesc: DescriptionObject?
        get() = this.descriptions.find { it.planCategoryId == this.category.id }
    fun serviceDesc(serviceId: String): DescriptionObject? {
        return this.descriptions.find { it.serviceId == serviceId }
    }
    val planPrice: Int
        get() {
            var fullPrice = 0
            val dict = HashMap<String, Int>()
            for (service in this.services) {
                val price = (service.recurringAmount ?: service.purchaseAmount ?: 0)
                if (price > 0) {
                    when (service.activation) {
                        Service.Activation.mandatory -> fullPrice += price
                        Service.Activation.selective -> {
                            val key = "${service.type.name}_${service.category.name}"
                            if (dict[key] == null || dict[key]!! > price) {
                                dict[key] = price
                            }
                        }
                        Service.Activation.optional -> {}
                    }
                }
            }
            dict.values.forEach { fullPrice += it }
            return fullPrice
        }
}

fun PlanObject.serviceDesc(serviceId: String): DescriptionObject? {
    return this.planInfo.serviceDesc(serviceId)
}

val PlanObject.planDesc: DescriptionObject?
    get() = this.planInfo.descriptions.find { it.planId == this.plan.planID }
val PlanObject.providerDesc: DescriptionObject?
    get() = this.planInfo.providerDesc
val PlanObject.categoryDesc: DescriptionObject?
    get() = this.categoryDesc
