package nexus.billing.ui.asset

import android.animation.LayoutTransition
import android.graphics.PorterDuff
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.ScrollView
import android.widget.TextView
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.models.PlanObject
import nexus.billing.ui.rows.RowAssetType

class AssetTypeFragment: BaseFragment() {

    companion object {
        const val TAG = "AssetTypeFragment"
        fun getInstance(planObject: PlanObject): AssetTypeFragment {
            val fragment = AssetTypeFragment()
            fragment.planObject = planObject
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }
    private var planObject: PlanObject? = null
    private val viewModel: AssetTypeViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        setTitle(R.string.number_selection)
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val descTv = TextView(context)
        contentVertical.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.text = getString(R.string.number_selection_desc)
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.typeface = Theme.font
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.setPadding(dp(20f), dp(10f), dp(20f), dp(10f))

        val scroll = ScrollView(requireContext())
        scroll.elevation = dp(5f).toFloat()
        val sLp = ViewUtils.linearMatchWrap()
        scroll.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        sLp.setMargins(dp(20f), 0, dp(20f), 0)
        contentVertical.addView(scroll, sLp)

        val typesLyt = LinearLayout(requireContext())
        scroll.addView(typesLyt, ViewUtils.scrollMatchMatch())
        typesLyt.orientation = LinearLayout.VERTICAL

        this.viewModel.services.observe(viewLifecycleOwner) { result ->
            when(result.status) {
                Resource.Status.LOADING -> {
                    this.progressDialog.show()
                }
                Resource.Status.ERROR -> {
                    this.progressDialog.dismiss()
                    ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                }
                Resource.Status.SUCCESS -> {
                    this.progressDialog.dismiss()
                    result.data?.let { services ->
                        for ((i, service) in services.withIndex()) {
                            val item = RowAssetType(requireContext())
                            typesLyt.addView(item, ViewUtils.linearMatchWrap())
                            item.showData(service)
                            item.setOnClickListener {
                                this.planObject?.let {
                                    this.pushFragment(SelectAssetFragment.getInstance(it, service) {
                                        activity?.supportFragmentManager?.popBackStack()
                                    })
                                }
                            }

                            if (i < services.size - 1) {
                                val line = View(requireContext())
                                line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
                                typesLyt.addView(line, ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, 1, 0f))
                            }
                        }
                    }
                }
            }
        }
        this.planObject?.let {
            if (this.viewModel.services.value?.data == null) {
                this.viewModel.loadAssets(it.id)
            }
        } ?: run {
            requireActivity().supportFragmentManager.popBackStack()
        }
        return rootView
    }
}