package nexus.billing.ui.asset

import android.animation.LayoutTransition
import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.graphics.Color
import android.graphics.PorterDuff
import android.graphics.Typeface
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.assets.Asset
import nexus.billing.methods.service.Service
import nexus.billing.models.DescriptionObject
import nexus.billing.models.PlanObject
import nexus.billing.ui.rows.AssetsView
import nexus.billing.ui.rows.RowPlanOption
import nexus.core.extentions.toDisplayNumber
import nexus.core.util.Currency
import nexus.core.util.formattedAmount

class SelectAssetFragment: BaseFragment() {

    companion object {
        const val TAG = "SelectAssetFragment"
        fun getInstance(planObject: PlanObject, serviceInfo: Pair<Service, DescriptionObject?>, closeRunnable: Runnable? = null): SelectAssetFragment {
            val fragment = SelectAssetFragment()
            fragment.serviceInfo = serviceInfo
            fragment.planObject = planObject
            fragment.baseTag = TAG
            fragment.closeRunnable = closeRunnable
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }
    private var closeRunnable: Runnable? = null
    private var serviceInfo: Pair<Service, DescriptionObject?>? = null
    private var planObject: PlanObject? = null
    private val viewModel: SelectAssetViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val descTv = TextView(context)
        contentVertical.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.text = getString(R.string.select_asset_desc)
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.typeface = Theme.font
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.setPadding(dp(20f), dp(10f), dp(20f), dp(10f))

        val servicesCard = AssetsView(requireContext())
        val sLp = ViewUtils.linearMatchWrap()
        sLp.setMargins(dp(20f), 0, dp(20f), 0)
        contentVertical.addView(servicesCard, sLp)
        serviceInfo?.let { service ->
            this.setTitle("${(service.first.recurringAmount ?: 0).formattedAmount(service.first.currency ?: Currency.USD) }/${getString(R.string.month_little)}")
            servicesCard.showData(service)
            servicesCard.listener = object : AssetsView.AssetsListener {
                override fun assetSelected(asset: Asset) {
                    viewModel.selectAsset(planObject!!.id, service.first.id, asset.id).observe(viewLifecycleOwner) {
                        when(it.status) {
                            Resource.Status.LOADING -> {
                                progressDialog.show()
                            }
                            Resource.Status.ERROR -> {
                                progressDialog.dismiss()
                                ViewUtils.showAlertDialog(requireContext(), null, getString(R.string.request_error_repeat))
                            }
                            Resource.Status.SUCCESS -> {
                                progressDialog.dismiss()
                                showAssetDialog(requireContext(), service, asset)
                            }
                        }
                    }
                }
            }
        }

        return rootView
    }

    private fun showAssetDialog(context: Context, service: Pair<Service, DescriptionObject?>, asset: Asset) {
        val b = AlertDialog.Builder(context)
        var dialog: Dialog? = null
        val view = LinearLayout(context)
        view.setPadding(dp(20f))
        view.orientation = LinearLayout.VERTICAL
        view.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))

        val titleTv = TextView(context)
        titleTv.text = getString(R.string.congratulations)
        titleTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 20f)
        titleTv.setTypeface(Theme.font, Typeface.BOLD)
        titleTv.setPadding(0, 0, 0, dp(5f))
        titleTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        view.addView(titleTv, ViewUtils.linearMatchWrap())

        val numTv = TextView(context)
        numTv.text = getString(R.string.phone_number)
        numTv.setPadding(0, dp(5f), 0, 0)
        numTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        numTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        numTv.typeface = Theme.font
        view.addView(numTv, ViewUtils.linearMatchWrap())
        val numOption = RowPlanOption(context)
        numOption.nameTv.text = asset.number.toDisplayNumber()
        service.first.recurringAmount?.let {
            numOption.valueTv.text = it.formattedAmount(service.first.currency)
        }
        numOption.icon.setImageResource(R.drawable.ic_plan_number)
        numOption.setPadding(0, dp(5f), 0, dp(5f))
        view.addView(numOption, ViewUtils.linearMatchWrap())
        val planDescTv = TextView(context)
        planDescTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        planDescTv.text = getString(R.string.plan_registered_desc)
        planDescTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        planDescTv.typeface = Theme.font
        view.addView(planDescTv, ViewUtils.linearMatchWrap())

        val okBtn = Button(context)
        okBtn.background = ViewUtils.getRect(context.resources.getColor(R.color.green), dp(10f))
        okBtn.text = getString(R.string.okay)
        okBtn.setTextColor(Color.WHITE)
        okBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        okBtn.typeface = Theme.font
        okBtn.setPadding(dp(20f), 0, dp(20f), 0)
        okBtn.isAllCaps = false
        val sLp = ViewUtils.linearParams(LinearLayout.LayoutParams.WRAP_CONTENT, dp(40f), 0f)
        sLp.gravity = Gravity.END
        sLp.topMargin = dp(30f)
        sLp.bottomMargin = dp(5f)
        view.addView(okBtn, sLp)
        okBtn.setOnClickListener {
            dialog?.dismiss()
            activity?.supportFragmentManager?.popBackStack()
            this.closeRunnable?.run()
        }

        b.setView(view)
        dialog = b.create();
        dialog.show()
    }
}