package nexus.billing.ui.asset

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.liveData
import androidx.lifecycle.switchMap
import nexus.appui.utils.Resource
import nexus.billing.billingKit
import nexus.billing.methods.service.Service
import nexus.billing.models.DescriptionObject
import nexus.billing.models.PlanObject
import nexus.core.NCPCore

class SelectAssetViewModel: ViewModel() {

    private val profilePlanId = MutableLiveData<String>()

    val services: LiveData<Resource<List<Pair<Service, DescriptionObject?>>>> = this.profilePlanId.switchMap { profilePlanId ->
        liveData {
            emit(Resource.loading(null))
            try {
                val plans = NCPCore.instance.billingKit.controller.requestPlanAssets(profilePlanId)
                emit(Resource.success(plans))
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun loadAssets(profilePlanId: String) {
        this.profilePlanId.value = profilePlanId
    }

    fun selectAsset(profilePlanId: String, serviceId: String, assetId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                NCPCore.instance.billingKit.controller.requestPlanAssetsSelect(profilePlanId, serviceId, assetId)?.let { result ->
                    emit(Resource.success(result))
                } ?: run {
                    emit(Resource.error("Error select asset", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }
}