package nexus.billing.ui.plan

import android.animation.LayoutTransition
import android.graphics.PorterDuff
import android.os.Bundle
import android.transition.TransitionInflater
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import androidx.activity.OnBackPressedCallback
import androidx.core.view.setMargins
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import nexus.billing.R
import nexus.billing.ui.profile.ProfilePlansViewModel
import nexus.billing.ui.rows.AddServiceView
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils

class AddPlanFragment: BaseFragment() {

    companion object {
        const val TAG = "ProfilePlansFragment"
        fun getInstance(): AddPlanFragment {
            val fragment = AddPlanFragment()
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }

    private val viewModel: ProfilePlansViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)

        requireActivity().onBackPressedDispatcher
            .addCallback(this, object : OnBackPressedCallback(true) {
                override fun handleOnBackPressed() {
                    if (isEnabled) {
                        isEnabled = false
                        requireActivity().onBackPressed()
                    }
                }
            })
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setSubtitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.setTitle(R.string.add)
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val addService = AddServiceView(context)
        this.viewModel.plans.observe(viewLifecycleOwner) { result ->
            when(result.status) {
                Resource.Status.LOADING -> {
                    addService.progress.visibility = View.VISIBLE
                }
                Resource.Status.ERROR -> {
                    addService.progress.visibility = View.GONE
                }
                Resource.Status.SUCCESS -> {
                    addService.progress.visibility = View.GONE
                    result.data?.let { plans ->
                        addService.showData(plans) { publicPlan ->
                            pushFragment(SelectCountryFragment.getInstance(publicPlan.category.id, plans))
                        }
                    }
                }
            }
        }
        if ((this.viewModel.plans.value?.data?.size ?: 0) == 0) {
            this.viewModel.loadPlans()
        }
        contentVertical.addView(addService, ViewUtils.linearMatchWrap().apply { setMargins(dp(20f)) })

        return rootView
    }
}