package nexus.billing.ui.plan

import android.animation.LayoutTransition
import android.graphics.PorterDuff
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.transition.TransitionInflater
import com.google.android.material.appbar.MaterialToolbar
import nexus.billing.R
import nexus.billing.ui.rows.RowPlanNotify
import nexus.appui.Theme
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.ViewUtils

class PlanNotifyFragment: BaseFragment() {

    companion object {
        const val TAG = "PlanNotifyFragment"
        fun getInstance(): PlanNotifyFragment {
            val fragment = PlanNotifyFragment()
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }

    private lateinit var rv: RecyclerView
    private lateinit var adapter: NotifyAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Log.d(TAG, "onCreate")
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()

        val rootView = LinearLayout(context)
        rootView.orientation = LinearLayout.VERTICAL
        rootView.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))
        rootView.layoutTransition = LayoutTransition()
        rootView.layoutParams = ViewUtils.frameMatchMatch()
        this.toolbar = MaterialToolbar(context)
        rootView.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.setTitle(R.string.notifications)
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val notifies = ArrayList<PlanNotify>()
        for (i in 1..10) {
            notifies.add(PlanNotify("Tamos", "Уникальное предложение: мегабайты в подарок за входящие звонки с других мобильных операторов!\n1 минута – 10 мегабайт. Абонентская плата – 2 сомони в месяц.", R.drawable.ic_tamos))
        }

        this.rv = RecyclerView(context)
        rootView.addView(this.rv, ViewUtils.linearMatchWrap())
        this.rv.layoutManager = LinearLayoutManager(context)
        this.adapter = NotifyAdapter(notifies)
        this.rv.adapter = adapter
        return rootView
    }

    class PlanNotify(val name: String, val text: String, val img: Int)

    class NotifyAdapter(val notifyList: List<PlanNotify>) : RecyclerView.Adapter<NotifyAdapter.NotifyHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): NotifyHolder {
            return NotifyHolder(RowPlanNotify(parent.context))
        }

        override fun onBindViewHolder(holder: NotifyHolder, position: Int) {
            val notify = notifyList[position]
            val item = holder.itemView as RowPlanNotify
            item.showNotify(notify)
        }

        override fun getItemCount(): Int {
            return notifyList.size
        }

        inner class NotifyHolder(itemView: View) : RecyclerView.ViewHolder(itemView)
    }
}