package nexus.billing.ui.plan

import android.animation.LayoutTransition
import android.app.AlertDialog
import android.app.Dialog
import android.graphics.Color
import android.graphics.PorterDuff
import android.graphics.Typeface
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.ScrollView
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.cardview.widget.CardView
import androidx.core.view.setPadding
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.R
import nexus.billing.methods.common.Verification
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.methods.service.Service
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.billing.ui.checkEmptyLyt
import nexus.billing.ui.isCoreConnected
import nexus.billing.ui.profile.ProfilePlanViewFragment
import nexus.billing.ui.route.PlanRoutesFragment
import nexus.billing.ui.rows.RowOption
import nexus.billing.ui.rows.RowPlanCountry
import nexus.billing.ui.rows.RowPlanPackage
import nexus.billing.ui.rows.RowPlanService
import nexus.billing.ui.rows.RowPlanTopInfo
import nexus.core.util.formattedAmount
import java.util.Locale

class PublicPlanViewFragment: BaseFragment() {

    companion object {
        const val TAG = "PublicPlanViewFragment"
        fun getInstance(plan: PlanInfo): PublicPlanViewFragment {
            val fragment = PublicPlanViewFragment()
            fragment.baseTag = TAG
            fragment.planInfo = plan
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }

    private var planInfo: PlanInfo? = null
    private val viewModel: PublicPlanViewViewModel by viewModels()
    private lateinit var addBtn: Button
    private lateinit var cardIdentity: CardView

    private lateinit var servicesLyt: LinearLayout
    private lateinit var identityLyt: LinearLayout
    private lateinit var voiceLyt: LinearLayout
    private lateinit var smsLyt: LinearLayout
    private lateinit var dataLyt: LinearLayout
    private lateinit var othersLyt: LinearLayout
    private lateinit var assetsLyt: LinearLayout

    private lateinit var identityTv: TextView
    private lateinit var servicesTv: TextView
    private lateinit var voiceTv: TextView
    private lateinit var smsTv: TextView
    private lateinit var dataTv: TextView
    private lateinit var othersTv: TextView
    private lateinit var assetsTv: TextView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = LinearLayout(context)
        rootView.orientation = LinearLayout.VERTICAL
        rootView.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))
        rootView.setOnClickListener {}

        this.toolbar = MaterialToolbar(context)
        rootView.addView(this.toolbar, ViewUtils.linearMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.setTitle(this.planInfo?.description?.title ?: this.planInfo?.name ?: "")
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }
        this.toolbar.setOnMenuItemClickListener {
            return@setOnMenuItemClickListener true
        }

        val scroll = ScrollView(context)
        rootView.addView(scroll, ViewUtils.linearMatchMatch())
        val scrollContent = LinearLayout(context)
        scrollContent.orientation = LinearLayout.VERTICAL
        scrollContent.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))
        scrollContent.layoutTransition = LayoutTransition()
        scrollContent.isClickable = true
        scroll.addView(scrollContent, ViewUtils.scrollMatchMatch())

//        INFO LAYOUT
        val cardPlanInfo = CardView(context)
        cardPlanInfo.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        cardPlanInfo.cardElevation = dp(5f).toFloat()
        cardPlanInfo.radius = dp(15f).toFloat()
        scrollContent.addView(cardPlanInfo, ViewUtils.linearMatchWrap().apply { setMargins(dp(20f), dp(20f), dp(20f), dp(10f)) })
        val topInfo = RowPlanTopInfo(context)
        topInfo.setPadding(dp(10f))
        cardPlanInfo.addView(topInfo, ViewUtils.linearMatchWrap())
        this.planInfo?.let { topInfo.showData(it, false) }

//        IDENT CARD
        this.identityTv = TextView(context)
        scrollContent.addView(this.identityTv, ViewUtils.linearMatchWrap())
        this.identityTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.identityTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.identityTv.isAllCaps = true
        this.identityTv.text = getString(R.string.req_ident)
        this.identityTv.setPadding(dp(25f), dp(15f), 0, 0)

        this.cardIdentity = CardView(context)
        this.cardIdentity.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        this.cardIdentity.cardElevation = dp(5f).toFloat()
        this.cardIdentity.radius = dp(15f).toFloat()
        scrollContent.addView(this.cardIdentity, ViewUtils.linearMatchWrap().apply { setMargins(dp(20f), dp(10f), dp(20f), dp(10f)) })
        this.identityLyt = LinearLayout(context)
        this.identityLyt.orientation = LinearLayout.VERTICAL
        this.cardIdentity.addView(identityLyt, ViewUtils.linearMatchWrap())

//        ASSETS
        this.assetsTv = TextView(context)
        scrollContent.addView(this.assetsTv, ViewUtils.linearMatchWrap())
        this.assetsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.assetsTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.assetsTv.isAllCaps = true
        this.assetsTv.text = getString(R.string.assets)
        this.assetsTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.assetsLyt = LinearLayout(context)
        this.assetsLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.assetsLyt, ViewUtils.linearMatchWrap())

//        VOICE
        this.voiceTv = TextView(context)
        scrollContent.addView(this.voiceTv, ViewUtils.linearMatchWrap())
        this.voiceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.voiceTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.voiceTv.isAllCaps = true
        this.voiceTv.text = getString(R.string.voice_service)
        this.voiceTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.voiceLyt = LinearLayout(context)
        this.voiceLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.voiceLyt, ViewUtils.linearMatchWrap())

//        SMS
        this.smsTv = TextView(context)
        scrollContent.addView(this.smsTv, ViewUtils.linearMatchWrap())
        this.smsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.smsTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.smsTv.isAllCaps = true
        this.smsTv.text = getString(R.string.sms)
        this.smsTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.smsLyt = LinearLayout(context)
        this.smsLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.smsLyt, ViewUtils.linearMatchWrap())

//        DATA
        this.dataTv = TextView(context)
        scrollContent.addView(this.dataTv, ViewUtils.linearMatchWrap())
        this.dataTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.dataTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.dataTv.isAllCaps = true
        this.dataTv.text = getString(R.string.data_service)
        this.dataTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.dataLyt = LinearLayout(context)
        this.dataLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.dataLyt, ViewUtils.linearMatchWrap())

//        OTHERS
        this.othersTv = TextView(context)
        scrollContent.addView(this.othersTv, ViewUtils.linearMatchWrap())
        this.othersTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.othersTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.othersTv.isAllCaps = true
        this.othersTv.text = getString(R.string.others_service)
        this.othersTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.othersLyt = LinearLayout(context)
        this.othersLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.othersLyt, ViewUtils.linearMatchWrap())

//        SERVICES
        this.servicesTv = TextView(context)
        scrollContent.addView(this.servicesTv, ViewUtils.linearMatchWrap())
        this.servicesTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.servicesTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.servicesTv.isAllCaps = true
        this.servicesTv.text = getString(R.string.services)
        this.servicesTv.setPadding(dp(25f), dp(15f), 0, 0)

        this.servicesLyt = LinearLayout(context)
        this.servicesLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.servicesLyt, ViewUtils.linearMatchWrap())

        val descTv = TextView(context)
        scrollContent.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.typeface = Theme.font
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.setPadding(dp(20f))
        descTv.text = this.planInfo?.description?.longText
        descTv.visibility = if (this.planInfo?.description?.longText.isNullOrEmpty()) View.GONE else View.VISIBLE

        this.addBtn = Button(requireContext())
        this.addBtn.background = ViewUtils.getRect(theme.getColor(Theme.key_primaryColor), dp(8f))
        this.addBtn.setTextColor(Color.WHITE)
        this.addBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.addBtn.gravity = Gravity.CENTER
        this.addBtn.isAllCaps = false
        this.addBtn.setText(R.string.subscribe)
        val bLp = ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, dp(45f), 0f).apply {
            setMargins(dp(20f), dp(20f),  dp(20f), dp(5f))
        }
        scrollContent.addView(this.addBtn, bLp)
        this.addBtn.setDebounceClickListener {
             ViewUtils.showDialog(requireActivity(), getString(R.string.subscribe), getString(R.string.want_subscribe)) { _, _ ->
                 this.planInfo?.let { plan ->
                     if (isCoreConnected(requireActivity())) {
                         this.viewModel.selectPlan(plan.planID).observe(viewLifecycleOwner) {
                             when(it.status) {
                                 Resource.Status.LOADING -> {
                                     this.progressDialog.show()
                                 }
                                 Resource.Status.ERROR -> {
                                     this.progressDialog.dismiss()
                                     ViewUtils.showAlertDialog(requireContext(), null, getString(R.string.request_error_repeat))
                                 }
                                 Resource.Status.SUCCESS -> {
                                     this.progressDialog.dismiss()
                                     it.data?.let { planObject ->
                                         planObject.planInfo.services = plan.services
                                         showPlanRegisteredDialog(requireActivity() as AppCompatActivity, planObject)
                                     } ?: run {
                                         ViewUtils.showAlertDialog(requireContext(), null, getString(R.string.request_error_repeat))
                                     }
                                 }
                             }
                         }
                     }
                 }
             }
        }

        this.planInfo?.let {
            this.showIdentities(it.verificationRequired)
            this.showServices(it.services)
        }
        return rootView
    }

    private fun showIdentities(identities: Array<Verification>) {
        this.identityLyt.removeAllViews()
        if (identities.isEmpty()) {
            this.identityTv.visibility = View.GONE
            this.cardIdentity.visibility = View.GONE
        } else {
            this.identityTv.visibility = View.VISIBLE
            this.cardIdentity.visibility = View.VISIBLE
        }
        for (identity in identities) {
            val row = RowPlanCountry(requireContext())
            this.identityLyt.addView(row, ViewUtils.linearMatchWrap())
            row.arr.visibility = View.GONE
            row.codeTv.visibility = View.GONE
            row.setCountry(identity.country)
        }
    }

    private fun showServices(services: Array<Service>) {
        this.servicesLyt.removeAllViews()
        this.voiceLyt.removeAllViews()
        this.smsLyt.removeAllViews()
        this.dataLyt.removeAllViews()
        this.othersLyt.removeAllViews()
        this.assetsLyt.removeAllViews()

        for (service in services.filter { it.activation != Service.Activation.optional }) {
            val cardLp = ViewUtils.linearMatchWrap()
            cardLp.setMargins(dp(20f), dp(5f), dp(20f), dp(5f))
            when (service.type) {
                Service.Types.`package`,
                Service.Types.route -> {
                    val packageItem = RowPlanPackage(requireContext())
                    packageItem.showData(service, this.planInfo?.serviceDesc(service.id))
                    CoroutineScope(Dispatchers.IO).launch {
                        planInfo?.let {
                            viewModel.getPlanRoutes(planId = it.planID, service.id).let { routes ->
                                withContext(Dispatchers.Main) {
                                    packageItem.showRoutes(routes) {
                                        pushFragment(PlanRoutesFragment.getInstance(routes, service.currency, this@PublicPlanViewFragment.planInfo?.serviceDesc(service.id)))
                                    }
                                }
                            }
                        }
                    }
                    when(service.category) {
                        Service.Category.voice -> {
                            this.voiceLyt.addView(packageItem, cardLp)
                        }
                        Service.Category.sms -> {
                            this.smsLyt.addView(packageItem, cardLp)
                        }
                        Service.Category.data -> {
                            this.dataLyt.addView(packageItem, cardLp)
                        }
                        else -> {
                            this.othersLyt.addView(packageItem, cardLp)
                        }
                    }
                }
                Service.Types.asset -> {
                    val assetItem = RowPlanService(requireContext())
                    assetItem.showData(service, this.planInfo?.serviceDesc(service.id))
                    this.assetsLyt.addView(assetItem, cardLp)
                }
                else -> {
                    val serviceItem = RowPlanService(requireContext())
                    serviceItem.showData(service, this.planInfo?.serviceDesc(service.id))
                    this.servicesLyt.addView(serviceItem, cardLp)
                }
            }
        }

        checkEmptyLyt(this.servicesLyt, this.servicesTv)
        checkEmptyLyt(this.voiceLyt, this.voiceTv)
        checkEmptyLyt(this.smsLyt, this.smsTv)
        checkEmptyLyt(this.dataLyt, this.dataTv)
        checkEmptyLyt(this.othersLyt, this.othersTv)
        checkEmptyLyt(this.assetsLyt, this.assetsTv)
    }

    private fun showPlanRegisteredDialog(context: AppCompatActivity, plan: PlanObject) {
        val b = AlertDialog.Builder(context)
        var dialog: Dialog? = null
        val view = LinearLayout(context)
        view.setPadding(dp(20f))
        view.orientation = LinearLayout.VERTICAL
        view.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))

        val titleTv = TextView(context)
        titleTv.text = context.getString(R.string.congratulations)
        titleTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 20f)
        titleTv.setTypeface(Theme.font, Typeface.BOLD)
        titleTv.setPadding(0, 0, 0, dp(5f))
        titleTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        view.addView(titleTv, ViewUtils.linearMatchWrap())
        val descTv = TextView(context)
        descTv.text = context.getString(R.string.you_register_plan)
        descTv.setPadding(0, 0, 0, dp(5f))
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        descTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        descTv.typeface = Theme.font
        view.addView(descTv, ViewUtils.linearMatchWrap())
        val infoView = RowPlanTopInfo(context)
        infoView.showData(plan)
        view.addView(infoView, ViewUtils.linearMatchWrap())

        val actionsTv = TextView(context)
        actionsTv.setTextColor(Color.RED)
        actionsTv.text = getString(R.string.required_actions)
        actionsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        actionsTv.setPadding(0, dp(10f), 0, dp(5f))
        actionsTv.typeface = Theme.font
        view.addView(actionsTv, ViewUtils.linearMatchWrap())

        val actionsLyt = LinearLayout(context)
        actionsLyt.orientation = LinearLayout.VERTICAL
        view.addView(actionsLyt, ViewUtils.linearMatchWrap())
        for (action in plan.plan.actions) {
            var actionItem: RowOption? = null
            when {
                action.identity?.info != null -> {
                    if (action.identity.info.state == ProfilePlan.Action.State.pending) {
                        actionItem = RowOption(context)
                        val verification = action.identity.info.required.firstOrNull()
                        actionItem.icon.setImageResource(R.drawable.ic_plan_verification)
                        verification?.let {
                            actionItem?.nameTv?.text = "${context.getString(R.string.verification)} ${Locale(it.country, it.country).displayCountry}"
                        } ?: run {
                            actionItem?.nameTv?.text = context.getString(R.string.verification)
                        }
                    }
                }
                action.asset?.info != null -> {
                    if (action.asset.info.state == ProfilePlan.Action.State.pending) {
                        actionItem = RowOption(context)
                        actionItem.icon.setImageResource(R.drawable.ic_sim)
                        actionItem.nameTv.text = context.getString(R.string.select_did_number)
                    }
                }
                action.support?.info != null -> {
                    if (action.support.info.state == ProfilePlan.Action.State.pending) {
                        actionItem = RowOption(context)
                        actionItem.nameTv.text = context.getString(R.string.talk_to_support)
                        actionItem.icon.setImageResource(R.drawable.ic_support)
                    }
                }
                action.balance?.info != null -> {
                    if (action.balance.info.state == ProfilePlan.Action.State.pending) {
                        actionItem = RowOption(context)
                        actionItem.nameTv.text = context.getString(R.string.payment_required)
                        actionItem.icon.setImageResource(R.drawable.ic_balance)
                        actionItem.valueTv.text = action.balance.info.required.formattedAmount(action.balance.info.currency)
                    }
                }
            }
            actionItem?.let {
                actionItem.background = null
                actionItem.arr.visibility = View.GONE
                actionsLyt.addView(actionItem, ViewUtils.linearMatchWrap())
            }
        }
        if (actionsLyt.childCount == 0) {
            actionsTv.visibility = View.GONE
            actionsLyt.visibility = View.GONE
        } else {
            actionsTv.visibility = View.VISIBLE
            actionsLyt.visibility = View.VISIBLE
        }

        val planDescTv = TextView(context)
        planDescTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        planDescTv.text = context.getString(R.string.plan_registered_desc)
        planDescTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        planDescTv.setPadding(0, dp(10f), 0, 0)
        planDescTv.typeface = Theme.font
        view.addView(planDescTv, ViewUtils.linearMatchWrap())

        val okBtn = Button(context)
        okBtn.background = ViewUtils.getRect(context.resources.getColor(R.color.green), dp(10f))
        okBtn.text = context.getString(R.string.continuee)
        okBtn.setTextColor(Color.WHITE)
        okBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        okBtn.typeface = Theme.font
        okBtn.setPadding(dp(20f), 0, dp(20f), 0)
        okBtn.isAllCaps = false
        val sLp = ViewUtils.linearParams(LinearLayout.LayoutParams.WRAP_CONTENT, dp(40f), 0f)
        sLp.gravity = Gravity.END
        sLp.topMargin = dp(35f)
        sLp.bottomMargin = dp(5f)
        view.addView(okBtn, sLp)
        okBtn.setOnClickListener {
            dialog?.dismiss()
            val fragment = ProfilePlanViewFragment.getInstance(plan.id, true)
            val fm = context.supportFragmentManager
            if (fm.backStackEntryCount > 1) {
                fm.popBackStack(fm.getBackStackEntryAt(1).id, FragmentManager.POP_BACK_STACK_INCLUSIVE)
            } else {
                fm.popBackStack()
            }
            fm.beginTransaction()
                .setReorderingAllowed(true)
                .replace(nexus.appui.R.id.fragment_details, fragment, fragment.baseTag)
                .addToBackStack(null).commit()
        }

        b.setView(view)
        dialog = b.create()
        dialog.setCancelable(false)
        dialog.show()
    }
}