package nexus.billing.ui.plan

import androidx.lifecycle.LiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.liveData
import nexus.appui.utils.Resource
import nexus.billing.billingKit
import nexus.billing.methods.common.Route
import nexus.billing.models.PlanObject
import nexus.core.NCPCore

class PublicPlanViewViewModel: ViewModel() {

    private var routes = HashMap<String, Array<Route>>()

    fun selectPlan(planId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                NCPCore.instance.billingKit.controller.requestSelectPlan(planId)?.let {
                    emit(Resource.success(it))
                } ?: run {
                    emit(Resource.error("Error select", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun planRoutes(planId: String, serviceId: String): LiveData<Resource<Array<Route>>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val routes = NCPCore.instance.billingKit.controller.requestPlanServiceRoutes(planId, serviceId)
                emit(Resource.success(routes))
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), emptyArray()))
            }
        }
    }

    suspend fun getPlanRoutes(planId: String, serviceId: String): Array<Route> {
        return this.routes[serviceId]?.let {
            it
        } ?: run {
            val newRoutes = NCPCore.instance.billingKit.controller.requestPlanServiceRoutes(planId, serviceId)
            this.routes[serviceId] = newRoutes
            newRoutes
        }
    }
}