package nexus.billing.ui.plan

import android.animation.LayoutTransition
import android.content.Context
import android.graphics.PorterDuff
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.appbar.MaterialToolbar
import nexus.billing.R
import nexus.billing.models.PlanInfo
import nexus.billing.ui.rows.RowPublicPlan
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener

class RegisterPlansFragment: BaseFragment() {

    companion object {
        const val TAG = "RegisterPlansFragment"
        fun getInstance(publicPlans: ArrayList<PlanInfo>): RegisterPlansFragment {
            val fragment = RegisterPlansFragment()
            fragment.publicPlans = publicPlans
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }
    private var publicPlans = ArrayList<PlanInfo>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    private lateinit var adapter: PublicPlanAdapter

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.setTitle(R.string.select_plan)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val descTv = TextView(context)
        contentVertical.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.text = getString(R.string.select_plan_desc)
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.typeface = Theme.font
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.setPadding(dp(20f), dp(10f), dp(20f), dp(10f))

        val rv = RecyclerView(context)
        rv.layoutManager = LinearLayoutManager(context)
        contentVertical.addView(rv, ViewUtils.linearMatchMatch())
        this.adapter = PublicPlanAdapter(requireContext())
        rv.adapter = this.adapter
        this.adapter.submitList(this.publicPlans.sortedBy { it.name })

        return rootView
    }

    private val publicPlanUtil = object : DiffUtil.ItemCallback<PlanInfo>() {
        override fun areItemsTheSame(oldItem: PlanInfo, newItem: PlanInfo): Boolean {
            return oldItem.planID == newItem.planID
        }

        override fun areContentsTheSame(oldItem: PlanInfo, newItem: PlanInfo): Boolean {
            return oldItem.planID == newItem.planID && oldItem.services.size == newItem.services.size
        }
    }

    inner class PublicPlanAdapter(val context: Context) : ListAdapter<PlanInfo, PublicPlanAdapter.ViewHolder>(publicPlanUtil) {

        inner class ViewHolder(view: View): RecyclerView.ViewHolder(view)

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            return ViewHolder(LinearLayout(context))
        }

        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val plan = getItem(position)
            val root = holder.itemView as LinearLayout
            val view = RowPublicPlan(context)
            view.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
            view.setPadding(dp(12f))
            view.elevation = dp(5f).toFloat()
            view.showData(plan)
            view.setDebounceClickListener {
                pushFragment(PublicPlanViewFragment.getInstance(plan))
            }

            val lp = ViewUtils.linearMatchWrap()
            lp.setMargins(dp(20f), dp(10f), dp(20f), dp(10f))
            root.addView(view, lp)
        }
    }
}