package nexus.billing.ui.profile

import android.graphics.PorterDuff
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.ScrollView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.R
import nexus.billing.methods.service.Service
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.billing.models.serviceDesc
import nexus.billing.ui.checkEmptyLyt
import nexus.billing.ui.formattedAmount
import nexus.billing.ui.route.PlanRoutesFragment
import nexus.billing.ui.rows.RowPlanPackage
import nexus.billing.ui.rows.RowPlanService

class AddServiceFragment: BaseFragment() {

    companion object {
        const val TAG = "AddServiceFragment"
        fun getInstance(planObject: PlanObject): AddServiceFragment {
            val fragment = AddServiceFragment()
            fragment.planObject = planObject
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }

    private val viewModel: AddServiceViewModel by viewModels()
    private var planObject: PlanObject? = null

    private lateinit var servicesLyt: LinearLayout
    private lateinit var voiceLyt: LinearLayout
    private lateinit var smsLyt: LinearLayout
    private lateinit var dataLyt: LinearLayout
    private lateinit var othersLyt: LinearLayout

    private lateinit var servicesTv: TextView
    private lateinit var voiceTv: TextView
    private lateinit var smsTv: TextView
    private lateinit var dataTv: TextView
    private lateinit var othersTv: TextView
    private lateinit var emptyTv: TextView


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = LinearLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.orientation = LinearLayout.VERTICAL
        rootView.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))

        this.toolbar = MaterialToolbar(context)
        rootView.addView(this.toolbar, ViewUtils.linearMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.setTitle(R.string.add_services)
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val scroll = ScrollView(requireContext())
        scroll.isFillViewport = true
        rootView.addView(scroll, ViewUtils.linearMatchMatch())

        val frame = FrameLayout(requireContext())
        scroll.addView(frame, ViewUtils.frameMatchMatch())

        val progress = ProgressBar(requireContext())
        frame.addView(progress, ViewUtils.frameMatchWrap(Gravity.CENTER))

        this.emptyTv = TextView(context)
        frame.addView(this.emptyTv, ViewUtils.frameWrapWrap(Gravity.CENTER))
        this.emptyTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.emptyTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.emptyTv.visibility = View.GONE
        this.emptyTv.text = getString(R.string.no_service_available)
        this.emptyTv.setPadding(dp(20f), 0, dp(20f), 0)

        val scrollContent = LinearLayout(requireContext())
        frame.addView(scrollContent, ViewUtils.scrollMatchMatch())
        scrollContent.orientation = LinearLayout.VERTICAL
        scrollContent.setPadding(0, dp(10f), 0, 0)

        //        VOICE
        this.voiceTv = TextView(context)
        scrollContent.addView(this.voiceTv, ViewUtils.linearMatchWrap())
        this.voiceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.voiceTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.voiceTv.visibility = View.GONE
        this.voiceTv.isAllCaps = true
        this.voiceTv.text = getString(R.string.voice_service)
        this.voiceTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.voiceLyt = LinearLayout(context)
        this.voiceLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.voiceLyt, ViewUtils.linearMatchWrap())

//        SMS
        this.smsTv = TextView(context)
        scrollContent.addView(this.smsTv, ViewUtils.linearMatchWrap())
        this.smsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.smsTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.smsTv.visibility = View.GONE
        this.smsTv.isAllCaps = true
        this.smsTv.text = getString(R.string.sms)
        this.smsTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.smsLyt = LinearLayout(context)
        this.smsLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.smsLyt, ViewUtils.linearMatchWrap())

//        DATA
        this.dataTv = TextView(context)
        scrollContent.addView(this.dataTv, ViewUtils.linearMatchWrap())
        this.dataTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.dataTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.dataTv.visibility = View.GONE
        this.dataTv.isAllCaps = true
        this.dataTv.text = getString(R.string.data_service)
        this.dataTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.dataLyt = LinearLayout(context)
        this.dataLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.dataLyt, ViewUtils.linearMatchWrap())

//        OTHERS
        this.othersTv = TextView(context)
        scrollContent.addView(this.othersTv, ViewUtils.linearMatchWrap())
        this.othersTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.othersTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.othersTv.visibility = View.GONE
        this.othersTv.isAllCaps = true
        this.othersTv.text = getString(R.string.others_service)
        this.othersTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.othersLyt = LinearLayout(context)
        this.othersLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.othersLyt, ViewUtils.linearMatchWrap())


//        SERVICES
        this.servicesTv = TextView(context)
        scrollContent.addView(this.servicesTv, ViewUtils.linearMatchWrap())
        this.servicesTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.servicesTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.servicesTv.visibility = View.GONE
        this.servicesTv.isAllCaps = true
        this.servicesTv.text = getString(R.string.services)
        this.servicesTv.setPadding(dp(25f), dp(15f), 0, 0)

        this.servicesLyt = LinearLayout(context)
        this.servicesLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.servicesLyt, ViewUtils.linearMatchWrap())

        this.planObject?.let { plan ->
            this.viewModel.getPlan(plan.plan.planID).observe(viewLifecycleOwner) { result ->
                when(result.status) {
                    Resource.Status.LOADING -> {
                        progress.visibility = View.VISIBLE
                    }
                    Resource.Status.SUCCESS -> {
                        progress.visibility = View.GONE
                        result.data?.let { info ->
                            this.showServices(plan, info)
                        }
                    }
                    Resource.Status.ERROR -> {
                        progress.visibility = View.GONE
                    }
                }
            }
        } ?: run {
            requireActivity().supportFragmentManager.popBackStack()
        }
        return rootView
    }

    private fun showServices(plan: PlanObject, planInfo: PlanInfo) {
        this.servicesLyt.removeAllViews()
        this.voiceLyt.removeAllViews()
        this.smsLyt.removeAllViews()
        this.dataLyt.removeAllViews()
        this.othersLyt.removeAllViews()

        val services = planInfo.services.filter { service -> plan.plan.services.find { it.serviceID == service.id } == null
                && service.activation == Service.Activation.optional && service.type != Service.Types.simcard && service.type != Service.Types.voicemail }
        this.emptyTv.visibility = if (services.isEmpty()) View.VISIBLE else View.GONE
        for (service in services) {
            val cardLp = ViewUtils.linearMatchWrap()
            cardLp.setMargins(dp(20f), dp(5f), dp(20f), dp(5f))
            when (service.type) {
                Service.Types.`package`,
                Service.Types.route -> {
                    val serviceItem = RowPlanPackage(requireContext())
                    serviceItem.showData(service, plan.serviceDesc(service.id))
                    CoroutineScope(Dispatchers.IO).launch {
                        this@AddServiceFragment.viewModel.getPlanRoutes(planId = planInfo.planID, service.id).let { routes ->
                            withContext(Dispatchers.Main) {
                                serviceItem.showRoutes(routes) {
                                    pushFragment(PlanRoutesFragment.getInstance(routes, service.currency, planInfo.serviceDesc(service.id)))
                                }
                            }
                        }
                    }
                    when(service.category) {
                        Service.Category.voice -> {
                            this.voiceLyt.addView(serviceItem, cardLp)
                        }
                        Service.Category.sms -> {
                            this.smsLyt.addView(serviceItem, cardLp)
                        }
                        Service.Category.data -> {
                            this.dataLyt.addView(serviceItem, cardLp)
                        }
                        else -> {
                            this.othersLyt.addView(serviceItem, cardLp)
                        }
                    }
                    serviceItem.setDebounceClickListener {
                        this.activeService(plan.id, service)
                    }
                }
                else -> {
                    val serviceItem = RowPlanService(requireContext())
                    serviceItem.showData(service, plan.serviceDesc(service.id))
                    this.servicesLyt.addView(serviceItem, cardLp)
                    serviceItem.setDebounceClickListener {
                        this.activeService(plan.id, service)
                    }
                }
            }
        }

        checkEmptyLyt(this.servicesLyt, this.servicesTv)
        checkEmptyLyt(this.voiceLyt, this.voiceTv)
        checkEmptyLyt(this.smsLyt, this.smsTv)
        checkEmptyLyt(this.dataLyt, this.dataTv)
        checkEmptyLyt(this.othersLyt, this.othersTv)
    }

    private fun activeService(profilePlanId: String, service: Service) {
        val price = service.recurringAmount ?: 0
        var priceTxt = ""
        if (price > 0) {
            priceTxt = "\nService cost: ${price.formattedAmount(service.currency)}"
        }

        val b = AlertDialog.Builder(requireContext(), Theme.dialogTheme)
        b.setMessage(getString(R.string.want_add_service))
        b.setNegativeButton(nexus.appui.R.string.cancel) { dialog, _ -> dialog.cancel() }
        b.setPositiveButton(nexus.appui.R.string.ok) { dialog, _ ->
            this.viewModel.addService(profilePlanId, service.id).observe(viewLifecycleOwner) { result ->
                when(result.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                        requireActivity().supportFragmentManager.popBackStack()
//                        result.data?.let { result ->
//                            this.viewModel.planInfo?.let {
//                                this.showServices(result, it)
//                            }
//                        }
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                }
            }
        }
        b.create().show()
    }
}