package nexus.billing.ui.profile

import androidx.lifecycle.LiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.liveData
import nexus.appui.utils.Resource
import nexus.billing.billingKit
import nexus.billing.methods.common.Route
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.core.NCPCore

class AddServiceViewModel: ViewModel() {

    private var routes = HashMap<String, Array<Route>>()
    var planInfo: PlanInfo? = null

    fun getPlan(planId: String): LiveData<Resource<PlanInfo>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestGetPlanWithInfo(planId)
                if (result != null) {
                    this@AddServiceViewModel.planInfo = result
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("plan not found", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun activateService(profilePlanId: String, serviceId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanServiceActivate(profilePlanId, serviceId)
                if (result != null) {
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("error activate service", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun removeService(profilePlanId: String, serviceId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanServiceRemove(profilePlanId, serviceId)
                if (result != null) {
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("error remove service", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun addService(profilePlanId: String, serviceId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanServiceAdd(profilePlanId, serviceId)
                if (result != null) {
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("error add service", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    suspend fun getPlanRoutes(planId: String, serviceId: String): Array<Route> {
        return this.routes[serviceId]?.let {
            it
        } ?: run {
            val newRoutes = NCPCore.instance.billingKit.controller.requestPlanServiceRoutes(planId, serviceId)
            this.routes[serviceId] = newRoutes
            newRoutes
        }
    }

}