package nexus.billing.ui.profile

import android.animation.LayoutTransition
import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.PorterDuff
import android.graphics.Typeface
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.ScrollView
import android.widget.TextView
import androidx.activity.OnBackPressedCallback
import androidx.appcompat.app.AppCompatActivity
import androidx.cardview.widget.CardView
import androidx.core.view.setPadding
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.ViewUtils.frameMatchMatch
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.R
import nexus.billing.billingKit
import nexus.billing.methods.common.Route
import nexus.billing.methods.plan.Plan
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.methods.profile.ProfileService
import nexus.billing.methods.service.Service
import nexus.billing.models.PlanObject
import nexus.billing.models.planDesc
import nexus.billing.models.serviceDesc
import nexus.billing.ui.asset.AssetTypeFragment
import nexus.billing.ui.checkEmptyLyt
import nexus.billing.ui.profile.voicemail.VoiceMailFragment
import nexus.billing.ui.route.PlanRoutesFragment
import nexus.billing.ui.rows.RowForward
import nexus.billing.ui.rows.RowNgn
import nexus.billing.ui.rows.RowOption
import nexus.billing.ui.rows.RowPlanAction
import nexus.billing.ui.rows.RowPlanOption
import nexus.billing.ui.rows.RowPlanPackage
import nexus.billing.ui.rows.RowPlanTopInfo
import nexus.core.NCPCore
import nexus.core.extentions.toDisplayNumber
import nexus.core.methods.access.EndpointViewPermission
import nexus.core.util.formattedAmount
import nexus.passport.passportKit
import nexus.passport.ui.verification.VerifyActivity
import java.text.SimpleDateFormat
import java.util.Date

class ProfilePlanViewFragment: BaseFragment() {

    companion object {
        const val TAG = "ProfilePlanViewFragment"
        const val OPEN_ACTION = "openAction"
        const val PLAN_ID = "planId"
        fun getInstance(profilePlanId: String, openAction: Boolean = false): ProfilePlanViewFragment {
            val fragment = ProfilePlanViewFragment()
            fragment.baseTag = TAG
            fragment.arguments = Bundle().apply {
                putBoolean(OPEN_ACTION, openAction)
                putString(PLAN_ID, profilePlanId)
            }
            return fragment
        }
    }

    private val MENU_ADD = 11
    private val MENU_DELETE = 22
    private val verifyReqCode = 555
    private var canGoBack = true
    private var updateForward = true
    private val viewModel: ProfilePlanViewViewModel by viewModels()

    private lateinit var addOrActiveBtn: Button

    private lateinit var cardAssets: CardView
    private lateinit var assetsLyt: LinearLayout

    private lateinit var cardServices: CardView
    private lateinit var servicesLyt: LinearLayout

    private lateinit var actionsLyt: LinearLayout
    private lateinit var voiceLyt: LinearLayout
    private lateinit var smsLyt: LinearLayout
    private lateinit var dataLyt: LinearLayout
    private lateinit var othersLyt: LinearLayout

    private lateinit var servicesTv: TextView
    private lateinit var assetsTv: TextView
    private lateinit var voiceTv: TextView
    private lateinit var smsTv: TextView
    private lateinit var dataTv: TextView
    private lateinit var othersTv: TextView

    private lateinit var priceItem: RowOption
    private lateinit var renewItem: RowOption
    private lateinit var activeItem: RowOption
    private lateinit var forwardItem: RowForward
    private lateinit var voiceMailItem: RowOption
    private lateinit var forwardLine: View
    private lateinit var progressBar: ProgressBar

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = LinearLayout(context)
        rootView.orientation = LinearLayout.VERTICAL
        rootView.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))
        rootView.setOnClickListener{}

        this.toolbar = MaterialToolbar(context)
        rootView.addView(this.toolbar, ViewUtils.linearMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.popupTheme = if (Theme.isDark) nexus.appui.R.style.ToolBarDark else nexus.appui.R.style.ToolBarLight
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }
        this.toolbar.setOnMenuItemClickListener {
            if (it.itemId == MENU_DELETE) {
                this.viewModel.profilePlan.value?.let {
                    this.showDeleteDialog(requireContext(), it)
                }
            } else if (it.itemId == MENU_ADD) {
                this.viewModel.profilePlan.value?.let {
                    this.pushFragment(AddServiceFragment.getInstance(it))
                }
            }
            true
        }

        val scroll = ScrollView(context)
        scroll.isFillViewport = true
        rootView.addView(scroll, ViewUtils.linearMatchMatch())
        val scrollContent = LinearLayout(context)
        scrollContent.orientation = LinearLayout.VERTICAL
        scrollContent.layoutTransition = LayoutTransition()
        scrollContent.isClickable = true
        scroll.addView(scrollContent, ViewUtils.scrollMatchMatch())

//        ACTIONS
        this.actionsLyt = LinearLayout(context)
        this.actionsLyt.orientation = LinearLayout.VERTICAL
        this.actionsLyt.setPadding(0, dp(15f), 0, 0)
        scrollContent.addView(this.actionsLyt, ViewUtils.linearMatchWrap())

//        INFO LAYOUT
        val cardPlanInfo = CardView(context)
        cardPlanInfo.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        cardPlanInfo.cardElevation = dp(5f).toFloat()
        cardPlanInfo.radius = dp(10f).toFloat()
        cardPlanInfo.visibility = View.GONE
        scrollContent.addView(cardPlanInfo, ViewUtils.linearMatchWrap().apply { setMargins(dp(20f), dp(20f), dp(20f), dp(10f)) })
        val infoLyt = LinearLayout(context)
        infoLyt.orientation = LinearLayout.VERTICAL
        infoLyt.setPadding(dp(10f))
        cardPlanInfo.addView(infoLyt, ViewUtils.linearMatchWrap())
        val topInfo = RowPlanTopInfo(context)
        val tLp = ViewUtils.linearMatchWrap()
        tLp.bottomMargin = dp(10f)
        infoLyt.addView(topInfo, tLp)

        val line = View(context)
        line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        infoLyt.addView(line, ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, 1, 0f))

        this.priceItem = RowOption(context)
        this.priceItem.background = null
        this.priceItem.icon.setImageResource(R.drawable.ic_cards)
        this.priceItem.arr.visibility = View.GONE
        infoLyt.addView(this.priceItem, ViewUtils.linearMatchWrap())
        this.renewItem = RowOption(context)
        this.renewItem.background = null
        this.renewItem.icon.setImageResource(R.drawable.ic_calendar)
        this.renewItem.arr.visibility = View.GONE
        infoLyt.addView(this.renewItem, ViewUtils.linearMatchWrap())
        this.activeItem = RowOption(context)
        this.activeItem.background = null
        this.activeItem.icon.setImageResource(R.drawable.ic_checkmark)
        this.activeItem.icon.setPadding(dp(3f))
        this.activeItem.arr.visibility = View.GONE
        infoLyt.addView(this.activeItem, ViewUtils.linearMatchWrap())

        this.forwardLine = View(context)
        this.forwardLine.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        infoLyt.addView(this.forwardLine, ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, 1, 0f))

        this.forwardItem = RowForward(context)
        this.forwardItem.icon.setImageResource(R.drawable.ic_forwarding)
        this.forwardItem.icon.setPadding(dp(3f))
        infoLyt.addView(this.forwardItem, ViewUtils.linearMatchWrap())
        this.voiceMailItem = RowOption(context)
        this.voiceMailItem.icon.setPadding(dp(3f))
        this.voiceMailItem.setPadding(0, dp(4f), 0, dp(4f))
        this.voiceMailItem.arr.visibility = View.GONE
        this.voiceMailItem.nameTv.setText(R.string.voice_mail)
        this.voiceMailItem.icon.setImageResource(R.drawable.ic_voicemail)
        infoLyt.addView(this.voiceMailItem, ViewUtils.linearMatchWrap())

//        ASSETS
        this.assetsTv = TextView(context)
        this.assetsTv.visibility = View.GONE
        scrollContent.addView(this.assetsTv, ViewUtils.linearMatchWrap())
        this.assetsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.assetsTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.assetsTv.isAllCaps = true
        this.assetsTv.text = getString(R.string.assets)
        this.assetsTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.cardAssets = CardView(context)
        this.cardAssets.visibility = View.GONE
        this.cardAssets.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        this.cardAssets.cardElevation = dp(5f).toFloat()
        this.cardAssets.radius = dp(10f).toFloat()
        val cardLp = ViewUtils.linearMatchWrap()
        cardLp.setMargins(dp(20f), dp(5f), dp(20f), dp(10f))
        scrollContent.addView(this.cardAssets, cardLp)
        this.assetsLyt = LinearLayout(context)
        this.assetsLyt.orientation = LinearLayout.VERTICAL
        this.assetsLyt.setPadding(dp(10f))
        this.cardAssets.addView(this.assetsLyt, ViewUtils.linearMatchWrap())

//        VOICE
        this.voiceTv = TextView(context)
        this.voiceTv.visibility = View.GONE
        scrollContent.addView(this.voiceTv, ViewUtils.linearMatchWrap())
        this.voiceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.voiceTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.voiceTv.isAllCaps = true
        this.voiceTv.text = getString(R.string.voice_service)
        this.voiceTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.voiceLyt = LinearLayout(context)
        this.voiceLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.voiceLyt, ViewUtils.linearMatchWrap())

 //        SMS
        this.smsTv = TextView(context)
        this.smsTv.visibility = View.GONE
        scrollContent.addView(this.smsTv, ViewUtils.linearMatchWrap())
        this.smsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.smsTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.smsTv.isAllCaps = true
        this.smsTv.text = getString(R.string.sms)
        this.smsTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.smsLyt = LinearLayout(context)
        this.smsLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.smsLyt, ViewUtils.linearMatchWrap())

//        DATA
        this.dataTv = TextView(context)
        this.dataTv.visibility = View.GONE
        scrollContent.addView(this.dataTv, ViewUtils.linearMatchWrap())
        this.dataTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.dataTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.dataTv.isAllCaps = true
        this.dataTv.text = getString(R.string.data_service)
        this.dataTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.dataLyt = LinearLayout(context)
        this.dataLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.dataLyt, ViewUtils.linearMatchWrap())

//        OTHERS
        this.othersTv = TextView(context)
        this.othersTv.visibility = View.GONE
        scrollContent.addView(this.othersTv, ViewUtils.linearMatchWrap())
        this.othersTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.othersTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.othersTv.isAllCaps = true
        this.othersTv.text = getString(R.string.others_service)
        this.othersTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.othersLyt = LinearLayout(context)
        this.othersLyt.orientation = LinearLayout.VERTICAL
        scrollContent.addView(this.othersLyt, ViewUtils.linearMatchWrap())

//        SERVICES
        this.servicesTv = TextView(context)
        this.servicesTv.visibility = View.GONE
        scrollContent.addView(this.servicesTv, ViewUtils.linearMatchWrap())
        this.servicesTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.servicesTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.servicesTv.isAllCaps = true
        this.servicesTv.text = getString(R.string.services)
        this.servicesTv.setPadding(dp(25f), dp(10f), 0, 0)
        this.cardServices = CardView(context)
        this.cardServices.visibility = View.GONE
        this.cardServices.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        this.cardServices.cardElevation = dp(5f).toFloat()
        this.cardServices.radius = dp(10f).toFloat()
        scrollContent.addView(this.cardServices, cardLp)
        this.servicesLyt = LinearLayout(context)
        this.servicesLyt.orientation = LinearLayout.VERTICAL
        this.servicesLyt.setPadding(dp(10f))
        this.cardServices.addView(this.servicesLyt, ViewUtils.linearMatchWrap())

        val descTv = TextView(context)
        scrollContent.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.typeface = Theme.font
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.setPadding(dp(20f))

//        ACTION BUTTONS

        this.addOrActiveBtn = Button(requireContext())
        this.addOrActiveBtn.background = ViewUtils.getRect(resources.getColor(R.color.green), dp(8f))
        this.addOrActiveBtn.setTextColor(Color.WHITE)
        this.addOrActiveBtn.visibility = View.GONE
        this.addOrActiveBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.addOrActiveBtn.typeface = Theme.font
        this.addOrActiveBtn.setPadding(dp(20f), 0, dp(20f), 0)
        this.addOrActiveBtn.isAllCaps = false
        val bLp = ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, dp(45f), 0f).apply {
            setMargins(dp(20f), dp(20f),  dp(20f), dp(5f))
        }
        scrollContent.addView(this.addOrActiveBtn, bLp)
        this.addOrActiveBtn.setText(R.string.activate)
        this.addOrActiveBtn.setDebounceClickListener {
            this.activatePlan()
        }

        val frame = FrameLayout(context)
        scrollContent.addView(frame, ViewUtils.linearMatchWrap())
        this.progressBar = ProgressBar(context)
        this.progressBar.visibility = View.GONE
        frame.addView(this.progressBar, ViewUtils.frameWrapWrap(Gravity.CENTER))

        this.viewModel.profilePlan.observe(viewLifecycleOwner) { planObject ->
            if (planObject == null) {
                if (this.canGoBack) {
                    this.activity?.supportFragmentManager?.popBackStack()
                }
            } else {
                this.setTitle(planObject.planDesc?.title ?: planObject.plan.name)
                planObject.let { topInfo.showData(it) }
                cardPlanInfo.setDebounceClickListener {
                    this.pushFragment(ShowInfoFragment.getInstance(planObject))
                }
                descTv.text = planObject.planDesc?.longText
                descTv.visibility = if (planObject.planDesc?.longText.isNullOrEmpty()) View.GONE else View.VISIBLE
                cardPlanInfo.visibility = View.VISIBLE
                this.cardServices.visibility = View.VISIBLE
                this.cardAssets.visibility = View.VISIBLE

                this.showData(planObject)
            }
        }

        arguments?.getString(PLAN_ID)?.let {
            this.viewModel.loadPlan(it).observe(viewLifecycleOwner) {
                when(it.status) {
                    Resource.Status.LOADING -> {
                        this.progressBar.visibility = View.VISIBLE
                    }
                    Resource.Status.ERROR -> {
                        this.progressBar.visibility = View.GONE
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressBar.visibility = View.GONE
                    }
                }
            }
        } ?: run {
            if (this.canGoBack) {
                this.activity?.supportFragmentManager?.popBackStack()
            }
        }
        this.activity?.onBackPressedDispatcher?.addCallback(viewLifecycleOwner, object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
                if (isEnabled && isVisible) {
                    activity?.supportFragmentManager?.popBackStack()
                }
            }
        })
        return rootView.apply { layoutParams = frameMatchMatch() }
    }

    private fun activatePlan() {
        this.viewModel.profilePlan.value?.let { planObject ->
            this.viewModel.activatePlan(planObject.plan.id).observe(viewLifecycleOwner) {
                when(it.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                    }
                }
            }
        }
    }

    private fun createMenu() {
        this.toolbar.menu?.clear()
        this.viewModel.profilePlan.value?.let { plan ->
            if (plan.planInfo.type != Plan.Types.external || NCPCore.instance.currentAccount?.hasPermission(
                    EndpointViewPermission.ViewName.BILLING_EXTERNAL_DELETE_PLAN) == true) {
                this.toolbar.menu?.add(0, MENU_DELETE, 0, R.string.delete)?.setShowAsAction(MenuItem.SHOW_AS_ACTION_NEVER)
            }
            val count = plan.planInfo.services.filter { service -> plan.plan.services.find { it.serviceID == service.id } == null
                    && service.activation == Service.Activation.optional }.size
            if (count > 0) {
                this.toolbar.menu?.add(0, MENU_ADD, 0, R.string.add)
                    ?.setIcon(ViewUtils.getMenuIcon(R.drawable.ic_add, requireContext()))
                    ?.setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS)
            }
        }
    }

    private fun showData(planObject: PlanObject) {
        val interval = planObject.plan.recurringInterval?.let {
            when(it.toLong()) {
                86400_000L -> getString(R.string.daily)
                604800_000L -> getString(R.string.weekly)
                2629743_000L -> getString(R.string.monthly)
                31556926_000L -> getString(R.string.yearly)
                else -> ""
            }
        } ?: ""
        val price = planObject.price
        this.priceItem.nameTv.text = "${price.formattedAmount(planObject.currency)} $interval"
        this.priceItem.visibility = if (price > 0) View.VISIBLE else View.GONE
        planObject.plan.nextActivationAt?.let {
            this.renewItem.visibility = View.VISIBLE
            this.renewItem.nameTv.text = "${getString(R.string.renews)} ${SimpleDateFormat("MMMM dd, yyyy").format(Date(it.toLong()))}"
        } ?: run {
            this.renewItem.visibility = View.GONE
        }
        planObject.plan.activatedAt?.let {
            this.activeItem.visibility = View.VISIBLE
            this.activeItem.nameTv.text = "${getString(R.string.active_since)} ${SimpleDateFormat("MMMM dd, yyyy").format(Date(it.toLong()))}"
        } ?: run {
            this.activeItem.visibility = View.GONE
        }
        this.showActions(planObject.plan.actions)
        this.showServices(planObject.plan.services.filter { it.state != ProfileService.State.removed }.toTypedArray())
        this.showActiveBtn()
        this.createMenu()
    }

    private fun showActions(actions: Array<ProfilePlan.Action>) {
        this.actionsLyt.removeAllViews()
        var checkForActivate = true
        for (action in actions) {
            var actionItem: RowPlanAction? = null
            when {
                action.identity?.info != null -> {
                    if (action.identity.info.state == ProfilePlan.Action.State.pending) {
                        this.viewModel.actionVerify = action.identity.info.required.firstOrNull()
                        actionItem = RowPlanAction(requireContext())
                        actionItem.setOnClickListener {
                            this.viewModel.profilePlan.value?.let { planObject ->
                                val verification = action.identity.info.required.firstOrNull()
                                VerifyActivity.getInstance(this@ProfilePlanViewFragment,
                                    countryISOCode = verification?.country,
                                    verificationLevel = verification?.level,
                                    requestCode = this.verifyReqCode)
                            }
                        }
                    }
                }
                action.asset?.info != null -> {
                    if (action.asset.info.state == ProfilePlan.Action.State.pending) {
                        if (this.viewModel.actionSelectAssets == null) {
                            this.viewModel.actionSelectAssets = true
                        }
                        checkForActivate = false
                        actionItem = RowPlanAction(requireContext())
                        actionItem.setOnClickListener {
                            this.viewModel.profilePlan.value?.let {
                                pushFragment(AssetTypeFragment.getInstance(it))
                            }
                        }
                    }
                }
                action.support?.info != null -> {
                    if (action.support.info.state == ProfilePlan.Action.State.pending) {
                        checkForActivate = false
                        actionItem = RowPlanAction(requireContext())
                        actionItem.setOnClickListener {
                            this.viewModel.profilePlan.value?.let {
                                NCPCore.instance.billingKit.controller.delegate?.openSupportChat(requireActivity() as AppCompatActivity, nexus.appui.R.id.fragment_details)
                            }
                        }
                    }
                }
                action.balance?.info != null -> {
                    if (action.balance.info.state == ProfilePlan.Action.State.pending) {
                        actionItem = RowPlanAction(requireContext())
                        actionItem.setOnClickListener {
                            this.viewModel.profilePlan.value?.let {
                                action.balance.info.profilePaymentSource?.let {
                                    NCPCore.instance.billingKit.controller.delegate?.openTopUpBalance(requireActivity(), it)
                                } ?: run {
                                    ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.topup_balance))
                                }
                            }
                        }
                    }
                }
            }
            actionItem?.let {
                actionItem.showData(action)
                this.actionsLyt.addView(actionItem, ViewUtils.linearMatchWrap().apply { setMargins(dp(20f), dp(5f), dp(20f), dp(10f)) })
            }
        }

        if (arguments?.getBoolean(OPEN_ACTION, false) == true) {
            if (this.viewModel.actionSelectAssets == true) {
                this.viewModel.actionSelectAssets = false
                this.viewModel.profilePlan.value?.let {
                    pushFragment(AssetTypeFragment.getInstance(it))
                }
            } else {
                this.viewModel.actionVerify?.let { verify ->
                    this.viewModel.profilePlan.value?.let {
                        VerifyActivity.getInstance(
                            this@ProfilePlanViewFragment,
                            countryISOCode = verify.country,
                            verificationLevel = verify.level,
                            requestCode = this.verifyReqCode
                        )
                        this.viewModel.actionVerify = null
                        arguments?.putBoolean(OPEN_ACTION, false)
                    }
                }
            }
        }


        this.actionsLyt.visibility = if (this.actionsLyt.childCount == 0) View.GONE else View.VISIBLE
        if (this.viewModel.profilePlan.value?.plan?.state == ProfilePlan.State.pending && checkForActivate) {
            CoroutineScope(Dispatchers.IO).launch {
                var haveMoney = true
                actions.find { action -> action.balance?.info?.state == ProfilePlan.Action.State.pending }?.let {
                    haveMoney = NCPCore.instance.billingKit.controller.delegate?.checkBalance(it.balance?.info?.required ?: 0 ) ?: false
                }
                var haveIdentity = false
                actions.find { action -> action.identity?.info?.state == ProfilePlan.Action.State.pending }?.let {
                    haveIdentity = it.identity?.info?.required?.firstOrNull()?.let { verify ->
                        NCPCore.instance.passportKit.controller.checkLevelIdentity(verify.country, verify.level)
                    } ?: run {
                        false
                    }
                }
                if (haveMoney && haveIdentity) {
                    withContext(Dispatchers.Main) {
                        this@ProfilePlanViewFragment.activatePlan()
                    }
                }
            }
        }
    }

    private fun showServices(services: Array<ProfileService>) {
        this.viewModel.profilePlan.value?.let { planObject ->
            this.servicesLyt.removeAllViews()
            this.assetsLyt.removeAllViews()
            this.voiceLyt.removeAllViews()
            this.smsLyt.removeAllViews()
            this.dataLyt.removeAllViews()
            this.othersLyt.removeAllViews()
            this.forwardItem.visibility = View.GONE
            this.voiceMailItem.visibility = View.GONE

            for (service in services.filter { it.state != ProfileService.State.completed }) {
                if (service.type == Service.Types.`package` || service.type == Service.Types.route) {
                    val packageItem = RowPlanPackage(requireContext())
                    packageItem.showData(Pair(service, planObject.serviceDesc(service.serviceID)), planObject.currency, service.type == Service.Types.`package`, false)
                    var routes: Array<Route> = emptyArray()
                    CoroutineScope(Dispatchers.IO).launch {
                        this@ProfilePlanViewFragment.viewModel.getPlanRoutes(planId = planObject.plan.planID, service.serviceID).let { r ->
                            withContext(Dispatchers.Main) {
                                routes = r
                                packageItem.showRoutes(routes) {
                                    pushFragment(PlanRoutesFragment.getInstance(routes, planObject.currency, planObject.serviceDesc(service.serviceID)))
                                }
                            }
                        }
                    }
                    val cardLp = ViewUtils.linearMatchWrap().apply { setMargins(dp(20f), dp(5f), dp(20f), dp(7f)) }
                    packageItem.setDebounceClickListener {
                        this.pushFragment(ShowInfoFragment.getInstance(planObject, service, routes))
                    }
                    when(service.category) {
                        Service.Category.voice -> {
                            this.voiceLyt.addView(packageItem, cardLp)
                        }
                        Service.Category.sms -> {
                            this.smsLyt.addView(packageItem, cardLp)
                        }
                        Service.Category.data -> {
                            this.dataLyt.addView(packageItem, cardLp)
                        }
                        else -> {
                            this.othersLyt.addView(packageItem, cardLp)
                        }
                    }
                    packageItem.activeBtn.setDebounceClickListener {
                        planObject.id.let { id ->
                            this.activeService(id, service.id)
                        }
                    }
                    packageItem.removeBtn.setDebounceClickListener {
                        planObject.id.let { id ->
                            this.removeService(id, service.id)
                        }
                    }
                } else {
                    when (service.type) {
                        Service.Types.simcard -> {
                            this.forwardItem.visibility = View.VISIBLE
                            this.forwardItem.forwardSwitch.setOnCheckedChangeListener(null)
                            this.forwardItem.forwardSwitch.isChecked = service.simcard?.forwarded == true
                            this.forwardItem.forwardSwitch.setOnCheckedChangeListener { _, isChecked ->
                                if (this.updateForward) {
                                    this.viewModel.setForwarding(isChecked, service.id).observe(viewLifecycleOwner) {
                                        when (it.status) {
                                            Resource.Status.LOADING -> {
                                                this.progressDialog.show()
                                            }
                                            Resource.Status.SUCCESS -> {
                                                this.progressDialog.dismiss()
                                            }
                                            Resource.Status.ERROR -> {
                                                this.progressDialog.dismiss()
                                                ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat), false)
                                                this.updateForward = false
                                                this.forwardItem.forwardSwitch.isChecked = service.simcard?.forwarded == true
                                            }
                                        }
                                    }
                                } else {
                                    this.updateForward = true
                                }
                            }
                        }
                        Service.Types.voicemail -> {
                            this.voiceMailItem.visibility = View.VISIBLE
                            this.voiceMailItem.valueTv.text = service.voicemail?.name
                            this.voiceMailItem.setDebounceClickListener {
                                this.pushFragment(VoiceMailFragment.getInstance(planObject.id, service))
                            }
                        }
                        Service.Types.asset -> {
                            val serviceItem = RowNgn(requireContext())
                            serviceItem.showData(Pair(service, planObject.serviceDesc(service.serviceID)))
                            serviceItem.arr.visibility = View.GONE
                            serviceItem.background = null
                            service.asset?.let { serviceItem.nameTv.text = it.number.toDisplayNumber() }
                            this.assetsLyt.addView(serviceItem, ViewUtils.linearMatchWrap())
                        }
                        else -> {
                            val serviceItem = RowOption(requireContext())
                            serviceItem.showData(planObject.serviceDesc(service.serviceID))
                            serviceItem.arr.visibility = View.GONE
                            serviceItem.background = null
                            this.servicesLyt.addView(serviceItem, ViewUtils.linearMatchWrap())
                            serviceItem.setDebounceClickListener {
                                this.pushFragment(ShowInfoFragment.getInstance(planObject, service))
                            }
                        }
                    }
                }
            }
            checkEmptyLyt(this.servicesLyt, this.servicesTv)
            checkEmptyLyt(this.assetsLyt, this.assetsTv)
            checkEmptyLyt(this.voiceLyt, this.voiceTv)
            checkEmptyLyt(this.smsLyt, this.smsTv)
            checkEmptyLyt(this.dataLyt, this.dataTv)
            checkEmptyLyt(this.othersLyt, this.othersTv)
            this.forwardLine.visibility = if (this.forwardItem.visibility == View.VISIBLE || this.voiceMailItem.visibility == View.VISIBLE) View.VISIBLE else View.GONE
        }
    }


    private fun activeService(profilePlanId: String, serviceId: String) {
        val b = androidx.appcompat.app.AlertDialog.Builder(requireContext(), Theme.dialogTheme)
        b.setMessage(getString(R.string.want_activate_service))
        b.setNegativeButton(nexus.appui.R.string.cancel) { dialog, _ -> dialog.cancel() }
        b.setPositiveButton(nexus.appui.R.string.ok) { dialog, _ ->
            this.viewModel.activateService(profilePlanId, serviceId).observe(viewLifecycleOwner) { result ->
                when(result.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                        requireActivity().supportFragmentManager.popBackStack()
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                }
            }
        }
        b.create().show()
    }

    private fun removeService(profilePlanId: String, serviceId: String) {
        val b = androidx.appcompat.app.AlertDialog.Builder(requireContext(), Theme.dialogTheme)
        b.setMessage(getString(R.string.want_delete_service))
        b.setNegativeButton(nexus.appui.R.string.cancel) { dialog, _ -> dialog.cancel() }
        b.setPositiveButton(nexus.appui.R.string.ok) { dialog, _ ->
            this.viewModel.removeService(profilePlanId, serviceId).observe(viewLifecycleOwner) { result ->
                when(result.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                        requireActivity().supportFragmentManager.popBackStack()
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                }
            }
        }
        b.create().show()
    }

    private fun showActiveBtn() {
        this.viewModel.profilePlan.value?.let { planObject ->
            val showActive = planObject.plan.actions.find { it.identity?.info?.state == ProfilePlan.Action.State.pending
                    || it.asset?.info?.state == ProfilePlan.Action.State.pending
                    || it.support?.info?.state == ProfilePlan.Action.State.pending } == null
            if (showActive && (planObject.plan.state == ProfilePlan.State.pending || planObject.plan.state == ProfilePlan.State.inactive)) {
                this.addOrActiveBtn.visibility = View.VISIBLE
            } else {
                this.addOrActiveBtn.visibility = View.GONE
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when(requestCode) {
            this.verifyReqCode -> {
                if (this.view != null) {
                    this.viewModel.updateProfilePlan().observe(viewLifecycleOwner) {
                        when(it.status) {
                            Resource.Status.LOADING -> { this.progressDialog.show() }
                            Resource.Status.ERROR, Resource.Status.SUCCESS -> { this.progressDialog.dismiss() }
                        }
                    }
                }
            }
        }
    }

    private fun showDeleteDialog(context: Context, plan: PlanObject) {
        val b = AlertDialog.Builder(context)
        var dialog: Dialog? = null
        val view = LinearLayout(context)
        view.setPadding(dp(20f))
        view.orientation = LinearLayout.VERTICAL
        view.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))

        val titleTv = TextView(context)
        titleTv.text = getString(R.string.disable_plan)
        titleTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 20f)
        titleTv.setTypeface(Theme.font, Typeface.BOLD)
        titleTv.setPadding(0, 0, 0, dp(5f))
        titleTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        view.addView(titleTv, ViewUtils.linearMatchWrap())
        val descTv = TextView(context)
        descTv.text = getString(R.string.disabling_plan)
        descTv.setPadding(0, 0, 0, dp(5f))
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        descTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        descTv.typeface = Theme.font
        view.addView(descTv, ViewUtils.linearMatchWrap())
        val infoView = RowPlanTopInfo(context)
        infoView.showData(plan)
        view.addView(infoView, ViewUtils.linearMatchWrap())
        for (service in plan.plan.services) {
            when (service.type) {
                Service.Types.asset -> {
                    val numTv = TextView(context)
                    numTv.text = getString(R.string.phone_number)
                    numTv.setPadding(0, dp(5f), 0, 0)
                    numTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
                    numTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
                    numTv.typeface = Theme.font
                    view.addView(numTv, ViewUtils.linearMatchWrap())

                    val numberView = RowPlanOption(context)
                    numberView.nameTv.text = service.asset?.number?.toDisplayNumber()
                    plan.findServicePrice(service.serviceID)?.let {
                        numberView.valueTv.text = it.formattedAmount(plan.currency)
                    }
                    numberView.icon.setImageResource(R.drawable.ic_plan_number)
                    numberView.setPadding(0, dp(5f), 0, dp(5f))
                    view.addView(numberView, ViewUtils.linearMatchWrap())
                }
                else -> {}
            }
        }

        val planDescTv = TextView(context)
        planDescTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        planDescTv.text = getString(R.string.delete_plan_desc)
        planDescTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        planDescTv.typeface = Theme.font
        view.addView(planDescTv, ViewUtils.linearMatchWrap())

        val bottomLyt = FrameLayout(context)
        bottomLyt.setPadding(0, dp(30f), 0, dp(5f))
        view.addView(bottomLyt, ViewUtils.linearMatchWrap())

        val deleteBtn = Button(context)
        deleteBtn.background = ViewUtils.getRect(Color.RED, dp(10f))
        deleteBtn.text = getString(R.string.yes_delete)
        deleteBtn.setTextColor(Color.WHITE)
        deleteBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        deleteBtn.typeface = Theme.font
        deleteBtn.setPadding(dp(20f), 0, dp(20f), 0)
        deleteBtn.isAllCaps = false
        bottomLyt.addView(deleteBtn, ViewUtils.frameNoDp(LinearLayout.LayoutParams.WRAP_CONTENT, dp(40f), Gravity.START))
        deleteBtn.setOnClickListener {
            this.canGoBack = false
            this.viewModel.deletePlan(plan.plan.id).observe(viewLifecycleOwner) {
                when(it.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        dialog?.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                        dialog?.dismiss()
                        requireActivity().supportFragmentManager.popBackStack()
                    }
                }
            }
        }

        val backBtn = Button(context)
        backBtn.background = ViewUtils.getRect(Color.TRANSPARENT, dp(10f), context.resources.getColor(R.color.green))
        backBtn.text = getString(R.string.back)
        backBtn.setTextColor(context.resources.getColor(R.color.green))
        backBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        backBtn.typeface = Theme.font
        backBtn.setPadding(dp(20f), 0, dp(20f), 0)
        backBtn.isAllCaps = false
        bottomLyt.addView(backBtn, ViewUtils.frameNoDp(LinearLayout.LayoutParams.WRAP_CONTENT, dp(40f), Gravity.END))
        backBtn.setOnClickListener {
            dialog?.dismiss()
        }

        b.setView(view)
        dialog = b.create()
        dialog?.setCancelable(false)
        dialog?.show()
    }
}