package nexus.billing.ui.profile

import android.app.Activity
import android.graphics.Color
import android.graphics.Typeface
import android.text.InputFilter
import android.text.InputType
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.ScrollView
import android.widget.TextView
import android.widget.Toast
import androidx.activity.OnBackPressedCallback
import androidx.core.view.setPadding
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.R
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseDialog
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.billingKit
import nexus.billing.methods.common.Provider
import nexus.billing.methods.requests.profileplan.external.ProfilePlanRegister
import nexus.billing.methods.requests.profileplan.external.ProfilePlanValidate
import nexus.billing.models.PlanObject
import nexus.billing.ui.btnSelector
import nexus.billing.ui.rows.RowProvider
import nexus.core.NCPCore

class RegisterProfilePlanDialog(val activity: Activity): BaseDialog() {

    companion object {
        fun getInstance(activity: Activity): RegisterProfilePlanDialog {
            return RegisterProfilePlanDialog(activity)
        }
    }

    interface RegisterListener {
        fun registered(profilePlan: PlanObject?)
    }

    private val progressDialog = ViewUtils.getProgressDialog(this.activity)
    private var bottomDialog: BottomSheetDialog? = null
    private var loginEdt: EditText
    private var passwordEdt: EditText
    private var phoneNumberEdt: EditText
    private var countryCodeTv: TextView
    private var continueBtn: Button
    private var loginLyt: LinearLayout
    private var numberLyt: LinearLayout
    private var listener: RegisterListener? = null
    private var progressBar: ProgressBar? = null
    private var provider: Provider? = null

    init {
        val p10 = dp(10f)
        val p15 = dp(15f)
        val font = Theme.font

        this.bottomDialog = BottomSheetDialog(this.activity, R.style.SheetDialog)
        val contentLyt = LinearLayout(this.activity)
        contentLyt.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), p15, p15, 0, 0)
        contentLyt.orientation = LinearLayout.VERTICAL
        contentLyt.setPadding(dp(20f))

        val closeBtn = ImageView(this.activity)
        closeBtn.setImageResource(R.drawable.ic_close)
        closeBtn.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        closeBtn.setPadding(dp(3f))
        closeBtn.background = ViewUtils.getCircle(Theme.addTrans(Color.GRAY, 80))
        val cLp = ViewUtils.linearParams(27, 27)
        cLp.gravity = Gravity.END
        contentLyt.addView(closeBtn, cLp)
        closeBtn.setOnClickListener {
            this.close(null)
        }

        val tvTitle = TextView(this.activity)
        contentLyt.addView(tvTitle, ViewUtils.linearMatchWrap())
        tvTitle.setTypeface(font, Typeface.BOLD)
        tvTitle.text = activity.getString(nexus.billing.R.string.register_profile_plan)
        tvTitle.gravity = Gravity.CENTER
        tvTitle.setPadding(0, p10, 0, p15)
        tvTitle.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        tvTitle.setTextSize(TypedValue.COMPLEX_UNIT_SP, 20f)

        val tvDesc = TextView(this.activity)
        contentLyt.addView(tvDesc, ViewUtils.linearMatchWrap())
        tvDesc.typeface = font
        tvDesc.text = activity.getString(nexus.billing.R.string.select_provider_and_register_your_profile_plan)
        tvDesc.gravity = Gravity.CENTER
        tvDesc.setPadding(0, p10, 0, p15)
        tvDesc.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        tvDesc.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)

        val scroll = ScrollView(this.activity)
        scroll.isFillViewport = false
        contentLyt.addView(scroll, ViewUtils.linearMatchMatch())

        val vertContent = LinearLayout(this.activity)
        vertContent.orientation = LinearLayout.VERTICAL
        scroll.addView(vertContent, ViewUtils.frameMatchMatch())

        val tvSources = TextView(this.activity)
        vertContent.addView(tvSources, ViewUtils.linearMatchWrap())
        tvSources.setTypeface(font, Typeface.BOLD)
        tvSources.text = activity.getString(nexus.billing.R.string.select_provider)
        tvSources.setPadding(0, p10, 0, dp(20f))
        tvSources.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        tvSources.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)

        val frame = FrameLayout(this.activity)
        vertContent.addView(frame, ViewUtils.linearMatchWrap())

        val providersLyt = LinearLayout(this.activity)
        providersLyt.orientation = LinearLayout.VERTICAL
        providersLyt.visibility = View.GONE
        providersLyt.background = ViewUtils.getRect(Theme.addTrans(Theme.getColor(Theme.key_dividerGrey), 50), p10)
        frame.addView(providersLyt, ViewUtils.frameMatchWrap())
        loadProviders(tvSources, providersLyt)

        //        START SIGN IN LAYOUT
        this.loginLyt = LinearLayout(this.activity)
        this.loginLyt.orientation = LinearLayout.VERTICAL
        this.loginLyt.visibility = View.GONE
        vertContent.addView(this.loginLyt, ViewUtils.linearMatchWrap())
        this.loginLyt.setPadding(0, 0, 0, dp(20f))
        this.loginEdt = EditText(this.activity)
        this.loginLyt.addView(this.loginEdt, LinearLayout.LayoutParams.MATCH_PARENT, dp(48f))
        this.loginEdt.setPadding(dp(10f), 0, 0, 0)
        this.loginEdt.setHint(nexus.billing.R.string.user_id)
        this.loginEdt.setHintTextColor(Theme.getColor(Theme.key_textSecondary))
        this.loginEdt.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.loginEdt.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.loginEdt.background = ViewUtils.getRect(Color.TRANSPARENT, dp(10f), Theme.getColor(Theme.key_primaryColor))
        this.loginEdt.filters = arrayOf<InputFilter>(InputFilter.LengthFilter(50))
        this.loginEdt.isSingleLine = true
        this.loginEdt.typeface = Theme.font
        this.loginEdt.inputType = InputType.TYPE_CLASS_NUMBER

        val loginDv = View(this.activity)
        this.loginLyt.addView(loginDv, LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, 1))
        loginDv.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))

        this.passwordEdt = EditText(this.activity)
        this.loginLyt.addView(this.passwordEdt, ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, dp(48f), 0f).apply {
            topMargin = dp(20f)
        })
        this.passwordEdt.setHint(nexus.billing.R.string.password)
        this.passwordEdt.background = ViewUtils.getRect(Color.TRANSPARENT, dp(10f), Theme.getColor(Theme.key_primaryColor))
        this.passwordEdt.setHintTextColor(Theme.getColor(Theme.key_textSecondary))
        this.passwordEdt.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.passwordEdt.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.passwordEdt.filters = arrayOf<InputFilter>(InputFilter.LengthFilter(50))
        this.passwordEdt.isSingleLine = true
        this.passwordEdt.setPadding(dp(10f), 0, 0, 0)
        this.passwordEdt.typeface = Theme.font
        this.passwordEdt.inputType = InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_VARIATION_PASSWORD

        this.numberLyt = LinearLayout(this.activity)
        vertContent.addView(this.numberLyt, LinearLayout.LayoutParams.MATCH_PARENT, dp(48f))
        this.numberLyt.orientation = LinearLayout.HORIZONTAL
        this.numberLyt.visibility = View.GONE
        this.countryCodeTv = TextView(this.activity)
        this.numberLyt.addView(this.countryCodeTv, ViewUtils.linearWrapWrap())
        this.countryCodeTv.isFocusable = true
        this.countryCodeTv.isClickable = true
        this.countryCodeTv.setPadding(0, 0, p10, 0)
        this.countryCodeTv.isFocusableInTouchMode = true
        this.countryCodeTv.gravity = Gravity.CENTER
        this.countryCodeTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.countryCodeTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.countryCodeTv.typeface = Theme.font
        this.phoneNumberEdt = EditText(this.activity)
        this.numberLyt.addView(this.phoneNumberEdt, ViewUtils.linearParams(0, LinearLayout.LayoutParams.MATCH_PARENT, 1f))
        this.phoneNumberEdt.setHint(nexus.billing.R.string.phone_number)
        this.phoneNumberEdt.setPadding(dp(10f), 0, 0, 0)
        this.phoneNumberEdt.setHintTextColor(Theme.getColor(Theme.key_textSecondary))
        this.phoneNumberEdt.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.phoneNumberEdt.background = ViewUtils.getRect(Color.TRANSPARENT, dp(10f), Theme.getColor(Theme.key_primaryColor))
        this.phoneNumberEdt.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.phoneNumberEdt.isSingleLine = true
        this.phoneNumberEdt.setEms(9)
        this.phoneNumberEdt.typeface = Theme.font
        this.phoneNumberEdt.setSelectAllOnFocus(true)
        this.phoneNumberEdt.inputType = InputType.TYPE_CLASS_PHONE
        this.phoneNumberEdt.filters = arrayOf<InputFilter>(InputFilter.LengthFilter(50))

        this.continueBtn = Button(this.activity)
        vertContent.addView(this.continueBtn, ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, dp(47f), 0f).apply {
            topMargin = dp(20f)
        })
        this.continueBtn.isAllCaps = false
        this.continueBtn.visibility = View.GONE
        this.continueBtn.gravity = Gravity.CENTER
        this.continueBtn.setText(nexus.billing.R.string.continuee)
        this.continueBtn.setTextColor(Color.WHITE)
        this.continueBtn.background = btnSelector(focusedColor = Theme.addTrans(Theme.getColor(Theme.key_primaryColor), 90))
        this.continueBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText - 1)
        this.continueBtn.setDebounceClickListener {
            if (this.provider?.credentialType == Provider.CredentialType.phoneNumber) {
                val number = this.phoneNumberEdt.text.toString()
            } else {
                val login = this.loginEdt.text.toString()
                val password = this.passwordEdt.text.toString()
                if (login.isEmpty() || password.isEmpty()) {
                    Toast.makeText(this.activity, nexus.billing.R.string.login_and_password_is_required, Toast.LENGTH_SHORT).show()
                    return@setDebounceClickListener
                }
                this.registerLogin(login, password)
            }
        }

        this.progressBar = ProgressBar(this.activity)
        val pLp = ViewUtils.frameWrapWrap()
        pLp.gravity = Gravity.CENTER
        pLp.topMargin = p15
        pLp.bottomMargin = p15
        frame.addView(progressBar, pLp)
        this.bottomDialog?.setContentView(contentLyt, ViewUtils.frameNoDp(FrameLayout.LayoutParams.MATCH_PARENT, ViewUtils.displaySize(activity).y, Gravity.NO_GRAVITY))

        this.bottomDialog?.onBackPressedDispatcher?.addCallback(object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
                if (this@RegisterProfilePlanDialog.provider != null) {
                    this@RegisterProfilePlanDialog.provider = null
                    tvTitle.setText(nexus.billing.R.string.select_provider)
                    this@RegisterProfilePlanDialog.numberLyt.visibility = View.GONE
                    this@RegisterProfilePlanDialog.loginLyt.visibility = View.GONE
                    this@RegisterProfilePlanDialog.continueBtn.visibility = View.GONE
                    providersLyt.visibility = View.VISIBLE
                } else {
                    this@RegisterProfilePlanDialog.close(null)
                }
            }
        })
    }

    private fun registerLogin(login: String, password: String) {
        this.progressDialog.show()
        val providerId = this.provider?.id ?: return
        CoroutineScope(Dispatchers.IO).launch {
            try {
                val validate = NCPCore.instance.billingKit.controller.requestProfilePlanValidate(providerId, ProfilePlanValidate.Login(login, password), null)
                when (validate?.status) {
                ProfilePlanValidate.ValidateStatus.valid -> {
                    val result = NCPCore.instance.billingKit.controller.requestProfilePlanRegister(providerId, ProfilePlanRegister.Login(login, password), null)
                    withContext(Dispatchers.Main) {
                        progressDialog.dismiss()
                        if (result != null) {
                            this@RegisterProfilePlanDialog.close(result)
                        } else {
                            ViewUtils.showErrorDialog(activity, null, activity.getString(nexus.billing.R.string.request_error_repeat))
                        }
                    }
                }
                ProfilePlanValidate.ValidateStatus.incorrectPassword -> {
                    withContext(Dispatchers.Main) {
                        progressDialog.dismiss()
                        ViewUtils.showErrorDialog(activity, null, "Incorrect password or login")
                    }
                }
                ProfilePlanValidate.ValidateStatus.passwordNotYetSet -> {
                    withContext(Dispatchers.Main) {
                        progressDialog.dismiss()
                        ViewUtils.showErrorDialog(activity, null, "Password not yet set")
                    }
                }
                else -> {
                    withContext(Dispatchers.Main) {
                        progressDialog.dismiss()
                        ViewUtils.showErrorDialog(activity, null, activity.getString(nexus.billing.R.string.request_error_repeat))
                    }
                }
            }
            } catch (e: Exception) {
                withContext(Dispatchers.Main) {
                    progressDialog.dismiss()
                    ViewUtils.showErrorDialog(activity, null, activity.getString(nexus.billing.R.string.request_error_repeat))
                }
            }
        }
    }
    private fun registerPhone(phone: String, referenceId: String) {
        val providerId = this.provider?.id ?: return
        CoroutineScope(Dispatchers.IO).launch {
//            NCPCore.instance.billingKit.controller.requestProfilePlanRegister(providerId, null, ProfilePlanRegister.Validate(phone, referenceId))
        }
    }

    fun show(listener: RegisterListener?) {
        super.show()
        this.listener = listener
        this.bottomDialog?.behavior?.state = BottomSheetBehavior.STATE_EXPANDED
        this.bottomDialog?.setCancelable(false)
        this.bottomDialog?.show()
        this.bottomDialog?.window?.navigationBarColor = Theme.getColor(Theme.key_whiteBackground)
    }

    private fun loadProviders(tv: TextView, content: LinearLayout) {
        CoroutineScope(Dispatchers.IO).launch {
            val providers = NCPCore.instance.billingKit.controller.requestProfilePlanProviders()
            withContext(Dispatchers.Main) {
                this@RegisterProfilePlanDialog.progressBar?.visibility = View.GONE
                if (this@RegisterProfilePlanDialog.bottomDialog?.isShowing == true) {
                    for (provider in providers) {
                        val item = RowProvider(activity)
                        item.showProvider(provider)
                        content.addView(item, ViewUtils.linearMatchWrap().apply {
                            setMargins(0, dp(3f), 0, dp(7f))
                        })
                        item.setDebounceClickListener {
                            this@RegisterProfilePlanDialog.provider = provider.first
                            content.visibility = View.GONE
                            if (provider.first.credentialType == Provider.CredentialType.phoneNumber) {
                                tv.setText(nexus.billing.R.string.enter_your_phone_number)
                                this@RegisterProfilePlanDialog.numberLyt.visibility = View.VISIBLE
                                this@RegisterProfilePlanDialog.loginLyt.visibility = View.GONE
                                this@RegisterProfilePlanDialog.countryCodeTv.text = provider.first.country
                            } else if (provider.first.credentialType == Provider.CredentialType.credentials) {
                                tv.setText(nexus.billing.R.string.enter_your_login_and_password)
                                this@RegisterProfilePlanDialog.loginLyt.visibility = View.VISIBLE
                                this@RegisterProfilePlanDialog.numberLyt.visibility = View.GONE
                            }
                            this@RegisterProfilePlanDialog.continueBtn.visibility = View.VISIBLE
                        }
                    }
                    content.visibility = View.VISIBLE
                }
            }
        }
    }

    fun close(plan: PlanObject?) {
        this.listener?.registered(plan)
        this.bottomDialog?.dismiss()
    }
}