package nexus.billing.ui.profile.voicemail

import android.content.DialogInterface
import android.graphics.PorterDuff
import android.media.AudioFormat
import android.media.MediaPlayer
import android.media.MediaRecorder
import android.os.Bundle
import android.os.CountDownTimer
import android.text.Editable
import android.text.InputFilter
import android.text.InputType
import android.text.TextWatcher
import android.transition.TransitionInflater
import android.util.Log
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.ScrollView
import android.widget.TextView
import androidx.activity.OnBackPressedCallback
import androidx.appcompat.app.AlertDialog
import androidx.cardview.widget.CardView
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import io.reactivex.rxjava3.disposables.CompositeDisposable
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.ui.SelectMenuItem
import nexus.appui.ui.SelectMenuSheet
import nexus.appui.utils.AppPermissionUtils
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.recorder.AudioChunk
import nexus.appui.utils.recorder.AudioRecordConfig
import nexus.appui.utils.recorder.PullTransport
import nexus.appui.utils.recorder.PullableSource
import nexus.appui.utils.recorder.WavRecorder
import nexus.billing.R
import nexus.billing.methods.profile.ProfileService
import nexus.billing.methods.requests.voicemail.GetVoiceMail
import nexus.billing.ui.rows.RowVoiceMail
import nexus.core.NCPCore
import nexus.core.controllers.FileController
import nexus.core.controllers.FileTask
import nexus.core.controllers.account
import nexus.core.controllers.file
import nexus.core.util.Logger
import java.io.File
import java.io.IOException
import java.util.Locale
import java.util.Timer
import java.util.TimerTask
import java.util.UUID
import kotlin.math.min

class VoiceMailFragment: BaseFragment() {

    companion object {
        const val TAG = "VoiceMailFragment"
        fun getInstance(profilePlanId: String, profileService: ProfileService): VoiceMailFragment {
            val fragment = VoiceMailFragment()
            fragment.profileService = profileService
            fragment.profilePlanId = profilePlanId
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }

    private val AUDIO_PERMISSION = 55
    private val STORAGE_PERMISSION = 66

    private val viewModel: VoiceMailViewModel by viewModels()
    private var profileService: ProfileService? = null
    private var voiceMailResult: GetVoiceMail.Response? = null
    private var profilePlanId: String? = null

    private lateinit var defaultContent: LinearLayout
    private lateinit var customContent: LinearLayout
    private lateinit var tvRecord: TextView
    private lateinit var tvStop: TextView
    private lateinit var tvCurrentTime: TextView
    private lateinit var tvMaxTime: TextView
    private lateinit var defaultCard: CardView
    private lateinit var customCard: CardView
    private lateinit var recordProgress: ProgressBar

    private var minDuration = 5
    private var maxDuration = 30
    private var uploadingUrl: String? = null
    private var downloadingUrl: String? = null
    private var timer: Timer? = null
    private var startTime: Long = 0
    private var isRecording = false
    private var recorder: WavRecorder.Recorder? = null
    private var recordFile: File? = null
    private var enteredName = ""
    private var playTimer: CountDownTimer? = null
    private var player: MediaPlayer? = null
    private var lastPlayIcon: ImageView? = null
    private val bag = CompositeDisposable()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val p10 = dp(10f)
        val p5 = dp(5f)

        val rootView = LinearLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.orientation = LinearLayout.VERTICAL
        rootView.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))

        this.toolbar = MaterialToolbar(context)
        rootView.addView(this.toolbar, ViewUtils.linearMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.setTitle(R.string.voice_mail)
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val scroll = ScrollView(requireContext())
        scroll.isFillViewport = true
        rootView.addView(scroll, ViewUtils.linearMatchMatch())

        val mainLinear = LinearLayout(requireContext())
        scroll.addView(mainLinear, ViewUtils.scrollMatchMatch())
        mainLinear.orientation = LinearLayout.VERTICAL
        mainLinear.setPadding(0, dp(10f), 0, 0)

        // DEFAULT CARD
        this.defaultCard = CardView(requireContext())
        val cardLp = ViewUtils.linearMatchWrap()
        cardLp.setMargins(ViewUtils.cardMargin, p10, ViewUtils.cardMargin, p10)
        mainLinear.addView(this.defaultCard, cardLp)
        this.defaultCard.visibility = View.GONE
        this.defaultCard.cardElevation = dp(1f).toFloat()
        this.defaultCard.radius = ViewUtils.cardRadius
        this.defaultCard.setCardBackgroundColor(theme.getColor(Theme.key_blockView))

        val defaultLyt = LinearLayout(requireContext())
        this.defaultCard.addView(defaultLyt, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        defaultLyt.orientation = LinearLayout.VERTICAL
        val defTv = TextView(requireContext())
        defaultLyt.addView(defTv, ViewUtils.linearMatchWrap())
        defTv.typeface = Theme.font
        defTv.setText(R.string.default_greetings)
        defTv.setTextColor(theme.getColor(Theme.key_primaryColor))
        defTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        defTv.setPadding(dp(20f), dp(5f), p10, dp(2f))
        this.defaultContent = LinearLayout(requireContext())
        this.defaultContent.setPadding(dp(20f), 0, dp(20f), 0)
        defaultLyt.addView(this.defaultContent, ViewUtils.linearMatchWrap())
        this.defaultContent.orientation = LinearLayout.VERTICAL

        // DEFAULT CARD
        this.customCard = CardView(requireContext())
        mainLinear.addView(this.customCard, cardLp)
        this.customCard.visibility = View.GONE
        this.customCard.cardElevation = dp(1f).toFloat()
        this.customCard.radius = ViewUtils.cardRadius
        this.customCard.setCardBackgroundColor(theme.getColor(Theme.key_blockView))

        val customLyt = LinearLayout(requireContext())
        this.customCard.addView(customLyt, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        customLyt.orientation = LinearLayout.VERTICAL
        val customTv = TextView(requireContext())
        customLyt.addView(customTv, ViewUtils.linearMatchWrap())
        customTv.typeface = Theme.font
        customTv.setText(R.string.custom_greetings)
        customTv.setTextColor(theme.getColor(Theme.key_primaryColor))
        customTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        customTv.setPadding(dp(20f), dp(5f), p10, dp(2f))
        this.customContent = LinearLayout(requireContext())
        this.customContent.setPadding(dp(20f), 0, dp(20f), 0)
        customLyt.addView(this.customContent, ViewUtils.linearMatchWrap())
        this.customContent.orientation = LinearLayout.VERTICAL

        // RECORD CARD
        val recordCard = CardView(requireContext())
        mainLinear.addView(recordCard, cardLp)
        recordCard.cardElevation = dp(1f).toFloat()
        recordCard.radius = ViewUtils.cardRadius
        recordCard.setCardBackgroundColor(theme.getColor(Theme.key_blockView))
        val recordLyt = LinearLayout(requireContext())
        recordCard.addView(recordLyt, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        recordLyt.orientation = LinearLayout.VERTICAL

        val recTv = TextView(requireContext())
        recordLyt.addView(recTv, ViewUtils.linearMatchWrap())
        recTv.typeface = Theme.font
        recTv.setText(R.string.listen_record)
        recTv.setTextColor(theme.getColor(Theme.key_primaryColor))
        recTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        recTv.setPadding(dp(20f), p5, p10, 0)
        val progressLyt = LinearLayout(requireContext())
        recordLyt.addView(progressLyt, ViewUtils.linearMatchWrap())
        progressLyt.orientation = LinearLayout.HORIZONTAL
        progressLyt.gravity = Gravity.CENTER_VERTICAL
        progressLyt.setPadding(dp(20f), p10, dp(20f), p10)

        this.tvCurrentTime = TextView(requireContext())
        progressLyt.addView(this.tvCurrentTime, ViewUtils.frameWrapWrap())
        this.tvCurrentTime.typeface = Theme.font
        this.tvCurrentTime.setTextColor(theme.getColor(Theme.key_greyColor))
        this.tvCurrentTime.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.tvCurrentTime.text = "00:00"
        this.recordProgress = ProgressBar(requireContext(), null, android.R.attr.progressBarStyleHorizontal)
        this.recordProgress.max = 100
        val rLp = ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f)
        rLp.setMargins(p10, 0, p10, 0)
        progressLyt.addView(this.recordProgress, rLp)

        this.tvMaxTime = TextView(requireContext())
        progressLyt.addView(this.tvMaxTime, ViewUtils.frameWrapWrap())
        this.tvMaxTime.typeface = Theme.font
        this.tvMaxTime.setTextColor(theme.getColor(Theme.key_greyColor))
        this.tvMaxTime.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.tvMaxTime.text = "00:00"
        val line = View(requireContext())
        val lineLp = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, 1)
        recordLyt.addView(line, lineLp)
        line.setBackgroundColor(theme.getColor(Theme.key_dividerGrey))
        val btnLyt = FrameLayout(requireContext())
        recordLyt.addView(btnLyt, ViewUtils.linearMatchWrap())
        this.tvStop = TextView(requireContext())
        btnLyt.addView(this.tvStop, ViewUtils.frameWrapWrap())
        this.tvStop.typeface = Theme.font
        this.tvStop.setTextColor(theme.getColor(Theme.key_greyColor))
        this.tvStop.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.tvStop.setPadding(dp(20f), p10, dp(20f), p10)
        this.tvStop.setBackgroundResource(ViewUtils.selectableResId(requireContext()))
        this.tvStop.setOnClickListener {  }

        this.tvRecord = TextView(requireContext())
        val trLp = ViewUtils.frameWrapWrap()
        trLp.gravity = Gravity.END
        btnLyt.addView(this.tvRecord, trLp)
        this.tvRecord.typeface = Theme.font
        this.tvRecord.setText(R.string.record)
        this.tvRecord.setTextColor(theme.getColor(Theme.key_primaryColor))
        this.tvRecord.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.tvRecord.setPadding(dp(20f), p10, dp(20f), p10)
        this.tvRecord.setBackgroundResource(ViewUtils.selectableResId(requireContext()))
        this.tvRecord.setOnClickListener {
            if (this.playTimer == null) {
                if (AppPermissionUtils.checkRecordAudioPermission(this, AUDIO_PERMISSION) &&
                    AppPermissionUtils.checkWriteExternalStorage(requireActivity(), STORAGE_PERMISSION)) {
                    if (this.isRecording) {
                        stopRecording()
                    } else {
                        startRecording()
                    }
                }
            } else {
                stop()
            }
        }
        val tvInfo = TextView(requireContext())
        mainLinear.addView(tvInfo, ViewUtils.linearMatchWrap())
        tvInfo.typeface = Theme.font
        tvInfo.text = String.format(
            "%s\n%s",
            String.format(getString(R.string.max_duration), this.maxDuration),
            String.format(getString(R.string.min_duration), this.minDuration)
        )
        tvInfo.setTextColor(theme.getColor(Theme.key_greyColor))
        tvInfo.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        tvInfo.setPadding(ViewUtils.descPadding, dp(5f), ViewUtils.descPadding, p10)
        this.bag.add(NCPCore.instance.account.getCurrentSettings().subscribe({ settings ->
            val length = settings.settingsParams?.app?.voicemail?.greeting?.length
            if (length != null) {
                this.minDuration = length.min.toInt()
                this.maxDuration = length.max.toInt()
            }
            this.tvMaxTime.text = formatSecond(this.maxDuration.toLong())
            getVoiceMails()
        }, {
            this.tvMaxTime.text = formatSecond(this.maxDuration.toLong())
            getVoiceMails()
        }))
        requireActivity().onBackPressedDispatcher.addCallback(viewLifecycleOwner, object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
                if (this@VoiceMailFragment.timer != null) {
                    this@VoiceMailFragment.stopRecording()
                } else {
                    requireActivity().supportFragmentManager.popBackStack()
                }
            }
        })

        return rootView
    }

    private fun formatSecond(second: Long): String {
        return String.format(Locale.getDefault(), "%02d:%02d", second / 60, second % 60)
    }

    private fun getVoiceMails() {
        this.viewModel.getVoiceMail().observe(viewLifecycleOwner) { resp ->
            when (resp.status) {
                Resource.Status.LOADING -> {
                    this.progressDialog.show()
                }
                Resource.Status.SUCCESS -> {
                    this.voiceMailResult = resp.data ?: return@observe
                    this.progressDialog.dismiss()
                    this.showVoiceMails(resp.data!!)
                }
                Resource.Status.ERROR -> {
                    this.progressDialog.dismiss()
                }
            }
        }
    }

    private fun showVoiceMails(result: GetVoiceMail.Response) {
        this.defaultContent.removeAllViews()
        this.customContent.removeAllViews()
        this.progressDialog.dismiss()
        for ((i, greeting) in result.defaultGreetings.withIndex()) {
            createViewItem(this.defaultContent, greeting, i < result.defaultGreetings.size)
        }
        for ((i, greeting) in result.customGreetings.withIndex()) {
            createViewItem(this.customContent, greeting, i < result.customGreetings.size)
        }
        this.defaultCard.visibility = if (result.defaultGreetings.isNotEmpty()) View.VISIBLE else View.GONE
        this.customCard.visibility = if (result.customGreetings.isNotEmpty()) View.VISIBLE else View.GONE
    }

    private fun createViewItem(parent: LinearLayout, greeting: GetVoiceMail.Greeting, addLine: Boolean) {
        val isCurrent = greeting.id == this.profileService?.voicemail?.id
        val file = NCPCore.instance.file.getFile(FileController.FileType.VOICES, greeting.fileId + ".wav")
        val isDownloaded = file.exists()
        val rowDefault = RowVoiceMail(requireActivity())
        parent.addView(rowDefault, ViewUtils.linearMatchWrap())
        rowDefault.tvName.text = greeting.name
        rowDefault.check.visibility = if (isCurrent) View.VISIBLE else View.GONE
        rowDefault.actionBtn.visibility = if (greeting.fileId != null) View.VISIBLE else View.GONE
        rowDefault.actionBtn.setImageResource(if (isDownloaded) R.drawable.ic_play else R.drawable.ic_download)
        rowDefault.nameLyt.setOnClickListener {
            if (!isCurrent) {
                setGreeting(greeting.id)
            }
        }
        rowDefault.nameLyt.setOnLongClickListener {
            val sheetItems = ArrayList<SelectMenuItem>()
            sheetItems.add(SelectMenuItem(getString(R.string.delete), R.drawable.ic_delete_black_24dp))
            val sheet = SelectMenuSheet(requireContext(), greeting.name, sheetItems)
            sheet.showDialog { position ->
                if (position == 0) {
                    removeGreeting(greeting.id)
                }
            }
            false
        }
        rowDefault.actionBtn.setOnClickListener {
            if (isDownloaded) {
                if (rowDefault.actionBtn == lastPlayIcon) {
                    stop()
                } else {
                    play(rowDefault.actionBtn, file)
                }
            } else {
                if (AppPermissionUtils.checkWriteExternalStorage(requireActivity(), STORAGE_PERMISSION)) {
                    if (ViewUtils.isOnline(requireActivity())) {
                        this.progressDialog.show()
                        this.downloadingUrl = greeting.fileId
                        download(greeting)
                    }
                }
            }
        }
        if (addLine) {
            val dv = View(requireContext())
            val dvLp = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, 1)
            dvLp.setMargins(dp(50f), 0, 0, 0)
            parent.addView(dv, dvLp)
            dv.setBackgroundColor(theme.getColor(Theme.key_dividerGrey))
        }
    }

    private fun download(greeting: GetVoiceMail.Greeting) {
        greeting.fileId?.let { fileId ->
            val serverUrl = NCPCore.instance.file.getServerUrl(fileId, FileController.FileType.VOICES, false)
            val localUrl = NCPCore.instance.file.getFile(FileController.FileType.VOICES, greeting.fileId + ".wav")
            CoroutineScope(Dispatchers.IO).launch {
                NCPCore.instance.file.addDownloadTask(serverUrl, localUrl.absolutePath, true).collect { task ->
                    when(task.status.state) {
                        FileTask.State.INITIALIZED -> {}
                        FileTask.State.PROCESSING -> {}
                        FileTask.State.CANCELLED -> {}
                        FileTask.State.SUCCESS -> {
                            withContext(Dispatchers.Main) {
                                this@VoiceMailFragment.voiceMailResult?.let { this@VoiceMailFragment.showVoiceMails(it) }
                            }
                        }
                        FileTask.State.ERROR -> {
                            withContext(Dispatchers.Main) {
                                progressDialog.dismiss()
                                ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat), false)
                            }
                        }
                    }
                }
            }
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == STORAGE_PERMISSION && AppPermissionUtils.hasWriteExternalStorage(requireActivity())) { }
    }

    private fun startRecording() {
        try {
            stop()
            this.recordFile = file()
            this.recorder = WavRecorder.wav(PullTransport.Default(mic) { audioChunk: AudioChunk ->
                Log.d(TAG, "RecordingChunk:" + (audioChunk.maxAmplitude() / 200.0).toFloat())
            }, this.recordFile)
            this.tvRecord.setText(R.string.stop)
            this.recorder?.startRecording()
            this.startTime = System.currentTimeMillis()
            this.isRecording = true
            this.startTimer()
        } catch (e: Exception) {
            Logger.log(
                "$TAG.startRecording",
                (if (e.message != null) e.message else e.toString())!!,
                true
            )
        }
    }

    private fun stopRecording() {
        try {
            // stops the recording activity
            this.isRecording = false
            try {
                this.recorder?.stopRecording()
                this.recorder = null
            } catch (e: IOException) {
                e.printStackTrace()
            }
            stopTimer()
            this.tvRecord.setText(R.string.record)
            this.tvCurrentTime.text = "00:00"
            this.recordProgress.progress = 0
            if (this.recordFile != null) {
                val duration = (System.currentTimeMillis() - this.startTime) / 1000
                if (duration < this.minDuration) {
                    this.recordFile?.delete()
                    ViewUtils.showErrorDialog(requireActivity(), null, String.format(getString(R.string.min_duration), this.minDuration), false)
                    this.tvCurrentTime.text = "00:00"
                    this.recordProgress.progress = 0
                } else {
                    showNameDialog(this.recordFile!!.path)
                }
            }
        } catch (e: Exception) {
            Log.e(TAG, "stopRecording: ", e)
        }
    }

    private fun stopTimer() {
        this.timer?.cancel()
        this.timer = null
    }

    private fun startTimer() {
        this.timer = Timer()
        this.timer?.schedule(object : TimerTask() {
            override fun run() {
                this@VoiceMailFragment.activity?.runOnUiThread {
                    val time = (System.currentTimeMillis() - this@VoiceMailFragment.startTime) / 1000
                    this@VoiceMailFragment.tvCurrentTime.text = formatSecond(time)
                    val progress = (time * 100).toInt() / this@VoiceMailFragment.maxDuration
                    this@VoiceMailFragment.recordProgress.progress = min(progress, 100)
                    if (time >= this@VoiceMailFragment.maxDuration) {
                        stopRecording()
                    }
                }
            }
        }, 1000, 1000)
    }

    private fun upload(path: String) {
        this.uploadingUrl = path
        if (ViewUtils.isOnline(requireActivity())) {
            CoroutineScope(Dispatchers.IO).launch {
                NCPCore.instance.file.addUploadTask(this@VoiceMailFragment.uploadingUrl!!).collect { task ->
                    when (task.status.state) {
                        FileTask.State.INITIALIZED -> {}
                        FileTask.State.PROCESSING -> {}
                        FileTask.State.SUCCESS -> {
                            task.status.result?.let { fileId ->
                                withContext(Dispatchers.Main) {
                                    addGreeting(fileId, enteredName)
                                }
                            }
                        }
                        FileTask.State.ERROR -> {
                            withContext(Dispatchers.Main) {
                                this@VoiceMailFragment.progressDialog.dismiss()
                                val b = AlertDialog.Builder(requireContext(), Theme.dialogTheme)
                                b.setMessage(R.string.error_upload_file)
                                b.setNegativeButton(R.string.cancel) { dialog, _ -> dialog.dismiss() }
                                b.setPositiveButton(R.string.repeat) { dialog, _ ->
                                    dialog.dismiss()
                                    if (uploadingUrl != null) {
                                        upload(uploadingUrl!!)
                                    }
                                }
                                b.create().show()
                            }
                        }
                        else -> {}
                    }
                }
            }
            this.progressDialog.show()
        }
    }

    private fun setGreeting(id: String) {
        if (this.profilePlanId == null || this.profileService == null) return
        this.viewModel.setVoiceMail(this.profilePlanId!!, this.profileService!!.id, id).observe(viewLifecycleOwner) {
            when (it.status) {
                Resource.Status.LOADING -> {
                    this.progressDialog.show()
                }
                Resource.Status.SUCCESS -> {
                    this.progressDialog.dismiss()
                    this.profileService?.voicemail = it.data?.voicemail
                    this.voiceMailResult?.let { res ->
                        this.showVoiceMails(res)
                    }
                }
                Resource.Status.ERROR -> {
                    this.progressDialog.dismiss()
                    ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat), false)
                }
            }
        }
    }

    private fun showNameDialog(uploadUrl: String) {
        val lyt = LinearLayout(requireContext())
        lyt.setPadding(dp(10f), 0, dp(10f), 0)
        val edt = EditText(requireContext())
        lyt.addView(edt, ViewUtils.linearMatchWrap())
        edt.setHint(R.string.greeting_name)
        edt.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        edt.setHintTextColor(theme.getColor(Theme.key_greyColor))
        edt.inputType = InputType.TYPE_CLASS_TEXT or InputType.TYPE_TEXT_FLAG_CAP_SENTENCES
        edt.filters = arrayOf<InputFilter>(InputFilter.LengthFilter(48))
        val b = AlertDialog.Builder(requireContext(), Theme.dialogTheme)
        b.setTitle(R.string.set_greeting_name)
        b.setCancelable(false)
        b.setView(lyt)
        b.setNegativeButton(R.string.delete) { dialog, _ ->
            dialog.dismiss()
            uploadingUrl = null
            recordFile?.delete()
        }
        b.setPositiveButton(R.string.save) { _, _ ->
            enteredName = edt.text.toString().trim { it <= ' ' }
            upload(uploadUrl)
        }
        val dialog = b.create()
        dialog.show()
        val saveBtn = dialog.getButton(DialogInterface.BUTTON_POSITIVE)
        saveBtn?.isEnabled = false
        edt.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                saveBtn?.isEnabled = s.toString().trim { it <= ' ' }.isNotEmpty()
            }

            override fun afterTextChanged(s: Editable) {}
        })
    }

    private fun addGreeting(fileId: String, name: String) {
        this.viewModel.addVoiceMail(fileId, name).observe(viewLifecycleOwner) {
            when (it.status) {
                Resource.Status.LOADING -> {
                    this.progressDialog.show()
                }
                Resource.Status.SUCCESS -> {
                    this.progressDialog.dismiss()
                    this.getVoiceMails()
                }
                Resource.Status.ERROR -> {
                    this.progressDialog.dismiss()
                    ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat), false)
                }
            }
        }
    }

    private fun removeGreeting(greetingId: String) {
        this.viewModel.removeVoiceMail(greetingId).observe(viewLifecycleOwner) {
            when (it.status) {
                Resource.Status.LOADING -> {
                    this.progressDialog.show()
                }
                Resource.Status.SUCCESS -> {
                    this.progressDialog.dismiss()
                    this.getVoiceMails()
                }
                Resource.Status.ERROR -> {
                    this.progressDialog.dismiss()
                    ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat), false)
                }
            }
        }
    }

    private val mic: PullableSource
        get() = PullableSource.Default(
            AudioRecordConfig.Default(MediaRecorder.AudioSource.MIC, AudioFormat.ENCODING_PCM_16BIT, AudioFormat.CHANNEL_IN_MONO, 44100)
        )

    private fun file(): File {
        return NCPCore.instance.file.getFile(FileController.FileType.VOICES, UUID.randomUUID().toString() + ".wav")
    }

    private fun play(icon: ImageView, file: File) {
        try {
            stop()
            this.tvRecord.setText(R.string.stop)
            this.lastPlayIcon = icon
            this. lastPlayIcon?.setImageResource(R.drawable.ic_pause)
            this.player = MediaPlayer()
            this.player?.setDataSource(file.path)
            this.player?.prepare()
            this.player?.start()
            this.player?.isLooping = false
            this.player?.setOnCompletionListener { mp: MediaPlayer? -> stop() }
            val duration = this.player?.duration ?: 0
            this.playTimer = object : CountDownTimer((duration * 1000).toLong(), 1000) {
                override fun onTick(millisUntilFinished: Long) {
                    requireActivity().runOnUiThread {
                        val current = this@VoiceMailFragment.player!!.currentPosition / 1000
                        this@VoiceMailFragment.tvCurrentTime.text = formatSecond(current.toLong())
                        this@VoiceMailFragment.recordProgress.progress = player!!.currentPosition * 100 / duration
                    }
                }

                override fun onFinish() {}
            }.start()
        } catch (e: IOException) {
            Log.e(TAG, "stopRecording: " + e.message)
        }
    }

    override fun onDestroy() {
        this.stopTimer()
        super.onDestroy()
    }

    private fun stop() {
        try {
            this.tvRecord.setText(R.string.record)
            this.playTimer?.cancel()
            this.playTimer = null
            if (this.player?.isPlaying == true) this.player?.stop()
            this.player = null
            this.recordProgress.progress = 0
            this.lastPlayIcon?.setImageResource(R.drawable.ic_play)
            this.lastPlayIcon = null
            this.tvCurrentTime.text = "00:00"
        } catch (e: Exception) {
            Log.e(TAG, "stop: ", e)
        }
    }
}