package nexus.billing.ui.profile.voicemail

import androidx.lifecycle.LiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.liveData
import nexus.billing.billingKit
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceVoiceMailSet
import nexus.billing.methods.requests.voicemail.AddVoiceMail
import nexus.billing.methods.requests.voicemail.GetVoiceMail
import nexus.billing.methods.requests.voicemail.RemoveVoiceMail
import nexus.billing.models.PlanInfo
import nexus.core.NCPCore
import nexus.appui.utils.Resource

class VoiceMailViewModel: ViewModel() {

    var planInfo: PlanInfo? = null

    fun getPlan(planId: String): LiveData<Resource<PlanInfo>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestGetPlanWithInfo(planId)
                if (result != null) {
                    this@VoiceMailViewModel.planInfo = result
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("plan not found", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun getVoiceMail(): LiveData<Resource<GetVoiceMail.Response>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                NCPCore.instance.billingKit.controller.requestGetVoiceMail()?.let {
                    emit(Resource.success(it))
                } ?: run {
                    emit(Resource.error("Error", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun setVoiceMail(profilePlanId: String, profileServiceId: String, id: String): LiveData<Resource<ProfilePlanServiceVoiceMailSet.Response>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                NCPCore.instance.billingKit.controller.requestSetVoiceMail(profilePlanId, profileServiceId, id)?.let {
                    emit(Resource.success(it))
                } ?: run {
                    emit(Resource.error("not setted", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun addVoiceMail(fileId: String, name: String): LiveData<Resource<AddVoiceMail.Response>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                NCPCore.instance.billingKit.controller.requestAddVoiceMail(fileId, name)?.let {
                    emit(Resource.success(it))
                } ?: run {
                    emit(Resource.error("not added", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun removeVoiceMail(id: String): LiveData<Resource<RemoveVoiceMail.Response>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                NCPCore.instance.billingKit.controller.requestRemoveVoiceMail(id)?.let {
                    emit(Resource.success(it))
                } ?: run {
                    emit(Resource.error("not removed", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }
}