package nexus.billing.ui.rows

import android.content.Context
import android.view.Gravity
import android.view.View
import android.widget.LinearLayout
import android.widget.ProgressBar
import androidx.core.view.setPadding
import nexus.billing.R
import nexus.billing.models.PlanInfo
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener

class AddServiceView(var ctx: Context): LinearLayout(ctx) {

    val progress = ProgressBar(context)
    private val categoriesLyt = LinearLayout(context)

    init {
        this.layoutParams = ViewUtils.linearMatchWrap()
        this.orientation = VERTICAL
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), Theme.dp(10f))
        this.setPadding(Theme.dp(12f))
        this.elevation = Theme.dp(5f).toFloat()

        val infoView = RowPlanTopInfo(context)
        this.addView(infoView, ViewUtils.linearMatchWrap())
        infoView.nameTv.text = ctx.getString(R.string.add_services)
        infoView.image.setImageResource(R.drawable.ic_add_service)
        infoView.descTv.text = ctx.getString(R.string.add_service_desc)
        infoView.setPadding(0, 0, 0, Theme.dp(5f))

        val line = View(context)
        line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        this.addView(line, ViewUtils.linearParams(LayoutParams.MATCH_PARENT, 1, 0f))

        this.categoriesLyt.orientation = VERTICAL
        this.addView(this.categoriesLyt, ViewUtils.linearMatchWrap())

        this.progress.visibility = View.GONE
        val pLp = ViewUtils.linearMatchWrap()
        pLp.gravity = Gravity.CENTER_HORIZONTAL
        pLp.topMargin = Theme.dp(5f)
        this.categoriesLyt.addView(this.progress, pLp)
    }

    fun showData(plans: ArrayList<PlanInfo>, listener: (plan: PlanInfo) -> Unit) {
        this.categoriesLyt.removeAllViews()
        for (publicPlan in plans.distinctBy { it.category.id }) {
            val categoryItem = RowCategory(context)
            this.categoriesLyt.addView(categoryItem, ViewUtils.linearMatchWrap())
            categoryItem.showData(publicPlan)
            categoryItem.setDebounceClickListener {
                listener.invoke(publicPlan)
            }
        }
    }

}