package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Typeface
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.recycler.SpaceItemDecoration
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.billingKit
import nexus.billing.methods.assets.Asset
import nexus.billing.methods.service.Service
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.loadImage
import nexus.core.NCPCore
import nexus.core.util.Logger

class AssetsView(var ctx: Context): LinearLayout(ctx) {

    val rv = RecyclerView(ctx)
    val emptyTv = TextView(ctx)
    var listener: AssetsListener? = null
    private var iconLoaded = false
    private var adapter: AssetsAdapter? = null

    interface AssetsListener {
        fun assetSelected(asset: Asset)
    }

    init {
        this.orientation = VERTICAL
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), Theme.dp(10f))
        this.rv.layoutManager = LinearLayoutManager(ctx)
        this.rv.addItemDecoration(SpaceItemDecoration(0))
        this.addView(this.rv, ViewUtils.linearMatchWrap())
        this.emptyTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.emptyTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.emptyTv.gravity = Gravity.CENTER
        this.emptyTv.setTypeface(Theme.font, Typeface.BOLD)
        this.emptyTv.setPadding(0, Theme.dp(20f), 0, Theme.dp(20f))
        this.emptyTv.setText(R.string.numbers_not_found)
        this.addView(this.emptyTv, ViewUtils.linearMatchMatch())
    }

    fun showData(service: Pair<Service, DescriptionObject?>) {
        service.first.assets?.let {
            if (it.isNotEmpty()) this.emptyTv.visibility = GONE
            AssetsAdapter(it, service).also {
                this.rv.adapter = it
                this.adapter = it
            }
        }
        service.second?.icon?.let { loadServiceIcon(it) }
    }

    private fun loadServiceIcon(fileId: String) {
        CoroutineScope(Dispatchers.IO).launch {
            NCPCore.instance.billingKit.controller.loadImage(fileId, true).catch { error ->
                Logger.log("Utils:loadServiceIcon", error.toString())
            }.collect {
                withContext(Dispatchers.Main) {
                    this@AssetsView.iconLoaded = true
                    adapter?.notifyDataSetChanged()
                }
            }
        }
    }

    private inner class AssetsAdapter(val assets: Array<Asset>, val service: Pair<Service, DescriptionObject?>) : RecyclerView.Adapter<AssetsAdapter.AssetHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AssetHolder {
            return AssetHolder(RowAsset(parent.context))
        }

        override fun onBindViewHolder(holder: AssetHolder, position: Int) {
            val asset = this.assets[position]
            val item = holder.itemView as RowAsset
            if (iconLoaded) {
                this.service.second?.icon?.let { loadImage(it, item.icon) }
            }
            item.showAsset(asset)
            holder.itemView.setOnClickListener {
                this@AssetsView.listener?.assetSelected(asset)
            }
        }

        override fun getItemCount(): Int {
            return this.assets.size
        }

        inner class AssetHolder(itemView: View) : RecyclerView.ViewHolder(itemView)
    }
}
