package nexus.billing.ui.rows

import android.content.Context
import android.graphics.PorterDuff
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils.linearMatchWrap
import nexus.appui.utils.ViewUtils.selectableResId
import nexus.billing.R
import nexus.billing.models.PlanInfo
import nexus.billing.ui.loadImage

class RowCategory(val ctx: Context) : LinearLayout(ctx) {
    var icon: ImageView
    var arrow: ImageView
    var tvName: TextView
    var tvDesc: TextView

    init {
        val typeface = Theme.font
        val theme = Theme
        val p5 = dp(5f)
        this.orientation = HORIZONTAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.isClickable = true
        this.isFocusable = true
        this.setBackgroundResource(selectableResId(ctx))
        this.setPadding(0, p5, 0, p5)

        this.icon = ImageView(ctx)
        this.icon.setColorFilter(theme.getColor(Theme.key_rowTextBlack))
        this.addView(this.icon, dp(40f), dp(40f))
        val tvLayouts = LinearLayout(ctx)
        tvLayouts.setPadding(0, 0, p5, 0)
        val lytLp = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT, 1f)
        lytLp.setMargins(dp(10f), 0, 0, 0)
        this.addView(tvLayouts, lytLp)
        tvLayouts.orientation = VERTICAL

        this.tvName = TextView(ctx)
        tvLayouts.addView(this.tvName, linearMatchWrap())
        this.tvName.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        this.tvName.isSingleLine = true
        this.tvName.ellipsize = TextUtils.TruncateAt.END
        this.tvName.setPadding(0, 0, 0, dp(2f))
        this.tvName.isSingleLine = true
        this.tvName.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.tvName.setTypeface(typeface, Typeface.BOLD)
        this.tvDesc = TextView(ctx)

        tvLayouts.addView(this.tvDesc, LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT)
        this.tvDesc.setTextColor(theme.getColor(Theme.key_textSecondary))
        this.tvDesc.setTextSize(TypedValue.COMPLEX_UNIT_SP, 12f)
        this.tvDesc.typeface = typeface
        this.arrow = ImageView(ctx)
        this.addView(this.arrow, LayoutParams(dp(25f), dp(25f)))
        this.arrow.setImageResource(R.drawable.ic_arrow_right)
        this.arrow.setColorFilter(theme.getColor(Theme.key_greyColor), PorterDuff.Mode.SRC_IN)
    }

    fun showData(planInfo: PlanInfo) {
        this.tvName.text = planInfo.categoryDesc?.title ?: context.getString(R.string.category)
        this.tvDesc.text = planInfo.categoryDesc?.shortText ?: planInfo.categoryDesc?.longText
        planInfo.categoryDesc?.icon?.let { loadImage(it, this.icon) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
    }
}