package nexus.billing.ui.rows

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Color
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.appcompat.widget.SwitchCompat
import androidx.core.graphics.drawable.DrawableCompat
import nexus.billing.R
import nexus.billing.methods.common.Descriptions
import nexus.billing.ui.loadImage
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils

class RowForward(var ctx: Context): LinearLayout(ctx) {

    val icon: ImageView
    val nameTv: TextView
    val forwardSwitch: SwitchCompat

    init {
        this.orientation = HORIZONTAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.setPadding(0, dp(4f), 0, dp(4f))
        this.setBackgroundResource(ViewUtils.selectableResId(ctx))
        this.isClickable = true

        this.icon = ImageView(ctx)
        addView(this.icon, ViewUtils.linearParams(30, 30))
        this.icon.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        this.icon.setImageResource(R.drawable.ic_forwarding)
        this.nameTv = TextView(ctx)
        addView(this.nameTv, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.nameTv.isSingleLine = true
        this.nameTv.setText(R.string.forward_to_app)
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        this.nameTv.setPadding(dp(7f), 0, dp(5f), 0)
        this.forwardSwitch = SwitchCompat(ctx)
        addView(this.forwardSwitch, ViewUtils.linearWrapWrap())
        this.forwardSwitch.setBackgroundColor(Color.TRANSPARENT)
        this.forwardSwitch.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.forwardSwitch.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        DrawableCompat.setTintList(this.forwardSwitch.thumbDrawable, ColorStateList(
            arrayOf(intArrayOf(android.R.attr.state_checked), intArrayOf()),
            intArrayOf(Theme.getColor(Theme.key_primaryColor), Theme.getColor(Theme.key_greyColor)))
        )
        this.setOnClickListener { this.forwardSwitch.toggle() }
    }

    fun showData(description: Descriptions) {
        this.nameTv.text = description.title?: this.ctx.getString(R.string.service)
        description.icon?.let { loadImage(it, this.icon) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service)}
    }
}