package nexus.billing.ui.rows

import android.content.Context
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.profile.ProfileService
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.loadImage
import java.text.SimpleDateFormat
import java.util.Date

class RowNgn(var ctx: Context): LinearLayout(ctx) {

    val icon: ImageView
    val arr: ImageView
    val nameTv: TextView
    val valueTv: TextView
    val renewTv: TextView
    val stateTv: TextView

    init {
        this.orientation = VERTICAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.setBackgroundResource(ViewUtils.selectableResId(ctx))
        this.isClickable = true

        val layout = LinearLayout(this.ctx)
        addView(layout, ViewUtils.linearMatchWrap())
        layout.orientation = HORIZONTAL
        layout.gravity = Gravity.CENTER_VERTICAL
        layout.setPadding(0, dp(3f), 0, dp(3f))

        this.icon = ImageView(ctx)
        layout.addView(this.icon, ViewUtils.linearParams(30, 30))
        this.icon.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        this.icon.setImageResource(R.drawable.ic_plan_internet)

        this.nameTv = TextView(ctx)
        layout.addView(this.nameTv, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        this.nameTv.setPadding(dp(7f), 0, dp(5f), 0)

        this.valueTv = TextView(ctx)
        layout.addView(this.valueTv, ViewUtils.linearWrapWrap())
        this.valueTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.valueTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.valueTv.isSingleLine = true
        this.valueTv.ellipsize = TextUtils.TruncateAt.END

        this.arr = ImageView(ctx)
        layout.addView(arr, ViewUtils.linearParams(25, 25))
        this.arr.setImageResource(R.drawable.ic_arrow_right)
        this.arr.setColorFilter(Theme.getColor(Theme.key_greyColor))

        this.stateTv = TextView(context)
        addView(this.stateTv, ViewUtils.linearMatchWrap())
        this.stateTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.stateTv.typeface = Theme.font
        this.stateTv.isSingleLine = true
        this.stateTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14f)
        this.stateTv.visibility = View.GONE

        this.renewTv = TextView(context)
        addView(this.renewTv, ViewUtils.linearMatchWrap())
        this.renewTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.renewTv.typeface = Theme.font
        this.renewTv.isSingleLine = true
        this.renewTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14f)
        this.renewTv.visibility = View.GONE
    }

    fun showData(service: Pair<ProfileService, DescriptionObject?>) {
        val description = service.second
        this.nameTv.text = description?.title?: this.ctx.getString(R.string.service)
        description?.icon?.let { loadImage(it, this.icon) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
        if (service.first.state != ProfileService.State.active) {
            this.stateTv.visibility = View.GONE
            this.stateTv.text = when (service.first.state) {
                ProfileService.State.pending -> {
                    context.getString(R.string.plan_pending)
                }
                ProfileService.State.completed -> {
                    context.getString(R.string.completed)
                }
                ProfileService.State.inactive -> {
                    context.getString(R.string.inactive)
                }
                ProfileService.State.suspended -> {
                    context.getString(R.string.suspended)
                }
                else -> {
                    this.stateTv.visibility = View.GONE
                    ""
                }
            }
        } else {
            this.stateTv.visibility = View.GONE
        }
        service.first.nextActivationAt?.let {
            this.renewTv.visibility = View.VISIBLE
            this.renewTv.text = "${context.getString(R.string.renews)} ${SimpleDateFormat("MMMM dd, yyyy").format(Date(it.toLong()))}"
        } ?: run {
            this.renewTv.visibility = View.GONE
        }
    }
}