package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Color
import android.graphics.Typeface
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewTreeObserver
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.cardview.widget.CardView
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.profile.ProfilePlan
import nexus.core.util.formattedAmount
import java.util.Locale

class RowPlanAction(val ctx: Context): CardView(ctx) {

    private val contentLyt: LinearLayout
    val titleTv: TextView
    val descTv: TextView
    val balanceTv: TextView
    val icon: ImageView
    val line: View

    init {
        this.setCardBackgroundColor(Theme.getColor(Theme.key_blockView))
        this.cardElevation = Theme.dp(5f).toFloat()
        this.radius = Theme.dp(10f).toFloat()
        this.layoutParams = ViewUtils.linearMatchWrap()

        this.line = View(context)
        this.line.setBackgroundColor(Theme.getColor(Theme.key_primaryColor))
        this.addView(this.line, ViewUtils.frameNoDp(Theme.dp(6f), 0, Gravity.START))

        this.viewTreeObserver.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                this@RowPlanAction.viewTreeObserver.removeOnGlobalLayoutListener(this)
                this@RowPlanAction.line.layoutParams = LayoutParams(Theme.dp(6f), this@RowPlanAction.height)
            }
        })

        this.contentLyt = LinearLayout(context)
        this.contentLyt.setBackgroundResource(ViewUtils.selectableResId(context))
        this.contentLyt.isClickable = true
        this.contentLyt.orientation = LinearLayout.HORIZONTAL
        this.contentLyt.gravity = Gravity.CENTER_VERTICAL
        this.contentLyt.setPadding(Theme.dp(15f), Theme.dp(10f), Theme.dp(10f), Theme.dp(10f))
        this.addView(this.contentLyt, ViewUtils.linearMatchWrap())

        val stateLyt = LinearLayout(context)
        stateLyt.orientation = LinearLayout.VERTICAL
        stateLyt.gravity = Gravity.CENTER_VERTICAL
        stateLyt.setPadding(0, 0, Theme.dp(5f), 0)
        this.contentLyt.addView(stateLyt, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))

        this.titleTv = TextView(context)
        stateLyt.addView(this.titleTv, ViewUtils.linearMatchWrap())
        this.titleTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.titleTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.titleTv.setPadding(0, 0, 0, Theme.dp(2f))
        this.titleTv.setTypeface(Theme.font, Typeface.BOLD)

        this.descTv = TextView(context)
        stateLyt.addView(this.descTv, ViewUtils.linearMatchWrap())
        this.descTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.descTv.typeface = Theme.font

        this.balanceTv = TextView(context)
        this.contentLyt.addView(this.balanceTv, ViewUtils.linearWrapWrap())
        this.balanceTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.balanceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.balanceTv.visibility = GONE
        this.balanceTv.setTypeface(Theme.font, Typeface.BOLD)

        this.icon = ImageView(context)
        this.icon.setColorFilter(Theme.getColor(Theme.key_primaryColor))
        this.icon.setImageResource(R.drawable.ic_plan_warning)
        this.contentLyt.addView(this.icon, ViewUtils.linearParams(35, 35))

        val arr = ImageView(context)
        arr.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        arr.setImageResource(R.drawable.ic_arrow_right)
        this.contentLyt.addView(arr, ViewUtils.linearParams(20, 20))
    }

    override fun setOnClickListener(l: OnClickListener?) {
        this.contentLyt.setOnClickListener(l)
    }

    fun showData(action: ProfilePlan.Action) {
        val color = when {
            action.identity?.info != null -> {
                val verification = action.identity.info.required.firstOrNull()
                this.icon.setImageResource(R.drawable.ic_plan_verification)
                this.descTv.text = context.getString(R.string.action_ident_desc)
                this.titleTv.text = verification?.let {
                     "${context.getString(R.string.verification)} ${Locale(it.country, it.country).displayCountry}"
                } ?: run {
                    context.getString(R.string.verification)
                }
                0xffffa258.toInt()
            }
            action.asset?.info != null -> {
                this.icon.setImageResource(R.drawable.ic_sim)
                this.titleTv.text = context.getString(R.string.select_did_number)
                this.descTv.text = context.getString(R.string.action_asset_desc)
                Color.GREEN
            }
            action.support?.info != null -> {
                this.titleTv.text = context.getString(R.string.talk_to_support)
                this.descTv.text = context.getString(R.string.action_support_desc)
                this.icon.setImageResource(R.drawable.ic_support)
                0xff4682F4.toInt()
            }
            action.balance?.info != null -> {
                this.icon.visibility = GONE
                this.balanceTv.visibility = VISIBLE
                this.titleTv.text = context.getString(R.string.payment_required)
                this.descTv.text = context.getString(R.string.action_balance_desc)
                this.icon.setImageResource(R.drawable.ic_balance)
                this.balanceTv.text = action.balance.info.required.formattedAmount(action.balance.info.currency)
                Color.RED
            }
            else -> Color.BLACK
        }
        this.icon.setColorFilter(color)
        this.line.setBackgroundColor(color)
    }
}