package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Color
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import nexus.appui.R
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.Country
import nexus.appui.utils.ViewUtils
import java.util.*

class RowPlanCountry(ctx: Context) : LinearLayout(ctx) {

    var nameTv: TextView
    var codeTv: TextView
    var arr: ImageView
    private var flagTv: TextView

    init {
        this.layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT)
        this.orientation = HORIZONTAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.setBackgroundResource(ViewUtils.selectableResId(context))
        this.isClickable = true
        this.isFocusable = true
        this.setPadding(dp(10f))
        this.flagTv = TextView(ctx)
        this.flagTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.flagTv.text = "\uD83C\uDDF9\uD83C\uDDEF"
        this.flagTv.setTextColor(Color.BLACK)
        this.flagTv.gravity = Gravity.CENTER
        addView(flagTv, ViewUtils.linearParams(25, 25))
        this.nameTv = TextView(context)
        addView(this.nameTv, LayoutParams(0, LayoutParams.WRAP_CONTENT, 1f))
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 17f)
        this.nameTv.setPadding(dp(10f), 0, dp(10f), 0)
        this.nameTv.gravity = Gravity.START
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        this.nameTv.typeface = Theme.font
        this.codeTv = TextView(context)
        this.codeTv.setPadding(0, 0, dp(5f), 0)
        this.codeTv.setTextColor(ctx.resources.getColor(nexus.billing.R.color.green))
        this.codeTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.codeTv.gravity = Gravity.END
        this.codeTv.typeface = Theme.font
        addView(codeTv, ViewUtils.linearWrapWrap())
        this.arr = ImageView(ctx)
        addView(arr, ViewUtils.linearParams(15, 15))
        this.arr.setImageResource(R.drawable.ic_arrow_right)
        this.arr.setColorFilter(Theme.getColor(Theme.key_greyColor))
    }

    fun setCountry(country: String) {
        this.nameTv.text = Locale(country, country).displayCountry
        this.flagTv.text = Country.getFlag(country)
        ViewUtils.getAllCountries(context).find { it.code == country }?.let {
            this.codeTv.text = it.numberCode
        }
    }
}