package nexus.billing.ui.rows

import android.content.Context
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.loadImage

class RowPlanOption(var ctx: Context): LinearLayout(ctx) {

    val icon: ImageView
    val nameTv: TextView
    val valueTv: TextView

    init {
        this.orientation = HORIZONTAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.setPadding(0, dp(3f), 0 , dp(3f))

        this.icon = ImageView(ctx)
        addView(this.icon, ViewUtils.linearParams(30, 30))
        this.icon.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv = TextView(ctx)
        addView(this.nameTv, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        this.nameTv.setPadding(dp(7f), 0 , dp(5f), 0)
        this.valueTv = TextView(ctx)
        addView(this.valueTv, ViewUtils.linearWrapWrap())
        this.valueTv.setTextColor(ctx.resources.getColor(R.color.green))
        this.valueTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 12f)
        this.valueTv.isSingleLine = true
        this.valueTv.ellipsize = TextUtils.TruncateAt.END
    }

    fun showData(description: DescriptionObject?, value: String) {
        this.nameTv.text = description?.title ?: this.ctx.getString(R.string.service)
        this.valueTv.text = value
        description?.icon?.let { loadImage(it, this.icon, R.drawable.ic_plan_service) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
    }
}