package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Color
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.core.content.ContextCompat
import androidx.core.view.setPadding
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.common.Route
import nexus.billing.methods.profile.ProfileService
import nexus.billing.methods.service.Service
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.getProgressBg
import nexus.billing.ui.loadImage
import nexus.core.util.Currency
import nexus.core.util.formattedAmount
import java.text.SimpleDateFormat
import java.util.Date

class RowPlanPackage(val ctx: Context): CardView(ctx) {

    val icon: ImageView
    val nameTv: TextView
    val descTv: TextView
    val unitsTv: TextView
    val priceTv: TextView
    val packageProgress: ProgressBar
    val rateTv: TextView
    val stateTv: TextView
    val renewTv: TextView
    private val routContentLyt: LinearLayout
    private val progressLyt: LinearLayout
    private val infoLyt: LinearLayout
    private val routeLyt: LinearLayout
    private val stateLyt: LinearLayout
    val progress: ProgressBar
    val activeBtn: Button
    val removeBtn: Button

    init {
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), Theme.dp(10f))
        this.foreground = ContextCompat.getDrawable(context, ViewUtils.selectableResId(this.context))
        this.cardElevation = Theme.dp(3f).toFloat()
        this.radius = Theme.dp(15f).toFloat()

        val contentLyt = LinearLayout(ctx)
        this.addView(contentLyt, ViewUtils.cardMatchWrap())
        contentLyt.orientation = LinearLayout.VERTICAL
        contentLyt.setPadding(Theme.dp(10f))

        val pkgLyt = LinearLayout(ctx)
        contentLyt.addView(pkgLyt, ViewUtils.linearMatchWrap())
        pkgLyt.orientation = LinearLayout.HORIZONTAL
        pkgLyt.weightSum = 1f

        this.infoLyt = LinearLayout(ctx)
        pkgLyt.addView(this.infoLyt, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 0.65f))
        this.infoLyt.orientation = LinearLayout.VERTICAL
        this.infoLyt.setPadding(0, 0, Theme.dp(10f), 0)

        val titleLyt = LinearLayout(ctx)
        this.infoLyt.addView(titleLyt, ViewUtils.linearMatchWrap())
        titleLyt.gravity = Gravity.CENTER_VERTICAL
        titleLyt.orientation = LinearLayout.HORIZONTAL

        this.icon = ImageView(ctx)
        this.icon.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        titleLyt.addView(this.icon, ViewUtils.linearParams(30, 30))

        this.nameTv = TextView(ctx)
        this.nameTv.setPadding(Theme.dp(5f), 0, 0, 0)
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTypeface(Theme.font, Typeface.BOLD)
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        this.nameTv.marqueeRepeatLimit = 5
        this.nameTv.isSelected = true
        titleLyt.addView(this.nameTv, ViewUtils.linearMatchWrap())

        this.stateLyt = LinearLayout(ctx)
        this.infoLyt.addView(this.stateLyt, ViewUtils.linearMatchWrap())
        this.stateLyt.visibility = View.GONE
        this.stateLyt.setPadding(0, 0, 0, Theme.dp(3f))
        this.stateLyt.gravity = Gravity.CENTER_VERTICAL
        this.stateLyt.orientation = LinearLayout.HORIZONTAL

        val statusTv = TextView(ctx)
        statusTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        statusTv.typeface = Theme.font
        statusTv.text = "${this.ctx.getString(R.string.status)}: "
        statusTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.stateLyt.addView(statusTv, ViewUtils.linearWrapWrap())

        this.stateTv = TextView(ctx)
        this.stateTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.stateTv.typeface = Theme.font
        this.stateTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.stateLyt.addView(this.stateTv, ViewUtils.linearWrapWrap())

        this.descTv = TextView(ctx)
        this.descTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.descTv.typeface = Theme.font
        this.descTv.setPadding(0, Theme.dp(3f), 0, 0)
        this.descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        this.infoLyt.addView(this.descTv, ViewUtils.linearMatchWrap())

        this.progressLyt = LinearLayout(ctx)
        val pLp = ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 0.35f)
        pLp.gravity = Gravity.CENTER_VERTICAL
        pkgLyt.addView(this.progressLyt, pLp)
        this.progressLyt.orientation = LinearLayout.VERTICAL
        this.progressLyt.gravity = Gravity.END

        this.priceTv = TextView(ctx)
        this.priceTv.setPadding(0, 0, 0, Theme.dp(5f))
        this.priceTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.priceTv.typeface = Theme.font
        this.priceTv.visibility = View.GONE
        this.priceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.priceTv.isSingleLine = true
        this.progressLyt.addView(this.priceTv, ViewUtils.linearWrapWrap())

        this.packageProgress = ProgressBar(ctx, null, android.R.attr.progressBarStyleHorizontal)
        this.packageProgress.isIndeterminate = false
        this.packageProgress.progressDrawable = getProgressBg(ContextCompat.getColor(context, R.color.grey_soft), ContextCompat.getColor(context, R.color.green))
        this.progressLyt.addView(this.packageProgress, ViewUtils.linearParams(LinearLayout.LayoutParams.MATCH_PARENT, Theme.dp(10f), 0f))

        this.unitsTv = TextView(ctx)
        this.unitsTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.unitsTv.typeface = Theme.font
        this.unitsTv.isSingleLine = true
        this.unitsTv.gravity = Gravity.END
        this.unitsTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        this.unitsTv.marqueeRepeatLimit = 10
        this.unitsTv.isSelected = true
        this.unitsTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 12f)
        this.progressLyt.addView(this.unitsTv, ViewUtils.linearMatchWrap())

        this.renewTv = TextView(context)
        contentLyt.addView(this.renewTv, ViewUtils.linearMatchWrap())
        this.renewTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.renewTv.typeface = Theme.font
        this.renewTv.isSingleLine = true
        this.renewTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14f)
        this.renewTv.visibility = View.GONE

        //Routes

        this.routeLyt = LinearLayout(ctx)
        contentLyt.addView(this.routeLyt, ViewUtils.linearMatchWrap())
        this.routeLyt.orientation = LinearLayout.VERTICAL

        val routeLine = LinearLayout(ctx)
        this.routeLyt.addView(routeLine, ViewUtils.linearMatchWrap())
        routeLine.gravity = Gravity.CENTER_VERTICAL
        routeLine.setPadding(0, Theme.dp(5f), 0, Theme.dp(5f))
        routeLine.orientation = LinearLayout.HORIZONTAL

        val countryTv = TextView(ctx)
        countryTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        countryTv.setTypeface(Theme.font, Typeface.BOLD)
        countryTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        countryTv.isSingleLine = true
        countryTv.text = context.getString(R.string.country_region)
        countryTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        countryTv.marqueeRepeatLimit = 5
        countryTv.isSelected = true
        routeLine.addView(countryTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))

        this.rateTv = TextView(ctx)
        this.rateTv.setPadding(Theme.dp(5f), 0, 0, 0)
        this.rateTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.rateTv.typeface = Theme.font
        this.rateTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        this.rateTv.isSingleLine = true
        this.rateTv.visibility = GONE
        this.rateTv.text = context.getString(R.string.rate_per_min)
        routeLine.addView(this.rateTv, ViewUtils.linearWrapWrap())

        val line = View(context)
        line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        this.routeLyt.addView(line, ViewUtils.linearParams(LayoutParams.MATCH_PARENT, 1, 0f))

        this.routContentLyt = LinearLayout(ctx)
        this.routeLyt.addView(this.routContentLyt, ViewUtils.linearMatchWrap())
        this.routContentLyt.orientation = LinearLayout.VERTICAL

        this.progress = ProgressBar(context)
        this.routeLyt.addView(this.progress, ViewUtils.linearWrapWrap().apply { gravity = Gravity.CENTER_HORIZONTAL })

        val actionLyt = LinearLayout(ctx)
        contentLyt.addView(actionLyt, ViewUtils.cardMatchWrap())
        actionLyt.orientation = LinearLayout.HORIZONTAL

        this.activeBtn = Button(this.context)
        this.activeBtn.background = ViewUtils.getRect(resources.getColor(R.color.green), Theme.dp(8f))
        this.activeBtn.setTextColor(Color.WHITE)
        this.activeBtn.visibility = View.GONE
        this.activeBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.activeBtn.typeface = Theme.font
        this.activeBtn.setPadding(Theme.dp(20f), 0, Theme.dp(20f), 0)
        this.activeBtn.isAllCaps = false
        val bLp = ViewUtils.linearParams(0, Theme.dp(40f), 1f).apply {
            setMargins(0, Theme.dp(5f), Theme.dp(5f), Theme.dp(5f))
        }
        actionLyt.addView(this.activeBtn, bLp)
        this.activeBtn.setText(R.string.activate)

        this.removeBtn = Button(this.context)
        this.removeBtn.background = ViewUtils.getRect(resources.getColor(nexus.appui.R.color.red), Theme.dp(8f))
        this.removeBtn.setTextColor(Color.WHITE)
        this.removeBtn.visibility = View.GONE
        this.removeBtn.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.removeBtn.typeface = Theme.font
        this.removeBtn.setPadding(Theme.dp(20f), 0, Theme.dp(20f), 0)
        this.removeBtn.isAllCaps = false
        val rLp = ViewUtils.linearParams(0, Theme.dp(40f), 1f).apply {
            setMargins(Theme.dp(5f), Theme.dp(5f), 0, Theme.dp(5f))
        }
        actionLyt.addView(this.removeBtn, rLp)
        this.removeBtn.setText(R.string.delete)
    }

    fun showData(service: Pair<ProfileService, DescriptionObject?>, currency: Currency, showExpire: Boolean = false, showDelete: Boolean) {
        this.progressLyt.visibility = if (service.first.units != null) {
            this.infoLyt.layoutParams = ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 0.65f)
            VISIBLE
        } else {
            this.infoLyt.layoutParams = ViewUtils.linearMatchWrap()
            GONE
        }
        val total = service.first.units?.total ?: 0
        val available = service.first.units?.available ?: 0
        val unitsValue = when (service.first.category) {
            Service.Category.voice -> {
                this.packageProgress.progressDrawable = getProgressBg(ContextCompat.getColor(context, R.color.grey_soft), ContextCompat.getColor(context, R.color.orange_color))
                "$available/$total ${ctx.getString(R.string.min)}"
            }
            Service.Category.sms -> {
                this.packageProgress.progressDrawable = getProgressBg(ContextCompat.getColor(context, R.color.grey_soft), ContextCompat.getColor(context, R.color.green))
                "$available/$total ${ctx.getString(R.string.sms)}"
            }
            Service.Category.data -> {
                this.packageProgress.progressDrawable = getProgressBg(ContextCompat.getColor(context, R.color.grey_soft), Color.BLUE)
                "$available/$total ${ctx.getString(R.string.mb)}"
            }
            else -> {
                ""
            }
        }
        this.rateTv.text = "${context.getString(R.string.rate_per_min)} ($currency)"
        this.nameTv.text = service.second?.title ?: this.ctx.getString(R.string.service)
        this.descTv.visibility = if (service.second?.shortText.isNullOrEmpty()) GONE else View.VISIBLE
        this.descTv.text = service.second?.shortText
        this.unitsTv.text = unitsValue
        this.packageProgress.max = total
        this.packageProgress.progress = available
        service.second?.icon?.let { loadImage(it, this.icon) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
        var color = ContextCompat.getColor(this.ctx, R.color.red_color)
        this.stateLyt.visibility = View.VISIBLE
        val state = when (service.first.state) {
            ProfileService.State.pending -> {
                color = ContextCompat.getColor(this.ctx, R.color.orange_color)
                this.ctx.getString(R.string.plan_pending)
            }
            ProfileService.State.completed -> {
                this.ctx.getString(R.string.completed)
            }
            ProfileService.State.active -> {
                color = ContextCompat.getColor(this.ctx, R.color.green)
                this.stateLyt.visibility = View.GONE
                this.ctx.getString(R.string.active)
            }
            ProfileService.State.suspended -> this.ctx.getString(R.string.suspended)
            ProfileService.State.inactive -> this.ctx.getString(R.string.inactive)
            ProfileService.State.removed -> this.ctx.getString(R.string.removed)
        }
        this.stateTv.setTextColor(color)
        this.stateTv.text = state
        if (showExpire) {
            service.first.nextActivationAt?.let {
                this.renewTv.visibility = View.VISIBLE
                this.renewTv.text = "${context.getString(R.string.renews)} ${SimpleDateFormat("MMMM dd, yyyy").format(Date(it.toLong()))}"
            } ?: run {
                this.renewTv.visibility = View.GONE
            }
        } else {
            this.renewTv.visibility = View.GONE
        }

        if (service.first.activation == ProfileService.Activation.manual && showDelete) {
            when (service.first.state) {
                ProfileService.State.inactive,
                ProfileService.State.suspended,
                ProfileService.State.completed,
                ProfileService.State.pending -> {
                    this.activeBtn.visibility = View.VISIBLE
                    this.removeBtn.visibility = View.VISIBLE
                }
                ProfileService.State.active -> {
                    this.activeBtn.visibility = View.GONE
                    this.activeBtn.layoutParams = ViewUtils.linearParams(0, Theme.dp(40f), 1f).apply {
                        setMargins(0, Theme.dp(5f), 0, Theme.dp(5f))
                    }
                    this.removeBtn.visibility = View.VISIBLE
                }
                else -> {}
            }
        }
    }

    fun showData(service: Service, serviceDesc: DescriptionObject?) {
        this.priceTv.visibility = View.VISIBLE
        this.packageProgress.visibility = View.GONE
        this.unitsTv.visibility = View.GONE
        val amount = service.recurringAmount ?: 0
        this.priceTv.text = if (amount > 0) {
            "${amount.formattedAmount(service.currency)}/${context.getString(R.string.month_little)}"
        } else {
            ""
        }
        this.nameTv.text = serviceDesc?.title ?: this.ctx.getString(R.string.service)
        this.descTv.visibility = if (serviceDesc?.shortText.isNullOrEmpty()) GONE else View.VISIBLE
        this.descTv.text = serviceDesc?.shortText
        serviceDesc?.icon?.let { loadImage(it, this.icon) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
    }

    fun showRoutes(routes: Array<Route>, listener: OnClickListener) {
        this.routContentLyt.removeAllViews()
        this.progress.visibility = GONE
        this.routeLyt.visibility = if (routes.isNotEmpty()) VISIBLE else GONE
        this.rateTv.visibility = if (routes.count { it.amount != null } > 0) VISIBLE else GONE
        for ((i, route) in routes.withIndex()) {
            val item = RowRoute(context)
            item.setPadding(0, Theme.dp(5f), 0, Theme.dp(5f))
            this.routContentLyt.addView(item, ViewUtils.linearMatchWrap())
            item.showData(route)

            if (i == 4) {
                val item = RowRoute(context)
                item.setPadding(0, Theme.dp(5f), 0, Theme.dp(5f))
                this.routContentLyt.addView(item, ViewUtils.linearMatchWrap())
                item.nameTv.text = context.getString(R.string.shw_more)
                item.nameTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
                item.nameTv.gravity = Gravity.CENTER
                item.nameTv.setBackgroundResource(ViewUtils.selectableResId(context))
                item.setOnClickListener(listener)
                break
            }
        }
    }
}