package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.cardview.widget.CardView
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.service.Service
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.loadImage
import nexus.core.util.formattedAmount


class RowPlanService(val ctx: Context): CardView(ctx) {

    val icon: ImageView
    val nameTv: TextView
    val descTv: TextView
    val priceTv: TextView

    init {
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), Theme.dp(10f))
        this.cardElevation = Theme.dp(3f).toFloat()
        this.radius = Theme.dp(15f).toFloat()

        val contentLyt = LinearLayout(ctx)
        this.addView(contentLyt, ViewUtils.cardMatchWrap())
        contentLyt.orientation = LinearLayout.VERTICAL
        contentLyt.gravity = Gravity.CENTER_VERTICAL
        contentLyt.weightSum = 1f
        contentLyt.setPadding(Theme.dp(10f), Theme.dp(5f), Theme.dp(10f), Theme.dp(5f))

        val titleLyt = LinearLayout(ctx)
        contentLyt.addView(titleLyt, ViewUtils.linearMatchWrap())
        titleLyt.gravity = Gravity.CENTER_VERTICAL
        titleLyt.orientation = LinearLayout.HORIZONTAL
        this.icon = ImageView(ctx)
        this.icon.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        titleLyt.addView(this.icon, ViewUtils.linearParams(30, 30))

        this.nameTv = TextView(ctx)
        this.nameTv.setPadding(Theme.dp(5f), 0, 0, 0)
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTypeface(Theme.font, Typeface.BOLD)
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        this.nameTv.marqueeRepeatLimit = 5
        this.nameTv.isSelected = true
        titleLyt.addView(this.nameTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))

        this.priceTv = TextView(ctx)
        this.priceTv.setPadding(Theme.dp(5f), 0, 0, 0)
        this.priceTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.priceTv.typeface = Theme.font
        this.priceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.priceTv.isSingleLine = true
        titleLyt.addView(this.priceTv, ViewUtils.linearWrapWrap())

        val line = View(context)
        line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        contentLyt.addView(line, ViewUtils.linearParams(LayoutParams.MATCH_PARENT, 1, 0f))

        this.descTv = TextView(ctx)
        this.descTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.descTv.typeface = Theme.font
        this.descTv.setPadding(Theme.dp(5f), Theme.dp(5f), 0, Theme.dp(5f))
        this.descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        contentLyt.addView(this.descTv, ViewUtils.linearMatchWrap())
    }

    fun showData(service: Service, serviceDesc: DescriptionObject?) {
        this.nameTv.text = serviceDesc?.title ?: this.ctx.getString(R.string.service)
        this.descTv.text = serviceDesc?.shortText
        this.priceTv.text = service.recurringAmount?.let { amount ->
            "${amount.formattedAmount(service.currency)}/${context.getString(R.string.month_little)}"
        } ?: run {
            ""
        }
        serviceDesc?.icon?.let { loadImage(it, this.icon, R.drawable.ic_plan_number) }  ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
    }
}
