package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.billing.models.planDesc
import nexus.billing.ui.loadImage
import nexus.core.util.Currency
import nexus.core.util.formattedAmount

class RowPlanTopInfo(val ctx: Context): LinearLayout(ctx) {

    val image: ImageView
    val nameTv: TextView
    val descTv: TextView
    val costTv: TextView
    val costTimeTv: TextView

    init {
        this.orientation = HORIZONTAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.image = ImageView(ctx)
        addView(this.image, ViewUtils.linearParams(65, 65))
        val namesLyt = LinearLayout(ctx)
        namesLyt.setPadding(Theme.dp(10f), 0, Theme.dp(10f), 0)
        namesLyt.orientation = VERTICAL
        addView(namesLyt, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))
        this.nameTv = TextView(ctx)
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTypeface(Theme.font, Typeface.BOLD)
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 17f)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        this.nameTv.marqueeRepeatLimit = 5
        this.nameTv.isSelected = true
        namesLyt.addView(this.nameTv, ViewUtils.linearMatchWrap())
        this.descTv = TextView(ctx)
        this.descTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.descTv.typeface = Theme.font
        this.descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        namesLyt.addView(this.descTv, ViewUtils.linearMatchWrap())

        val costsLyt = LinearLayout(ctx)
        costsLyt.orientation = VERTICAL
        addView(costsLyt, ViewUtils.linearWrapWrap())
        this.costTv = TextView(ctx)
        this.costTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.costTv.typeface = Theme.font
        this.costTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 17f)
        this.costTv.isSingleLine = true
        costsLyt.addView(this.costTv, ViewUtils.linearWrapWrap())
        this.costTimeTv = TextView(ctx)
        this.costTimeTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.costTimeTv.typeface = Theme.font
        this.costTimeTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        this.costTimeTv.isSingleLine = true
        this.costTimeTv.gravity = Gravity.END
        this.costTimeTv.ellipsize = TextUtils.TruncateAt.END
        costsLyt.addView(this.costTimeTv, ViewUtils.linearMatchWrap())
    }

    fun showData(plan: PlanObject) {
        plan.planDesc?.logo?.let {
            loadImage(it, this.image)
        } ?: run {
            this.image.setImageResource(R.drawable.ic_default_plan)
        }
        this.nameTv.text = plan.planDesc?.title ?: plan.planInfo.name
        this.descTv.text = plan.planDesc?.shortText
    }

    fun showData(planInfo: PlanInfo, showCost: Boolean = false) {
        planInfo.description?.logo?.let {
            loadImage(it, this.image, R.drawable.ic_default_plan)
        } ?: run {
            this.image.setImageResource(R.drawable.ic_default_plan)
        }
        this.nameTv.text = planInfo.description?.title ?: planInfo.name
        this.descTv.text = planInfo.description?.shortText

        if (showCost) {
            val price = planInfo.planPrice
            this.costTv.text = if (price > 0) {
                this.costTimeTv.text = context.getString(R.string.month_little)
                val currency = planInfo.services.firstOrNull()?.currency ?: Currency.USD
                "${context.getString(R.string.from)} ${price.formattedAmount(currency)}"
            } else {
                ""
            }
        }
    }
}