package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Color
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.cardview.widget.CardView
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.models.PlanObject
import nexus.billing.models.planDesc
import nexus.billing.ui.loadImage
import nexus.core.extentions.toDisplayNumber
import nexus.core.util.formattedAmount
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

class RowProfilePlan(val ctx: Context): LinearLayout(ctx) {

    val image: ImageView
    val stateTv: TextView
    val nameTv: TextView
    val descTv: TextView
    val dateTv: TextView
    val costTv: TextView
    private val contentLyt: LinearLayout

    init {
        this.orientation = VERTICAL
        this.layoutParams = ViewUtils.linearMatchWrap()

        this.stateTv = TextView(context)
        this.addView(this.stateTv, ViewUtils.linearMatchWrap())
        this.stateTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.stateTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.stateTv.visibility = View.GONE
        this.stateTv.isAllCaps = true
        this.stateTv.setPadding(Theme.dp(25f), Theme.dp(15f), 0, 0)

        val card = CardView(this.ctx)
        this.addView(card, ViewUtils.linearMatchWrap().apply { setMargins(Theme.dp(20f), Theme.dp(3f), Theme.dp(20f), Theme.dp(10f))})
        card.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), Theme.dp(10f))
        card.cardElevation = Theme.dp(5f).toFloat()
        card.radius = Theme.dp(15f).toFloat()

        this.contentLyt = LinearLayout(ctx)
        card.addView(this.contentLyt, ViewUtils.cardMatchWrap())
        this.contentLyt.orientation = HORIZONTAL
        this.contentLyt.gravity = Gravity.CENTER_VERTICAL
        this.contentLyt.setBackgroundResource(ViewUtils.selectableResId(ctx))
        this.contentLyt.isClickable = true
        this.contentLyt.isFocusable = true
        this.contentLyt.setPadding(Theme.dp(15f), Theme.dp(10f), Theme.dp(10f), Theme.dp(10f))

        this.image = ImageView(ctx)
        this.contentLyt.addView(this.image, ViewUtils.linearParams(65, 65))

        val namesLyt = LinearLayout(ctx)
        namesLyt.setPadding(Theme.dp(10f), 0, Theme.dp(10f), 0)
        namesLyt.orientation = VERTICAL
        this.contentLyt.addView(namesLyt, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))

        this.nameTv = TextView(ctx)
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTypeface(Theme.font, Typeface.BOLD)
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        this.nameTv.marqueeRepeatLimit = 5
        this.nameTv.isSelected = true
        namesLyt.addView(this.nameTv, ViewUtils.linearMatchWrap())

        this.descTv = TextView(ctx)
        this.descTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.descTv.typeface = Theme.font
        this.descTv.isSingleLine = true
        this.descTv.ellipsize = TextUtils.TruncateAt.END
        this.descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        namesLyt.addView(this.descTv, ViewUtils.linearMatchWrap())

        this.dateTv = TextView(ctx)
        this.dateTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.dateTv.typeface = Theme.font
        this.dateTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.dateTv.isSingleLine = true
        this.dateTv.visibility = View.GONE
        this.dateTv.ellipsize = TextUtils.TruncateAt.END
        namesLyt.addView(this.dateTv, ViewUtils.linearMatchWrap())

        this.costTv = TextView(ctx)
        this.contentLyt.addView(this.costTv, ViewUtils.linearWrapWrap())
        this.costTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.costTv.setTypeface(Theme.font, Typeface.BOLD)
        this.costTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.costTv.isSingleLine = true

        val arr = ImageView(ctx)
        this.contentLyt.addView(arr, ViewUtils.linearParams(25, 25))
        arr.setImageResource(nexus.appui.R.drawable.ic_arrow_right)
        arr.setColorFilter(Theme.getColor(Theme.key_greyColor))
    }

    override fun setOnClickListener(l: OnClickListener?) {
        this.contentLyt.setOnClickListener(l)
    }

    fun showData(planObject: PlanObject) {
        planObject.planDesc?.logo?.let {
            loadImage(it, this.image, R.drawable.ic_default_plan)
        } ?: run {
            this.image.setImageResource(R.drawable.ic_default_plan)
        }
        this.nameTv.text = planObject.planDesc?.title ?: planObject.planInfo.name
        var color = Theme.getColor(Theme.key_greyColor)
        this.descTv.text = when(planObject.plan.state) {
            ProfilePlan.State.active -> {
                planObject.plan.nextActivationAt?.let {
                    this.dateTv.text = "${context.getString(R.string.renews)} ${SimpleDateFormat("MMMM dd, yyyy").format(Date(it.toLong()))}"
                    this.dateTv.visibility = VISIBLE
                }
                planObject.plan.services.find { it.asset != null }?.asset?.let {
                    it.number.toDisplayNumber()
                } ?: run {
                    planObject.planDesc?.shortText ?: ""
                }
            }
            else -> {
                color = Color.RED
                var actionText = ""
                for (action in planObject.plan.actions) {
                    when {
                        action.identity?.info != null -> {
                            if (action.identity.info.state == ProfilePlan.Action.State.pending) {
                                val verification = action.identity.info.required.firstOrNull()
                                actionText += verification?.let {
                                    "${context.getString(R.string.verification)} ${Locale(it.country, it.country).displayCountry}"
                                } ?: run {
                                    context.getString(R.string.verification)
                                }
                                actionText += "\n"
                            }
                        }
                        action.asset?.info != null -> {
                            if (action.asset.info.state == ProfilePlan.Action.State.pending) {
                                actionText += "${context.getString(R.string.select_did_number)}\n"
                            }
                        }
                        action.support?.info != null -> {
                            if (action.support.info.state == ProfilePlan.Action.State.pending) {
                                actionText += "${context.getString(R.string.talk_to_support)}\n"
                            }
                        }
                        action.balance?.info != null -> {
                            if (action.balance.info.state == ProfilePlan.Action.State.pending) {
                                actionText += context.getString(R.string.action_balance_desc)
                            }
                        }
                    }
                }
                this.descTv.isSingleLine = false
                if (actionText.isNotEmpty()) {
                    actionText.substring(0, actionText.length - 1)
                } else {
                    context.getString(R.string.action_required)
                }
            }
        }
        this.descTv.setTextColor(color)
        val sum = planObject.price
        this.costTv.visibility = if (sum > 0) View.VISIBLE else View.GONE
        this.costTv.text = sum.formattedAmount(planObject.currency)
    }
}