package nexus.billing.ui.rows

import android.content.Context
import android.graphics.PorterDuff
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.common.Provider
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.loadImage

class RowProvider(val ctx: Context) : LinearLayout(ctx) {

    var img: ImageView
    var nameTv: TextView
    var descTv: TextView

    init {
        val typeface = Theme.font
        this.layoutParams = ViewUtils.linearMatchWrap()
        this.gravity = Gravity.CENTER_VERTICAL
        this.setPadding(dp(10f))
        this.setBackgroundResource(ViewUtils.selectableResId(ctx))
        this.isClickable = true
        this.img = ImageView(ctx)
        addView(this.img, ViewUtils.linearParams(50, 50))
        val tvsLyt = LinearLayout(ctx)
        tvsLyt.setPadding(dp(10f), 0, dp(5f), 0)
        addView(tvsLyt, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))
        tvsLyt.orientation = VERTICAL
        this.nameTv = TextView(ctx)
        tvsLyt.addView(this.nameTv, ViewUtils.linearMatchWrap())
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.nameTv.setTypeface(typeface, Typeface.BOLD)
        this.descTv = TextView(ctx)
        tvsLyt.addView(this.descTv, ViewUtils.linearMatchWrap())
        this.descTv.setTextColor(Theme.getColor(Theme.key_textSecondary))
        this.descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.descTv.isSingleLine = true
        this.descTv.ellipsize = TextUtils.TruncateAt.END
        this.descTv.typeface = typeface
        val arrow = ImageView(ctx)
        addView(arrow, ViewUtils.linearParams(20, 20))
        arrow.setImageResource(R.drawable.ic_arrow_right)
        arrow.setColorFilter(Theme.getColor(Theme.key_greyColor), PorterDuff.Mode.SRC_IN)
    }

    fun showProvider(provider: Pair<Provider, DescriptionObject?>) {
        provider.second?.logo?.let {
            loadImage(it, this.img)
        } ?: run {
            this.img.setImageResource(R.drawable.ic_default_provider)
        }
        this.nameTv.text = provider.second?.title ?: provider.second?.name ?: provider.first.name
        this.descTv.text = provider.second?.shortText
    }
}