package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Color
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.core.view.setPadding
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.service.Service
import nexus.billing.models.PlanInfo

class RowPublicPlan(var ctx: Context): CardView(ctx) {

    val planInfo: RowPlanTopInfo
    val optionsLyt: LinearLayout
    val identLyt: LinearLayout
    private val contentLyt: LinearLayout

    init {
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        this.cardElevation = dp(5f).toFloat()
        this.radius = dp(15f).toFloat()

        this.contentLyt = LinearLayout(ctx)
        this.addView(this.contentLyt, ViewUtils.cardMatchWrap())
        this.contentLyt.orientation = LinearLayout.VERTICAL
        this.contentLyt.gravity = Gravity.CENTER_VERTICAL
        this.contentLyt.setBackgroundResource(ViewUtils.selectableResId(ctx))
        this.contentLyt.isClickable = true
        this.contentLyt.setPadding(dp(10f))
        this.contentLyt.isFocusable = true
        
        this.planInfo = RowPlanTopInfo(ctx)
        val iLp = ViewUtils.linearMatchWrap()
        iLp.bottomMargin = dp(10f)
        this.contentLyt.addView(planInfo, iLp)

        this.identLyt = LinearLayout(ctx)
        this.identLyt.gravity = Gravity.CENTER_VERTICAL
        this.identLyt.orientation = LinearLayout.HORIZONTAL
        this.identLyt.setPadding(0, 0, 0, dp(5f))
        this.contentLyt.addView(this.identLyt, ViewUtils.linearMatchWrap())
        val identTv = TextView(context)
        this.identLyt.addView(identTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))
        identTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        identTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        identTv.text = context.getString(R.string.require_ident)
        val identImg = ImageView(context)
        identImg.setImageResource(R.drawable.ic_error_outline)
        identImg.setColorFilter(Color.RED)
        this.identLyt.addView(identImg, ViewUtils.linearParams(15, 15))
        
        val line = View(context)
        line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        this.contentLyt.addView(line, ViewUtils.linearParams(LayoutParams.MATCH_PARENT, 1, 0f))
        
        this.optionsLyt = LinearLayout(ctx)
        this.optionsLyt.orientation = LinearLayout.VERTICAL
        this.optionsLyt.setPadding(0, dp(10f), 0, 0)
        this.contentLyt.addView(this.optionsLyt, ViewUtils.linearMatchWrap())
    }

    private fun showServices(plan: PlanInfo) {
        for (service in plan.services.filter { it.activation != Service.Activation.optional }.sortedByDescending { it.type }) {
            val value = if (service.type == Service.Types.`package` && (service.units ?: 0) > 0) {
                when(service.category) {
                    Service.Category.voice -> {
                        "${service.units ?: 0} ${context.getString(R.string.min)}"
                    }
                    Service.Category.sms -> {
                        "${service.units ?: 0} ${context.getString(R.string.sms)}"
                    }
                    Service.Category.data -> {
                        "${service.units ?: 0} ${context.getString(R.string.mb)}"
                    }
                    else -> ""
                }
            } else {
                ""
            }

            val serviceItem = RowPlanOption(context)
            this.optionsLyt.addView(serviceItem, ViewUtils.linearMatchWrap())
            serviceItem.showData(plan.serviceDesc(service.id), value)
        }
    }

    fun showData(plan: PlanInfo) {
        this.planInfo.showData(plan, true)
        this.identLyt.visibility = if (plan.verificationRequired.isEmpty()) {
            View.GONE
        } else {
            View.VISIBLE
        }
        this.showServices(plan)
    }

    override fun setOnClickListener(l: OnClickListener?) {
        this.contentLyt.setOnClickListener(l)
    }
}