package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Color
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import nexus.billing.R
import nexus.billing.methods.common.Route
import nexus.core.util.Currency
import nexus.core.util.formattedAmount
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.Country
import nexus.appui.utils.ViewUtils

class RowRoute(ctx: Context) : LinearLayout(ctx) {

    var nameTv: TextView
    var priceTv: TextView
    var flagTv: TextView

    init {
        this.layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT)
        this.orientation = HORIZONTAL
        this.gravity = Gravity.CENTER_VERTICAL
        this.setPadding(dp(10f), dp(5f), dp(10f), dp(5f))

        this.flagTv = TextView(ctx)
        this.flagTv.visibility = GONE
        this.flagTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)
        this.flagTv.gravity = Gravity.CENTER
        this.flagTv.setTextColor(Color.BLACK)
        this.flagTv.setPadding(0, 0, dp(10f), 0)
        addView(this.flagTv, ViewUtils.linearWrapWrap())
        this.nameTv = TextView(context)
        addView(this.nameTv, LayoutParams(0, LayoutParams.WRAP_CONTENT, 1f))
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 17f)
        this.nameTv.setPadding(0, 0, dp(5f), 0)
        this.nameTv.gravity = Gravity.START
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        this.nameTv.typeface = Theme.font
        this.priceTv = TextView(context)
        addView(this.priceTv, ViewUtils.linearWrapWrap())
        this.priceTv.setTextColor(resources.getColor(R.color.green))
        this.priceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        this.priceTv.gravity = Gravity.START
        this.priceTv.isSingleLine = true
        this.priceTv.ellipsize = TextUtils.TruncateAt.END
        this.priceTv.typeface = Theme.font
    }

    fun showData(route: Route, showFlag: Boolean = false, country: Country? = null) {
        this.nameTv.text = "${route.name} ${ if (route.prefix != null) "(+${route.prefix})" else ""}"
        this.priceTv.text = when(route.amount) {
            null -> {
                this.priceTv.visibility = GONE
                ""
            }
            0 -> this.context.getString(R.string.free)
            else -> route.amount?.formattedAmount(route.currency ?: Currency.USD)
        }
        this.flagTv.visibility = if (showFlag) View.VISIBLE else View.GONE
        country?.let { this.flagTv.text = it.flag }
    }
}