package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import androidx.cardview.widget.CardView
import androidx.core.content.ContextCompat
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.methods.requests.profileplan.ProfilePlanRoute
import nexus.billing.methods.service.Service
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.billing.models.planDesc
import nexus.billing.ui.getProgressBg
import nexus.billing.ui.loadImage
import nexus.core.extentions.toDisplayNumber
import nexus.core.util.Currency
import nexus.core.util.formattedAmount
import java.util.Locale

class RowSelectPlanView(val ctx: Context): CardView(ctx) {

    var image: ImageView
    var arr: ImageView
    var nameTv: TextView
    var numberTv: TextView
    var allTv: TextView
    var leftTv: TextView
    var costTv: TextView
    var unitTv: TextView
    val progress: ProgressBar
    val packageLyt: LinearLayout
    val contentLyt: LinearLayout

    init {
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), dp(10f))
        this.cardElevation = dp(3f).toFloat()
        this.radius = dp(15f).toFloat()

        this.contentLyt = LinearLayout(ctx)
        this.addView(this.contentLyt, ViewUtils.cardMatchWrap())
        this.contentLyt.orientation = LinearLayout.HORIZONTAL
        this.contentLyt.gravity = Gravity.CENTER_VERTICAL
        this.contentLyt.setBackgroundResource(ViewUtils.selectableResId(ctx))
        this.contentLyt.isClickable = true
        this.contentLyt.isFocusable = true
        this.contentLyt.setPadding(dp(10f), dp(7f), dp(10f), dp(7f))

        this.image = ImageView(this.ctx)
        this.contentLyt.addView(this.image, ViewUtils.linearParams(52, 52))

        val nameLyt = LinearLayout(this.ctx)
        nameLyt.orientation = LinearLayout.VERTICAL
        nameLyt.setPadding(dp(10f), 0, dp(10f), 0)
        this.contentLyt.addView(nameLyt, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))

        this.nameTv = TextView(this.ctx)
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 15f)
        this.nameTv.setTypeface(Theme.font, Typeface.BOLD)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.END
        nameLyt.addView(this.nameTv, ViewUtils.linearMatchWrap())

        this.numberTv = TextView(this.ctx)
        this.numberTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.numberTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.numberTv.typeface = Theme.font
        this.numberTv.isSingleLine = true
        this.numberTv.ellipsize = TextUtils.TruncateAt.END
        nameLyt.addView(this.numberTv, ViewUtils.linearMatchWrap())

        this.packageLyt = LinearLayout(this.ctx)
        this.packageLyt.orientation = LinearLayout.HORIZONTAL
        this.packageLyt.visibility = GONE
        this.packageLyt.setPadding(0, dp(2f), 0, dp(2f))
        nameLyt.addView(this.packageLyt, ViewUtils.linearMatchWrap())

        this.leftTv = TextView(this.ctx)
        this.leftTv.typeface = Theme.font
        this.leftTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.leftTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 12f)
        this.packageLyt.addView(this.leftTv, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))

        this.allTv = TextView(this.ctx)
        this.allTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.allTv.gravity = Gravity.END
        this.allTv.typeface = Theme.font
        this.allTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 12f)
        this.packageLyt.addView(this.allTv, ViewUtils.linearParams(0, LayoutParams.WRAP_CONTENT, 1f))

        this.progress = ProgressBar(this.ctx, null, android.R.attr.progressBarStyleHorizontal)
        this.progress.isIndeterminate = false
        this.progress.visibility = GONE
        this.progress.progressDrawable = getProgressBg(Theme.addTrans(Theme.getColor(Theme.key_primaryColor), 50), Theme.getColor(Theme.key_primaryColor))
        nameLyt.addView(this.progress, ViewUtils.linearParams(LayoutParams.MATCH_PARENT, dp(6f), 0f))

        val costLyt = LinearLayout(this.ctx)
        costLyt.orientation = LinearLayout.VERTICAL
        costLyt.gravity = Gravity.CENTER
        this.contentLyt.addView(costLyt, ViewUtils.linearWrapWrap())

        this.costTv = TextView(this.ctx)
        this.costTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
        this.costTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.costTv.setTypeface(Theme.font, Typeface.BOLD)
        this.costTv.gravity = Gravity.CENTER
        costLyt.addView(this.costTv, ViewUtils.linearWrapWrap())

        this.unitTv = TextView(this.ctx)
        this.unitTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.unitTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.unitTv.setTypeface(Theme.font, Typeface.BOLD)
        this.unitTv.gravity = Gravity.CENTER
        costLyt.addView(this.unitTv, ViewUtils.linearWrapWrap())

        this.arr = ImageView(this.ctx)
        this.contentLyt.addView(this.arr, ViewUtils.linearParams(15, 15))
        this.arr.setImageResource(R.drawable.ic_arrow_right)
        this.arr.visibility = GONE
        this.arr.setColorFilter(Theme.getColor(Theme.key_primaryColor))
    }

    fun showData(planObject: PlanObject, type: ProfilePlanRoute.RouteType, route: ProfilePlanRoute.Response.Routes? = null, showActions: Boolean = false) {
        planObject.planDesc?.logo?.let {
            loadImage(it, this.image)
        } ?: run {
            this.image.setImageResource(R.drawable.ic_default_plan)
        }
        this.nameTv.text = planObject.planDesc?.title ?: planObject.planInfo.name
        val unitsValue = if (type == ProfilePlanRoute.RouteType.voice) {
            ctx.getString(R.string.min)
        } else {
            ctx.getString(R.string.sms)
        }
        route?.route?.let {
            this.costTv.text = it.rate?.formattedAmount(it.currency ?: Currency.USD) ?: ""
            this.unitTv.text = if (it.rate != null) unitsValue else ""
            this.packageLyt.visibility = View.GONE
            this.progress.visibility = View.GONE
        }
        var available: Int? = null
        var total: Int? = null
        route?.`package`?.let {
            total = it.total
            available = it.available
        } ?: run {
            planObject.plan.services.find { it.type == Service.Types.`package` }?.units?.let {
                total = it.total
                available = it.available
            }
        }
        total?.let { total ->
            this.costTv.text = ""
            this.unitTv.text = ""
            this.allTv.text = "${ctx.getString(R.string.all_units)}: $total $unitsValue"
            this.leftTv.text = "${ctx.getString(R.string.left)}: $available $unitsValue"
            this.progress.max = total
            this.progress.progress = available ?: 0
            this.progress.visibility = View.VISIBLE
            this.packageLyt.visibility = View.VISIBLE
        }
        if (showActions || planObject.plan.state == ProfilePlan.State.pending) {
            var actionText = ""
            for (action in planObject.plan.actions) {
                when {
                    action.identity?.info != null -> {
                        if (action.identity.info.state == ProfilePlan.Action.State.pending) {
                            val verification = action.identity.info.required.firstOrNull()
                            actionText += verification?.let {
                                "${context.getString(R.string.verification)} ${Locale(it.country, it.country).displayCountry}"
                            } ?: run {
                                context.getString(R.string.verification)
                            }
                            actionText += "\n"
                        }
                    }
                    action.asset?.info != null -> {
                        if (action.asset.info.state == ProfilePlan.Action.State.pending) {
                            actionText += "${context.getString(R.string.select_did_number)}\n"
                        }
                    }
                    action.support?.info != null -> {
                        if (action.support.info.state == ProfilePlan.Action.State.pending) {
                            actionText += "${context.getString(R.string.talk_to_support)}\n"
                        }
                    }
                    action.balance?.info != null -> {
                        if (action.balance.info.state == ProfilePlan.Action.State.pending) {
                            actionText += context.getString(R.string.action_balance_desc)
                        }
                    }
                }
            }
            this.numberTv.isSingleLine = false
            this.numberTv.text = if (actionText.isNotEmpty()) {
                "${context.getString(R.string.to_activate_plan_need)}:\n${actionText.substring(0, actionText.length - 1)}"
            } else {
                actionText
            }
            this.numberTv.setTextColor(ContextCompat.getColor(this.context, R.color.orange_color))
        } else {
            this.numberTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
            planObject.plan.services.find { it.asset != null }?.let {
                this.numberTv.text = it.asset?.number?.toDisplayNumber()
                this.numberTv.visibility = VISIBLE
            } ?: run {
                this.numberTv.visibility = GONE
            }
        }
    }

    fun showData(planInfo: PlanInfo) {
        planInfo.description?.logo?.let {
            loadImage(it, this.image)
        } ?: run {
            this.image.setImageResource(R.drawable.ic_default_plan)
        }
        this.numberTv.text = planInfo.description?.shortText
        this.nameTv.text = planInfo.description?.title ?: planInfo.name
        this.arr.visibility = VISIBLE
    }

    fun setActive(active: Boolean) {
        this.background = ViewUtils.getRect(Theme.getColor(if (active) Theme.key_blockView else Theme.key_mainBackground), dp(10f))
        this.cardElevation = if (active) dp(3f).toFloat() else 0f
    }

    override fun setOnClickListener(l: OnClickListener?) {
        this.contentLyt.setOnClickListener(l)
    }
}