package nexus.billing.ui.rows

import android.content.Context
import android.graphics.Typeface
import android.text.TextUtils
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import androidx.cardview.widget.CardView
import nexus.appui.Theme
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.common.Descriptions
import nexus.billing.methods.common.Route
import nexus.billing.ui.loadImage
import nexus.core.util.Currency

class RowServiceRoute(val ctx: Context): CardView(ctx) {

    val icon: ImageView
    val nameTv: TextView
    val rateTv: TextView
    private val routesLyt: LinearLayout
    val progress: ProgressBar

    init {
        this.background = ViewUtils.getRect(Theme.getColor(Theme.key_blockView), Theme.dp(10f))
        this.cardElevation = Theme.dp(3f).toFloat()
        this.radius = Theme.dp(15f).toFloat()

        val contentLyt = LinearLayout(ctx)
        this.addView(contentLyt, ViewUtils.cardMatchWrap())
        contentLyt.orientation = LinearLayout.VERTICAL
        contentLyt.gravity = Gravity.CENTER_VERTICAL
        contentLyt.weightSum = 1f
        contentLyt.setPadding(Theme.dp(10f), Theme.dp(5f), Theme.dp(10f), Theme.dp(5f))

        val titleLyt = LinearLayout(ctx)
        contentLyt.addView(titleLyt, ViewUtils.linearMatchWrap())
        titleLyt.setPadding(0, Theme.dp(5f), 0, Theme.dp(5f))
        titleLyt.gravity = Gravity.CENTER_VERTICAL
        titleLyt.orientation = LinearLayout.HORIZONTAL
        this.icon = ImageView(ctx)
        this.icon.setColorFilter(Theme.getColor(Theme.key_rowTextBlack))
        titleLyt.addView(this.icon, ViewUtils.linearParams(30, 30))

        this.nameTv = TextView(ctx)
        this.nameTv.setPadding(Theme.dp(5f), 0, 0, 0)
        this.nameTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        this.nameTv.setTypeface(Theme.font, Typeface.BOLD)
        this.nameTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        this.nameTv.isSingleLine = true
        this.nameTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        this.nameTv.marqueeRepeatLimit = 5
        this.nameTv.isSelected = true
        titleLyt.addView(this.nameTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))

        val descLyt = LinearLayout(ctx)
        contentLyt.addView(descLyt, ViewUtils.linearMatchWrap())
        descLyt.gravity = Gravity.CENTER_VERTICAL
        descLyt.setPadding(0, 0, 0, Theme.dp(5f))
        descLyt.orientation = LinearLayout.HORIZONTAL

        val countryTv = TextView(ctx)
        countryTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        countryTv.setTypeface(Theme.font, Typeface.BOLD)
        countryTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        countryTv.isSingleLine = true
        countryTv.text = context.getString(R.string.country_region)
        countryTv.ellipsize = TextUtils.TruncateAt.MARQUEE
        countryTv.marqueeRepeatLimit = 5
        countryTv.isSelected = true
        descLyt.addView(countryTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))

        this.rateTv = TextView(ctx)
        this.rateTv.setPadding(Theme.dp(5f), 0, 0, 0)
        this.rateTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        this.rateTv.typeface = Theme.font
        this.rateTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13f)
        this.rateTv.isSingleLine = true
        this.rateTv.visibility = GONE
        this.rateTv.text = context.getString(R.string.rate_per_min)
        descLyt.addView(this.rateTv, ViewUtils.linearWrapWrap())

        val line = View(context)
        line.setBackgroundColor(Theme.getColor(Theme.key_dividerGrey))
        contentLyt.addView(line, ViewUtils.linearParams(LayoutParams.MATCH_PARENT, 1, 0f))

        this.routesLyt = LinearLayout(ctx)
        contentLyt.addView(this.routesLyt, ViewUtils.linearMatchWrap())
        this.routesLyt.orientation = LinearLayout.VERTICAL

        this.progress = ProgressBar(context)
        val pLp = ViewUtils.linearWrapWrap()
        pLp.gravity = Gravity.CENTER_HORIZONTAL
        contentLyt.addView(this.progress, pLp)
    }

    fun showData(description: Descriptions, currency: Currency) {
        this.nameTv.text = description.title ?: context.getText(R.string.service)
        this.rateTv.text = "${this.ctx.getString(R.string.rate_per_min)} ($currency)"
        description.icon?.let { loadImage(it, this.icon) } ?: run { this.icon.setImageResource(R.drawable.ic_plan_service) }
    }

    fun showRoutes(routes: Array<Route>, listener: OnClickListener) {
        this.routesLyt.removeAllViews()
        this.progress.visibility = GONE
        this.rateTv.visibility = if (routes.count { it.amount != null } > 0) VISIBLE else GONE
        for ((i, route) in routes.withIndex()) {
            val item = RowRoute(context)
            item.setPadding(0, Theme.dp(5f), 0, Theme.dp(5f))
            this.routesLyt.addView(item, ViewUtils.linearMatchWrap())
            item.showData(route)

            if (i == 4) {
                val moreBtn = RowRoute(context)
                moreBtn.setPadding(0, Theme.dp(5f), 0, Theme.dp(5f))
                this.routesLyt.addView(moreBtn, ViewUtils.linearMatchWrap())
                moreBtn.nameTv.text = context.getString(R.string.shw_more)
                moreBtn.nameTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
                moreBtn.nameTv.gravity = Gravity.CENTER
                moreBtn.nameTv.setBackgroundResource(ViewUtils.selectableResId(context))
                moreBtn.setOnClickListener(listener)
                break
            }
        }
    }
}
